@@ -8,10 +8,7 @@
  */
 
 #include <h/mh.h>
-
-#ifdef MHRC
-# include <pwd.h>
-#endif
+#include <pwd.h>
 
 #define nmhbindir(file) NMHBINDIR#file
 #define nmhetcdir(file) NMHETCDIR#file
@@ -34,21 +31,16 @@ etcpath (char *file)
 {
     static char epath[PATH_MAX];
     char *cp;
-#ifdef MHRC
     char *pp;
     struct passwd *pw;
-#endif
 
-#ifdef MHRC
     context_read();
-#endif
 
     switch (*file) {
 	case '/': 
 	    /* If already absolute pathname, return it */
 	    return file;
 
-#ifdef MHRC
 	case '~': 
 	    /* Expand ~username */
 	    if ((cp = strchr(pp = file + 1, '/')))
@@ -72,12 +64,16 @@ etcpath (char *file)
 	    if (access (epath, R_OK) != NOTOK)
 		return epath;	/* else fall */
 try_it:
-#endif /* MHRC */
 
 	default: 
 	    /* Check nmh Mail directory */
-	    if (access ((cp = m_mailpath (file)), R_OK) != NOTOK)
+	    if (access ((cp = m_mailpath (file)), R_OK) != NOTOK) {
+		/* Will leak because caller doesn't know cp was
+		   dynamically allocated. */
 		return cp;
+	    } else {
+		free (cp);
+	    }
     }
 
     /* Check nmh `etc' directory */
@@ -91,9 +87,9 @@ try_it:
  */
 
 struct swit anoyes[] = {
-    { "no", 0 },
-    { "yes", 0 },
-    { NULL, 0 }
+    { "no", 0, 0 },
+    { "yes", 0, 1 },
+    { NULL, 0, 0 }
 };
 
 /* 
@@ -106,6 +102,9 @@ char *mh_defaults = nmhetcdir (/mh.profile);
 /* default name of user profile */
 char *mh_profile = ".mh_profile";
 
+/* name of credentials file, defaults to .netrc in either Path or $HOME. */
+char *credentials_file;
+
 /* name of current message "sequence" */
 char *current = "cur";
 
@@ -141,6 +140,9 @@ char *nmhprivcache = "nmh-private-cache";
 /* profile entry for external ftp access command */
 char *nmhaccessftp = "nmh-access-ftp";
 
+/* profile entry for external url access command */
+char *nmhaccessurl = "nmh-access-url";
+
 char *mhlibdir = NMHLIBDIR;
 char *mhetcdir = NMHETCDIR;
 
@@ -154,15 +156,11 @@ char *mhetcdir = NMHETCDIR;
 char *context = "context";
 
 /*
- * Default name of file for public sequences.  If NULL,
- * then nmh will use private sequences by default, unless the
- * user defines a value using the "mh-sequences" profile entry.
+ * Default name of file for public sequences.  If "\0" (an empty
+ * "mh-sequences" profile entry), then nmh will use private sequences by
+ * default.
  */
-#ifdef NOPUBLICSEQ
-char *mh_seq = NULL;
-#else
 char *mh_seq = ".mh_sequences";
-#endif
 
 /* 
  * nmh globals
@@ -188,12 +186,6 @@ char *buildmimeproc = nmhbindir (/mhbuild);
  */
 char *catproc = "/bin/cat";
 
-/*
- * mhl runs this program as a visual-end.
- */
-
-char *faceproc = NULL;
-
 /*
  * This program is usually called directly by users, but it is
  * also invoked by the post program to process an "Fcc", or by
@@ -202,19 +194,19 @@ char *faceproc = NULL;
 
 char *fileproc = nmhbindir (/refile);
 
+/*
+ * This program is used to optionally format the bodies of messages by
+ * "mhl".
+ */
+
+char *formatproc = NULL;
+
 /* 
  * This program is called to incorporate messages into a folder.
  */
 
 char *incproc = nmhbindir (/inc);
 
-/*
- * When a user runs an nmh program for the first time, this program
- * is called to create his nmh profile, and mail directory.
- */
-
-char *installproc = nmhlibdir (/install-mh);
-
 /*
  * This is the default program invoked by a "list" response
  * at the "What now?" prompt.  It is also used by the draft
@@ -222,7 +214,7 @@ char *installproc = nmhlibdir (/install-mh);
  * draft message.
  */
 
-char *lproc = DEFAULT_PAGER;
+char *lproc = NULL;
 
 /*
  * This is the path for the Bell equivalent mail program.
@@ -236,7 +228,7 @@ char *mailproc = nmhbindir (/mhmail);
  * or message parts of type text/plain.
  */
 
-char *moreproc = DEFAULT_PAGER;
+char *moreproc = NULL;
 
 /* 
  * This is the program (mhl) used to filter messages.  It is
@@ -276,12 +268,6 @@ char *postproc = nmhlibdir (/post);
 
 char *rcvstoreproc = nmhlibdir (/rcvstore);
 
-/* 
- * This program is called to remove a folder.  
- */
-
-char *rmfproc = nmhbindir (/rmf);
-
 /* 
  * This program is called to remove a message by rmm or refile -nolink.
  * It's usually empty, which means to rename the file to a backup name.
@@ -331,14 +317,6 @@ char *whatnowproc = nmhbindir (/whatnow);
 
 char *whomproc = nmhbindir (/whom);
 
-/*
- * This is the editor invoked by the various message
- * composition programs.  It SHOULD be a full screen
- * editor, such as vi or emacs, but any editor will work.
- */
-
-char *defaulteditor = DEFAULT_EDITOR;
-
 /* 
  * This is the global nmh alias file.  It is somewhat obsolete, since
  * global aliases should be handled by the Mail Transport Agent (MTA).
@@ -354,7 +332,7 @@ char *AliasFile = nmhetcdir (/MailAliases);
  * Folders (directories) are created with this protection (mode)
  */
 
-char *foldprot = DEFAULT_FOLDER_MODE;
+char *foldprot = "700";
 
 /*
  * Every NEW message will be created with this protection.  When a
@@ -362,5 +340,5 @@ char *foldprot = DEFAULT_FOLDER_MODE;
  * to messages coming in through inc.
  */
 
-char *msgprot = DEFAULT_MESSAGE_MODE;
+char *msgprot = "600";
 
