@@ -13,6 +13,12 @@
 
 #include <h/nmh.h>
 
+/*
+ * We need this for uint32_t
+ */
+
+#include <inttypes.h>
+
 /*
  * Use prototypes for nmh/mh
  */
@@ -35,7 +41,7 @@ typedef unsigned char *POINTER;
 typedef unsigned short int UINT2;
 
 /* UINT4 defines a four byte word */
-typedef unsigned long int UINT4;
+typedef uint32_t UINT4;
 
 /* PROTO_LIST is defined depending on how PROTOTYPES is defined above.
 If using PROTOTYPES, then PROTO_LIST returns the list, otherwise it
