@@ -1,8 +1,6 @@
 
 /*
  * mime.h -- definitions for MIME
- *
- * $Id: mime.h,v 1.2 2006/02/18 16:26:37 levine Exp $
  */
 
 #define	VRSN_FIELD	"MIME-Version"
@@ -14,8 +12,11 @@
 #define	DESCR_FIELD	"Content-Description"
 #define	DISPO_FIELD	"Content-Disposition"
 #define	MD5_FIELD	"Content-MD5"
+#define	ATTACH_FIELD	"Attach"
 
-#define	isatom(c)   (!isspace (c) && !iscntrl (c) && (c) != '(' \
+#define	isatom(c)   (isascii((unsigned char) c) \
+                     && !isspace ((unsigned char) c) \
+		     && !iscntrl ((unsigned char) c) && (c) != '(' \
 	             && (c) != ')' && (c) != '<'  && (c) != '>' \
 	             && (c) != '@' && (c) != ','  && (c) != ';' \
 	             && (c) != ':' && (c) != '\\' && (c) != '"' \
@@ -25,13 +26,31 @@
  * Test for valid characters used in "token"
  * as defined in RFC2045
  */
-#define	istoken(c)  (!isspace (c) && !iscntrl (c) && (c) != '(' \
+#define	istoken(c)  (isascii((unsigned char) c) \
+                     && !isspace ((unsigned char) c) \
+		     && !iscntrl ((unsigned char) c) && (c) != '(' \
 	             && (c) != ')' && (c) != '<'  && (c) != '>' \
 	             && (c) != '@' && (c) != ','  && (c) != ';' \
 	             && (c) != ':' && (c) != '\\' && (c) != '"' \
 	             && (c) != '/' && (c) != '['  && (c) != ']' \
 	             && (c) != '?' && (c) != '=')
 
+/*
+ * Definitions for RFC 2231 encoding
+ */
+#define istspecial(c)  ((c) == '(' || (c) == ')' || (c) == '<' || (c) == '>' \
+		        || (c) == '@' || (c) == ',' || (c) == ';' \
+			|| (c) == ':' || (c) == '\\' || (c) == '"' \
+			|| (c) == '/' || (c) == '[' || (c) == ']' \
+			|| (c) == '?' || (c) == '=')
+
+#define isparamencode(c)  (!isascii((unsigned char) c) || \
+			   iscntrl((unsigned char) c) || istspecial(c) || \
+			   (c) == ' ' || (c) == '*' || (c) == '\'' || \
+			   (c) == '%')
+
+#define	MAXTEXTPERLN 78
+#define	MAXLONGLINE 998
 #define	CPERLIN	76
 #define	BPERLIN	(CPERLIN / 4)
 #define	LPERMSG	632
