@@ -1,8 +1,6 @@
 
 /*
  * msh.h -- definitions for msh
- *
- * $Id: msh.h,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
  */
 
 /* flags for stream */
@@ -24,7 +22,13 @@ struct Cmd {
 #define	MHNCHK 0x0001	/* did nontext check           */
 #define	MHNYES 0x0002	/* .. and known to be non-text */
 
-#define CUR (1 << (FFATTRSLOT + NUMATTRS - 1))
+/*
+ * Determine the number of user defined sequences we
+ * can have.  The first FFATTRSLOT sequence flags are for
+ * internal nmh message flags.
+ */
+#define	NUMATTRS  (27)
+#define CUR (FFATTRSLOT + NUMATTRS - 1)
 
 #ifdef BPOP
 # define VIRTUAL SELECT_EMPTY
@@ -39,7 +43,7 @@ struct Msg {
     char *m_scanl;
     struct tws m_tb;
     short m_flags;
-    seqset_t m_stats;
+    bvector_t m_stats;
 };
 
 #define	m_bboard_id  m_drop.d_id
@@ -55,7 +59,7 @@ extern int modified;		/* command modified folder */
 extern struct msgs *mp;		/* used a lot              */
 extern struct Msg *Msgs;	/* Msgs[0] not used        */
 
-FILE *msh_ready ();
+FILE *msh_ready (int, int);
 
 /*
  * COMMAND
@@ -77,11 +81,6 @@ extern int interrupted;		/* SIGINT detected  */
 extern int broken_pipe;		/* SIGPIPE detected */
 extern int told_to_quit;	/* SIGQUIT detected */
 
-#ifdef BSD42
-extern int should_intr;		/* signal handler should interrupt call */
-extern jmp_buf sigenv;		/* the environment pointer */
-#endif
-
 /*
  * prototypes
  */
