@@ -1,16 +1,11 @@
 
 /*
  * smtp.h -- definitions for the nmh SMTP Interface
- *
- * $Id: smtp.h,v 1.1.1.1 1999/04/30 18:08:35 doug Exp $
  */
 
-/* various modes for SMTP */
-#define	S_MAIL	0
-#define	S_SEND	1
-#define	S_SOML	2
-#define	S_SAML	3
-
+/* length is the length of the string in text[], which is also NUL
+ * terminated, so s.text[s.length] should always be 0.
+ */
 struct smtp {
     int code;
     int length;
@@ -21,8 +16,8 @@ struct smtp {
  * prototypes
  */
 /* int client (); */
-int sm_init (char *, char *, int, int, int, int, int);
-int sm_winit (int, char *);
+int sm_init (char *, char *, char *, int, int, int, int, int, int, char *, char *, int);
+int sm_winit (char *);
 int sm_wadr (char *, char *, char *);
 int sm_waend (void);
 int sm_wtxt (char *, int);
@@ -30,11 +25,6 @@ int sm_wtend (void);
 int sm_end (int);
 char *rp_string (int);
 
-#ifdef MPOP
-int sm_bulk (char *);
-#endif
-
-
 /* The remainder of this file is derived from "mmdf.h" */
 
 /*
