@@ -3,8 +3,6 @@
  * brkstring.c -- (destructively) split a string into
  *             -- an array of substrings
  *
- * $Id: brkstring.c,v 1.4 2006/01/02 03:25:18 bress Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -61,7 +59,7 @@ brkstring (char *str, char *brksep, char *brkterm)
 	if (!c || brkany (c, brkterm)) {
 	    *s = '\0';
 	    broken[i] = NULL;
-	    return broken;
+	    break;
 	}
 
 	/* set next start addr */
@@ -71,7 +69,7 @@ brkstring (char *str, char *brksep, char *brkterm)
 	    ;	/* empty body */
     }
 
-    return broken;	/* NOT REACHED */
+    return broken;
 }
 
 
