@@ -3,8 +3,6 @@
  * concat.c -- concatenate a variable number (minimum of 1)
  *             of strings in managed memory
  *
- * $Id: concat.c,v 1.3 2006/01/02 03:17:42 bress Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -15,7 +13,7 @@
 
 
 char *
-concat (char *s1, ...)
+concat (const char *s1, ...)
 {
     char *cp, *dp, *sp;
     size_t len;
