@@ -2,8 +2,6 @@
 /*
  * context_find.c -- find an entry in the context/profile list
  *
- * $Id: context_find.c,v 1.4 2006/03/08 12:14:16 bress Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -18,7 +16,7 @@ context_find (char *str)
     struct node *np;
 
     for (np = m_defs; np; np = np->n_next)
-	if (!mh_strcasecmp (np->n_name, str))
+	if (!strcasecmp (np->n_name ? np->n_name : "", str ? str : ""))
 	    return (np->n_field);
 
     return NULL;
