@@ -1,8 +1,6 @@
 /*
  * context_read.c -- find and read profile and context files
  *
- * $Id: context_read.c,v 1.10 2007/03/21 00:21:10 pm215 Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -25,7 +23,6 @@
  */
 
 #include <h/mh.h>				/* mh internals */
-#include <errno.h>				/* system call errors */
 #include <pwd.h>				/* structure for getpwuid() results */
 
 void
@@ -37,6 +34,7 @@ context_read (void)
     struct	stat		st;		/* stat() results */
     register	struct	passwd	*pw;		/* getpwuid() results */
     register	FILE		*ib;		/* profile and context file pointer */
+    int failed_to_lock = 0;
 
     /*
      *  If this routine _is_ called again (despite the wanings in the
@@ -138,9 +136,9 @@ context_read (void)
     
     ctxpath = getcpy (m_maildir (cp));
 
-    if ((ib = lkfopen (ctxpath, "r"))) {
+    if ((ib = lkfopendata (ctxpath, "r", &failed_to_lock))) {
 	readconfig ((struct node **) 0, ib, cp, 1);
-	lkfclose (ib, ctxpath);
+	lkfclosedata (ib, ctxpath);
     }
 
     return;
