@@ -2,8 +2,6 @@
 /*
  * context_replace.c -- add/replace an entry in the context/profile list
  *
- * $Id: context_replace.c,v 1.5 2006/03/08 12:14:16 bress Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -38,7 +36,7 @@ context_replace (char *key, char *value)
      * this key, and replace its value if found.
      */
     for (np = m_defs;; np = np->n_next) {
-	if (!mh_strcasecmp (np->n_name, key)) {
+	if (!strcasecmp (np->n_name ? np->n_name : "", key ? key : "")) {
 	    if (strcmp (value, np->n_field)) {
 		if (!np->n_context)
 		    admonish (NULL, "bug: context_replace(key=\"%s\",value=\"%s\")", key, value);
