@@ -2,8 +2,6 @@
 /*
  * done.c -- terminate the program
  *
- * $Id: done.c,v 1.5 2007/11/04 11:54:33 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -11,4 +9,4 @@
 
 #include <h/mh.h>
 
-void (*done) (int) = exit;
+void (*done) (int) NORETURN = exit;
