@@ -2,8 +2,6 @@
 /*
  * fmt_addr.c -- format an address field (from fmt_scan)
  *
- * $Id: fmt_addr.c,v 1.5 2006/01/02 03:25:18 bress Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -84,7 +82,7 @@ formataddr (char *orig, char *str)
 
     /* concatenate all the new addresses onto 'buf' */
     for (isgroup = 0; (cp = getname (str)); ) {
-	if ((mp = getm (cp, NULL, 0, fmt_norm, NULL)) == NULL)
+	if ((mp = getm (cp, NULL, 0, NULL, 0)) == NULL)
 	    continue;
 
 	if (isgroup && (mp->m_gname || !mp->m_ingrp)) {
@@ -114,3 +112,8 @@ formataddr (char *orig, char *str)
     last_dst = dst;
     return (buf);
 }
+
+char *concataddr (char *orig, char *str)
+{
+    return formataddr(orig, str);
+}
