@@ -5,6 +5,9 @@
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
+ *
+ * This is the engine that processes the format instructions created by
+ * fmt_compile (found in fmt_compile.c).
  */
 
 #include <h/mh.h>
@@ -13,31 +16,17 @@
 #include <h/tws.h>
 #include <h/fmt_compile.h>
 
-#ifdef TIME_WITH_SYS_TIME
+#ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
-# include <time.h>
-#else
-# ifdef TM_IN_SYS_TIME
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
 #endif
+#include <time.h>
 #ifdef MULTIBYTE_SUPPORT
 #  include <wctype.h>
 #  include <wchar.h>
 #endif
 
-#define	NFMTS MAXARGS
-
-extern char *formataddr ();	/* hook for custom address formatting */
-
-#ifdef LBL
-struct msgs *fmt_current_folder; /* current folder (set by main program) */
-#endif
-
-extern int fmt_norm;		/* defined in sbr/fmt_def.c = AD_NAME */
-struct mailname fmt_mnull;
+struct mailname fmt_mnull = { NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, 0,
+			      NULL, NULL };
 
 /*
  * static prototypes
@@ -58,32 +47,29 @@ match (char *str, char *sub)
     int c1, c2;
     char *s1, *s2;
 
-#ifdef LOCALE
     while ((c1 = *sub)) {
-	c1 = (isalpha(c1) && isupper(c1)) ? tolower(c1) : c1;
-	while ((c2 = *str++) && c1 != ((isalpha(c2) && isupper(c2)) ? tolower(c2) : c2))
+	c1 = (isascii((unsigned char) c1) && isalpha((unsigned char) c1) &&
+	      isupper((unsigned char) c1)) ? tolower((unsigned char) c1) : c1;
+	while ((c2 = *str++) && c1 != ((isascii((unsigned char) c2) &&
+					isalpha((unsigned char) c2) &&
+					isupper((unsigned char) c2)) ?
+					    tolower((unsigned char) c2) : c2))
 	    ;
 	if (! c2)
 	    return 0;
 	s1 = sub + 1; s2 = str;
-	while ((c1 = *s1++) && ((isalpha(c1) && isupper(c1)) ? tolower(c1) : c1) == ((isalpha(c2 =*s2++) && isupper(c2)) ? tolower(c2) : c2))
+	while ((c1 = *s1++) && ((isascii((unsigned char) c1) &&
+				 isalpha((unsigned char) c1) &&
+				 isupper((unsigned char) c1)) ?
+				 	tolower(c1) : c1) ==
+			((isascii((unsigned char) (c2 =*s2++)) &&
+			  isalpha((unsigned char) c2) &&
+			  isupper((unsigned char) c2)) ?
+			  	tolower((unsigned char) c2) : c2))
 	    ;
 	if (! c1)
 	    return 1;
     }
-#else
-    while ((c1 = *sub)) {
-	while ((c2 = *str++) && (c1 | 040) != (c2 | 040))
-	    ;
-	if (! c2)
-	    return 0;
-	s1 = sub + 1; s2 = str;
-	while ((c1 = *s1++) && (c1 | 040) == (*s2++ | 040))
-	    ;
-	if (! c1)
-	    return 1;
-    }
-#endif
     return 1;
 }
 
@@ -124,17 +110,19 @@ cpnumber(char **dest, int num, unsigned int wid, char fill, size_t n) {
  * no more than n bytes are copied
  */
 static void
-cptrimmed(char **dest, char *str, unsigned int wid, char fill, size_t n) {
+cptrimmed(char **dest, char **ep, char *str, unsigned int wid, char fill,
+	  char *epmax) {
     int remaining;     /* remaining output width available */
-    int c, ljust, w;
+    int c, ljust;
     int end;           /* number of input bytes remaining in str */
 #ifdef MULTIBYTE_SUPPORT
     int char_len;      /* bytes in current character */
+    int w;
     wchar_t wide_char;
+    char *altstr = NULL;
 #endif
     char *sp;          /* current position in source string */
     char *cp = *dest;  /* current position in destination string */
-    char *ep = cp + n; /* end of destination buffer */
     int prevCtrl = 1;
 
     /* get alignment */
@@ -144,12 +132,40 @@ cptrimmed(char **dest, char *str, unsigned int wid, char fill, size_t n) {
 	ljust++;
     }
     if ((sp = (str))) {
+#ifdef MULTIBYTE_SUPPORT
 	mbtowc(NULL, NULL, 0); /* reset shift state */
+#endif
 	end = strlen(str);
 	while (*sp && remaining > 0 && end > 0) {
 #ifdef MULTIBYTE_SUPPORT
 	    char_len = mbtowc(&wide_char, sp, end);
-	    if (char_len <= 0 || (cp + char_len > ep))
+
+	    /*
+	     * See the relevant comments in cpstripped() to explain what's
+	     * going on here; we want to handle the case where we get
+	     * characters that mbtowc() cannot handle
+	     */
+
+	    if (char_len < 0) {
+	    	altstr = "?";
+		char_len = mbtowc(&wide_char, altstr, 1);
+	    }
+
+	    if (char_len <= 0)
+	    	break;
+
+	    w = wcwidth(wide_char);
+
+	    /*
+	     * Multibyte characters can have a variable number of column
+	     * widths, so use the column width to bump the end pointer when
+	     * appropriate.
+	     */
+	    if (w >= 0  &&  char_len > 1 && epmax - *ep >= char_len - w) {
+		*ep += char_len - w;
+	    }
+
+	    if (w >= 0  &&  cp + w > *ep)
 		break;
 
 	    end -= char_len;
@@ -157,8 +173,12 @@ cptrimmed(char **dest, char *str, unsigned int wid, char fill, size_t n) {
 	    if (iswcntrl(wide_char) || iswspace(wide_char)) {
 		sp += char_len;
 #else
+            int c;
 	    end--;
-	    if (iscntrl(*sp) || isspace(*sp)) {
+            /* isnctrl(), etc., take an int argument.  Cygwin's ctype.h
+               intentionally warns if they are passed a char. */
+            c = (unsigned char) *sp;
+	    if (iscntrl(c) || isspace(c)) {
 		sp++;
 #endif
 		if (!prevCtrl) {
@@ -172,11 +192,11 @@ cptrimmed(char **dest, char *str, unsigned int wid, char fill, size_t n) {
 	    prevCtrl = 0;
 
 #ifdef MULTIBYTE_SUPPORT
-	    w = wcwidth(wide_char);
 	    if (w >= 0 && remaining >= w) {
-		strncpy(cp, sp, char_len);
+		strncpy(cp, altstr ? altstr : sp, char_len);
 		cp += char_len;
 		remaining -= w;
+		altstr = NULL;
 	    }
 	    sp += char_len;
 #else
@@ -187,9 +207,10 @@ cptrimmed(char **dest, char *str, unsigned int wid, char fill, size_t n) {
     }
 
     if (ljust) {
-	if (cp + remaining > ep)
-	    remaining = ep - cp;
-	ep = cp + remaining;
+        char *endfield;
+	if (cp + remaining > *ep)
+	    remaining = *ep - cp;
+	endfield = cp + remaining;
 	if (remaining > 0) {
 	    /* copy string to the right */
 	    while (--cp >= *dest)
@@ -199,47 +220,99 @@ cptrimmed(char **dest, char *str, unsigned int wid, char fill, size_t n) {
 	    for (c=remaining; c>0; c--)
 		*cp-- = fill;
 	}
-	*dest = ep;
+	*dest = endfield;
     } else {
 	/* pad remaining space */
-	while (remaining-- > 0 && cp < ep)
+	while (remaining-- > 0 && cp < *ep)
 		*cp++ = fill;
 	*dest = cp;
     }
 }
 
 static void
-cpstripped (char **start, char *end, char *str)
+cpstripped (char **dest, char **end, char *max, char *str)
 {
-    int c;
-    char *s = str;
+    int prevCtrl = 1;	/* This is 1 so we strip out leading spaces */
+    int len;
+#ifdef MULTIBYTE_SUPPORT
+    int char_len, w;
+    wchar_t wide_char;
+    char *altstr = NULL;
+#endif /* MULTIBYTE_SUPPORT */
 
-    if (!s)
+    if (!str)
 	return;
 
-    /* skip any initial control characters or spaces */
-    while ((c = (unsigned char) *s) &&
-#ifdef LOCALE
-	    (iscntrl(c) || isspace(c)))
-#else
-	    (c <= 32))
-#endif
-	s++;
+    len = strlen(str);
 
-    /* compact repeated control characters and spaces into a single space */
-    while((c = (unsigned char) *s++) && *start < end)
-	if (!iscntrl(c) && !isspace(c))
-	    *(*start)++ = c;
-	else {
-	    while ((c = (unsigned char) *s) &&
-#ifdef LOCALE
-		    (iscntrl(c) || isspace(c)))
-#else
-		    (c <= 32))
-#endif
-		s++;
-	    *(*start)++ = ' ';
+#ifdef MULTIBYTE_SUPPORT
+    mbtowc(NULL, NULL, 0);  /* Reset shift state */
+#endif /* MULTIBYTE_SUPPORT */
+
+    /*
+     * Process each character at a time; if we have multibyte support
+     * then deal with that here.
+     */
+
+    while (*str != '\0' && len > 0 && *dest < *end) {
+#ifdef MULTIBYTE_SUPPORT
+    	char_len = mbtowc(&wide_char, str, len);
+	w = wcwidth(wide_char);
+
+	/*
+	 * Account for multibyte characters, and increment the end pointer
+	 * by the number of "extra" bytes in this character.  That's the
+	 * character length (char_len) minus the column width (w).
+	 */
+	if (w >= 0  &&  char_len > 1  &&  max - *end >= char_len - w) {
+	    *end += char_len - w;
 	}
+
+	/*
+	 * If mbrtowc() failed, then we have a character that isn't valid
+	 * in the current encoding.  Replace it with a '?'.  We do that by
+	 * setting the alstr variable to the value of the replacement string;
+	 * altstr is used below when the bytes are copied into the output
+	 * buffer.
+	 */
+
+	if (char_len < 0) {
+	    altstr = "?";
+	    char_len = mbtowc(&wide_char, altstr, 1);
+	}
+
+	if (char_len <= 0 || *dest + char_len > *end)
+	    break;
+
+	len -= char_len;
+
+	if (iswcntrl(wide_char) || iswspace(wide_char)) {
+	    str += char_len;
+#else /* MULTIBYTE_SUPPORT */
+	int c = (unsigned char) *str;
+	len--;
+	if (iscntrl(c) || isspace(c)) {
+	    str++;
+#endif /* MULTIBYTE_SUPPORT */
+	    if (! prevCtrl) {
+	    	*(*dest)++ = ' ';
+	    }
+
+	    prevCtrl = 1;
+	    continue;
+	}
+
+	prevCtrl = 0;
+
+#ifdef MULTIBYTE_SUPPORT
+	memcpy(*dest, altstr ? altstr : str, char_len);
+	str += char_len;
+	*dest += char_len;
+	altstr = NULL;
+#else /* MULTIBYE_SUPPORT */
+	*(*dest)++ = *str++
+#endif /* MULTIBYTE_SUPPORT */
+    }
 }
 
 static char *lmonth[] = { "January",  "February","March",   "April",
@@ -291,12 +364,11 @@ get_x400_comp (char *mbox, char *key, char *buffer, int buffer_len)
 }
 
 struct format *
-fmt_scan (struct format *format, char *scanl, int width, int *dat)
+fmt_scan (struct format *format, char *scanl, size_t max, int width, int *dat,
+	  struct fmt_callbacks *callbacks)
 {
-    char *cp, *ep;
-    unsigned char *sp;
-    char *savestr = NULL;
-    unsigned char *str = NULL;
+    char *cp, *ep, *sp;
+    char *savestr = NULL, *str = NULL;
     char buffer[BUFSIZ], buffer2[BUFSIZ];
     int i, c, ljust, n;
     int value = 0;
@@ -306,8 +378,11 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
     struct tws *tws;
     struct mailname *mn;
 
+    /* ep keeps track of displayed characters.  They're limited by width.
+       The total number of characters, cp - scanl + 1 (for trailing NULL),
+       includes invisible control characters and is limited by max. */
     cp = scanl;
-    ep = scanl + width - 1;
+    ep = scanl + (width <= (int) max ? width : (int) max) - 1;
 
     for (fmt = format; fmt->f_type != FT_DONE; fmt++)
 	switch (fmt->f_type) {
@@ -315,6 +390,27 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	case FT_PARSEDATE:
 	    fmt->f_comp->c_flags &= ~CF_PARSED;
 	    break;
+	case FT_COMP:
+	case FT_COMPF:
+	case FT_LS_COMP:
+	case FT_LS_DECODECOMP:
+	    /*
+	     * Trim these components of any newlines.
+	     *
+	     * But don't trim the "body" and "text" components.
+	     */
+
+	    comp = fmt->f_comp;
+
+	    if (! (comp->c_flags & CF_TRIMMED) && comp->c_text &&
+		(i = strlen(comp->c_text)) > 0) {
+		if (comp->c_text[i - 1] == '\n' &&
+		    strcmp(comp->c_name, "body") != 0 &&
+		    strcmp(comp->c_name, "text") != 0)
+		    comp->c_text[i - 1] = '\0';
+		comp->c_flags |= CF_TRIMMED;
+	    }
+	    break;
 	}
 
     fmt = format;
@@ -323,10 +419,11 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	switch (fmt->f_type) {
 
 	case FT_COMP:
-	    cpstripped (&cp, ep, fmt->f_comp->c_text);
+	    cpstripped (&cp, &ep, scanl + max - 1, fmt->f_comp->c_text);
 	    break;
 	case FT_COMPF:
-	    cptrimmed (&cp, fmt->f_comp->c_text, fmt->f_width, fmt->f_fill, ep - cp);
+	    cptrimmed (&cp, &ep, fmt->f_comp->c_text, fmt->f_width, fmt->f_fill,
+		       scanl + max - 1);
 	    break;
 
 	case FT_LIT:
@@ -349,10 +446,37 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	    break;
 
 	case FT_STR:
-	    cpstripped (&cp, ep, str);
+	    cpstripped (&cp, &ep, scanl + max - 1, str);
 	    break;
 	case FT_STRF:
-	    cptrimmed (&cp, str, fmt->f_width, fmt->f_fill, ep - cp);
+	    cptrimmed (&cp, &ep, str, fmt->f_width, fmt->f_fill, 
+		       scanl + max - 1);
+	    break;
+	case FT_STRLIT:
+	    if (str) {
+		sp = str;
+		while ((c = *sp++) && cp < ep)
+		    *cp++ = c;
+	    }
+	    break;
+	case FT_STRLITZ:
+	    if (str) {
+		size_t len = strlen (str);
+
+		/* Don't want to emit part of an escape sequence.  So if
+		   there isn't enough room in the buffer for the entire
+		   string, skip it completely. */
+		if (cp - scanl + len + 1 < max) {
+		    for (sp = str; *sp; *cp++ = *sp++) continue;
+
+		    /* This string doesn't count against the width.
+		       So increase ep the same amount as cp, only if the
+		       scan buffer will always be large enough. */
+		    if (ep - scanl + len + 1 < max) {
+			ep += len;
+		    }
+		}
+	    }
 	    break;
 	case FT_STRFW:
 	    adios (NULL, "internal error (FT_STRFW)");
@@ -375,10 +499,16 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	    break;
 
 	case FT_DONE:
+	    if (callbacks && callbacks->trace_func)
+		callbacks->trace_func(callbacks->trace_context, fmt, value,
+				      str, scanl);
 	    goto finished;
 
 	case FT_IF_S:
 	    if (!(value = (str && *str))) {
+		if (callbacks && callbacks->trace_func)
+		    callbacks->trace_func(callbacks->trace_context, fmt, value,
+					  str, scanl);
 		fmt += fmt->f_skip;
 		continue;
 	    }
@@ -386,6 +516,9 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 
 	case FT_IF_S_NULL:
 	    if (!(value = (str == NULL || *str == 0))) {
+		if (callbacks && callbacks->trace_func)
+		    callbacks->trace_func(callbacks->trace_context, fmt, value,
+					  str, scanl);
 		fmt += fmt->f_skip;
 		continue;
 	    }
@@ -393,6 +526,9 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 
 	case FT_IF_V_EQ:
 	    if (value != fmt->f_value) {
+		if (callbacks && callbacks->trace_func)
+		    callbacks->trace_func(callbacks->trace_context, fmt, value,
+					  str, scanl);
 		fmt += fmt->f_skip;
 		continue;
 	    }
@@ -400,6 +536,9 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 
 	case FT_IF_V_NE:
 	    if (value == fmt->f_value) {
+		if (callbacks && callbacks->trace_func)
+		    callbacks->trace_func(callbacks->trace_context, fmt, value,
+					  str, scanl);
 		fmt += fmt->f_skip;
 		continue;
 	    }
@@ -407,6 +546,9 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 
 	case FT_IF_V_GT:
 	    if (value <= fmt->f_value) {
+		if (callbacks && callbacks->trace_func)
+		    callbacks->trace_func(callbacks->trace_context, fmt, value,
+					  str, scanl);
 		fmt += fmt->f_skip;
 		continue;
 	    }
@@ -414,6 +556,9 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 
 	case FT_IF_MATCH:
 	    if (!(value = (str && match (str, fmt->f_text)))) {
+		if (callbacks && callbacks->trace_func)
+		    callbacks->trace_func(callbacks->trace_context, fmt, value,
+					  str, scanl);
 		fmt += fmt->f_skip;
 		continue;
 	    }
@@ -428,6 +573,9 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 
 	case FT_IF_AMATCH:
 	    if (!(value = (str && uprf (str, fmt->f_text)))) {
+		if (callbacks && callbacks->trace_func)
+		    callbacks->trace_func(callbacks->trace_context, fmt, value,
+					  str, scanl);
 		fmt += fmt->f_skip;
 		continue;
 	    }
@@ -458,6 +606,9 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	    break;
 
 	case FT_GOTO:
+	    if (callbacks && callbacks->trace_func)
+		callbacks->trace_func(callbacks->trace_context, fmt, value,
+				      str, scanl);
 	    fmt += fmt->f_skip;
 	    continue;
 
@@ -493,12 +644,12 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 
 	case FT_LS_TRIM:
 	    if (str) {
-		    unsigned char *xp;
+		    char *xp;
 
 		    strncpy(buffer, str, sizeof(buffer));
 		    buffer[sizeof(buffer)-1] = '\0';
 		    str = buffer;
-		    while (isspace(*str))
+		    while (isspace((unsigned char) *str))
 			    str++;
 		    ljust = 0;
 		    if ((i = fmt->f_width) < 0) {
@@ -506,13 +657,13 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 			    ljust++;
 		    }
 
-		    if (!ljust && i > 0 && strlen(str) > i)
+		    if (!ljust && i > 0 && (int) strlen(str) > i)
 			    str[i] = '\0';
 		    xp = str;
 		    xp += strlen(str) - 1;
-		    while (xp > str && isspace(*xp))
+		    while (xp > str && isspace((unsigned char) *xp))
 			    *xp-- = '\0';
-		    if (ljust && i > 0 && strlen(str) > i)
+		    if (ljust && i > 0 && (int) strlen(str) > i)
 			str += strlen(str) - i;
 	    }
 	    break;
@@ -728,32 +879,10 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 		/* UNQUOTEs RFC-2822 quoted-string and quoted-pair */
 	case FT_LS_UNQUOTE:
 	    if (str) {	  	
-		int m;
 		strncpy(buffer, str, sizeof(buffer));
 		/* strncpy doesn't NUL-terminate if it fills the buffer */
 		buffer[sizeof(buffer)-1] = '\0';
-		str = buffer;
-	
-		/* we will parse from buffer to buffer2 */
-		n = 0; /* n is the input position in str */
-		m = 0; /* m is the ouput position in buffer2 */
-
-		while ( str[n] != '\0') {
-		    switch ( str[n] ) {
-			case '\\':
-			    n++;
-			    if ( str[n] != '\0')
-				buffer2[m++] = str[n++];
-			    break;
-			case '"':
-			    n++;
-			    break;
-			default:
-			    buffer2[m++] = str[n++];
-			    break;
-			}
-		}
-		buffer2[m] = '\0';
+		unquote_string(buffer, buffer2);
 		str = buffer2;
             }
 	    break;
@@ -790,7 +919,18 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 
 	case FT_FORMATADDR:
 	    /* hook for custom address list formatting (see replsbr.c) */
-	    str = formataddr (savestr, str);
+	    if (callbacks && callbacks->formataddr)
+		str = callbacks->formataddr (savestr, str);
+	    else
+		str = formataddr (savestr, str);
+	    break;
+
+	case FT_CONCATADDR:
+	    /* The same as formataddr, but doesn't do duplicate suppression */
+	    if (callbacks && callbacks->concataddr)
+		str = callbacks->concataddr (savestr, str);
+	    else
+		str = concataddr (savestr, str);
 	    break;
 
 	case FT_PUTADDR:
@@ -801,8 +941,7 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	     * (e.g., "To: ")
 	     */
 	    {
-	    unsigned char *lp;
-	    char *lastb;
+	    char *lp, *lastb;
 	    int indent, wid, len;
 
 	    lp = str;
@@ -811,21 +950,26 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	    sp = fmt->f_text;
 	    indent = strlen (sp);
 	    wid -= indent;
-	    while( (c = *sp++) && cp < ep)
-		*cp++ = c;
+	    if (wid <= 0) {
+	    	adios(NULL, "putaddr -- num register (%d) must be greater "
+			    "than label width (%d)", value, indent);
+	    }
+	    while( (c = (unsigned char) *sp++) && cp < ep)
+		*cp++ = (char) c;
 	    while (len > wid) {
 		/* try to break at a comma; failing that, break at a
 		 * space.
 		 */
 		lastb = 0; sp = lp + wid;
-		while (sp > lp && (c = *--sp) != ',') {
+		while (sp > lp && (c = (unsigned char) *--sp) != ',') {
 		    if (! lastb && isspace(c))
 			lastb = sp - 1;
 		}
 		if (sp == lp) {
 		    if (! (sp = lastb)) {
 			sp = lp + wid - 1;
-			while (*sp && *sp != ',' && !isspace(*sp))
+			while (*sp && *sp != ',' &&
+						!isspace((unsigned char) *sp))
 			    sp++;
 			if (*sp != ',')
 			    sp--;
@@ -834,7 +978,7 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 		len -= sp - lp + 1;
 		while (cp < ep && lp <= sp)
 		    *cp++ = *lp++;
-		while (isspace(*lp))
+		while (isspace((unsigned char) *lp))
 		    lp++, len--;
 		if (*lp) {
 		    if (cp < ep)
@@ -843,7 +987,7 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 			*cp++ = ' ';
 		}
 	    }
-	    cpstripped (&cp, ep, lp);
+	    cpstripped (&cp, &ep, scanl + max - 1, lp);
 	    }
 	    break;
 
@@ -854,7 +998,7 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	    if (comp->c_mn != &fmt_mnull)
 		mnfree (comp->c_mn);
 	    if ((sp = comp->c_text) && (sp = getname(sp)) &&
-		(mn = getm (sp, NULL, 0, fmt_norm, NULL))) {
+		(mn = getm (sp, NULL, 0, NULL, 0))) {
 		comp->c_mn = mn;
 		while (getname(""))
 		    ;
@@ -876,7 +1020,7 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 	    if (comp->c_mn != &fmt_mnull)
 		mnfree (comp->c_mn);
 	    if ((sp = comp->c_text) && (sp = getname(sp)) &&
-		(mn = getm (sp, NULL, 0, AD_NAME, NULL))) {
+		(mn = getm (sp, NULL, 0, NULL, 0))) {
 		comp->c_mn = mn;
 		if (ismymbox(mn))
 		    comp->c_flags |= CF_TRUE;
@@ -884,7 +1028,7 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 		    comp->c_flags &= ~CF_TRUE;
 		while ((sp = getname(sp)))
 		    if ((comp->c_flags & CF_TRUE) == 0 &&
-			(mn = getm (sp, NULL, 0, AD_NAME, NULL)))
+			(mn = getm (sp, NULL, 0, NULL, 0)))
 			if (ismymbox(mn))
 			    comp->c_flags |= CF_TRUE;
 	    } else {
@@ -897,35 +1041,48 @@ fmt_scan (struct format *format, char *scanl, int width, int *dat)
 		comp->c_mn = &fmt_mnull;
 	    }
 	    break;
+	}
 
-	case FT_ADDTOSEQ:
-#ifdef LBL
-	    /* If we're working on a folder (as opposed to a file), add the
-	     * current msg to sequence given in literal field.  Don't
-	     * disturb string or value registers.
-	     */
-	    if (fmt_current_folder)
-		    seq_addmsg(fmt_current_folder, fmt->f_text, dat[0], -1);
-#endif
-	    break;
+	/*
+	 * Call our tracing callback function, if one was supplied
+	 */
+
+	if (callbacks && callbacks->trace_func)
+	    callbacks->trace_func(callbacks->trace_context, fmt, value,
+	    			  str, scanl);
+	fmt++;
+    }
+
+    /* Emit any trailing sequences of zero display length. */
+    while (fmt->f_type != FT_DONE) {
+	if (fmt->f_type == FT_LS_LIT) {
+	    str = fmt->f_text;
+	    if (callbacks && callbacks->trace_func)
+		callbacks->trace_func(callbacks->trace_context, fmt, value,
+				      str, scanl);
+	} else if (fmt->f_type == FT_STRLITZ) {
+	    /* Don't want to emit part of an escape sequence.  So if
+	       there isn't enough room in the buffer for the entire
+	       string, skip it completely.  Need room for null
+	       terminator, and maybe trailing newline (added below). */
+	    if (str && (cp - scanl + strlen (str) + 1 < max)) {
+		for (sp = str; *sp; *cp++ = *sp++) continue;
+	    }
+	    if (callbacks && callbacks->trace_func)
+		callbacks->trace_func(callbacks->trace_context, fmt, value,
+				      str, scanl);
 	}
 	fmt++;
     }
-#ifndef JLR
-    finished:;
-    if (cp[-1] != '\n')
-	*cp++ = '\n';
-    *cp   = 0;
-    return ((struct format *)0);
-#else /* JLR */
-    if (cp[-1] != '\n')
-	*cp++ = '\n';
-    while (fmt->f_type != FT_DONE)
-	fmt++;
 
     finished:;
+    if (cp > scanl  &&  cp[-1] != '\n') {
+	if (cp - scanl < (int) max - 1) {
+	    *cp++ = '\n';
+	} else {
+	    cp[-1] = '\n';
+	}
+    }
     *cp = '\0';
-    return (fmt->f_value ? ++fmt : (struct format *) 0);
-
-#endif /* JLR */
+    return ((struct format *)0);
 }
