@@ -2,14 +2,13 @@
 /*
  * folder_delmsgs.c -- "remove" SELECTED messages from a folder
  *
- * $Id: folder_delmsgs.c,v 1.4 2004/10/12 20:31:14 jon Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
 #include <h/mh.h>
+#include <h/utils.h>
 
 /*
  * 1) If we are using an external rmmproc, then exec it.
@@ -26,7 +25,7 @@ folder_delmsgs (struct msgs *mp, int unlink_msgs, int nohook)
 {
     pid_t pid;
     int msgnum, vecp, retval = 0;
-    char buf[100], *dp, **vec;
+    char buf[100], *dp, **vec, *prog;
     char	msgpath[BUFSIZ];
 
     /*
@@ -42,13 +41,27 @@ folder_delmsgs (struct msgs *mp, int unlink_msgs, int nohook)
 	/* Mark that the sequence information has changed */
 	mp->msgflags |= SEQMOD;
 
-	if (mp->numsel > MAXARGS - 2)
-	    adios (NULL, "more than %d messages for %s exec", MAXARGS - 2,
-		   rmmproc);
-	vec = (char **) calloc ((size_t) (mp->numsel + 2), sizeof(*vec));
+	/*
+	 * Write out the sequence and context files; this will release
+	 * any locks before the rmmproc is called.
+	 */
+
+	seq_save (mp);
+	context_save ();
+
+	vec = argsplit(rmmproc, &prog, &vecp);
+
+	/*
+	 * argsplit allocates a MAXARGS vector by default,  If we need
+	 * something bigger, allocate it ourselves
+	 */
+
+	if (mp->numsel + vecp + 1 > MAXARGS)
+	    vec = (char **) mh_xrealloc (vec,
+                                         (size_t) ((mp->numsel + vecp + 1) *
+						     sizeof(*vec)));
 	if (vec == NULL)
 	    adios (NULL, "unable to allocate exec vector");
-	vecp = 1;
 	for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
 	    if (is_selected (mp, msgnum) &&
 		!(vec[vecp++] = strdup (m_name (msgnum))))
@@ -57,20 +70,20 @@ folder_delmsgs (struct msgs *mp, int unlink_msgs, int nohook)
 	vec[vecp] = NULL;
 
 	fflush (stdout);
-	vec[0] = r1bindex (rmmproc, '/');
 
-	switch (pid = vfork()) {
+	switch (pid = fork()) {
 	case -1:
 	    advise ("fork", "unable to");
 	    return -1;
 
 	case 0:
-	    execvp (rmmproc, vec);
+	    execvp (prog, vec);
 	    fprintf (stderr, "unable to exec ");
 	    perror (rmmproc);
 	    _exit (-1);
 
 	default:
+	    arglist_free(prog, vec);
 	    return (pidwait (pid, -1));
 	}
     }
@@ -101,7 +114,7 @@ folder_delmsgs (struct msgs *mp, int unlink_msgs, int nohook)
 
 	    if (unlink_msgs) {
 		/* just unlink the messages */
-		if (unlink (dp) == -1) {
+		if (m_unlink (dp) == -1) {
 		    admonish (dp, "unable to unlink");
 		    retval = -1;
 		    continue;
@@ -129,5 +142,12 @@ folder_delmsgs (struct msgs *mp, int unlink_msgs, int nohook)
     /* Mark that the sequence information has changed */
     mp->msgflags |= SEQMOD;
 
+    /*
+     * Write out sequence and context files
+     */
+
+    seq_save (mp);
+    context_save ();
+
     return retval;
 }
