@@ -2,7 +2,9 @@
 /*
  * gans.c -- get an answer from the user
  *
- * $Id: gans.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
+ * This code is Copyright (c) 2002, by the authors of nmh.  See the
+ * COPYRIGHT file in the root directory of the nmh distribution for
+ * complete copyright information.
  */
 
 #include <h/mh.h>
@@ -24,12 +26,7 @@ gans (char *prompt, struct swit *ansp)
 	    if (i == EOF)
 		return 0;
 	    if (cp < &ansbuf[sizeof ansbuf - 1]) {
-#ifdef LOCALE
 		i = (isalpha(i) && isupper(i)) ? tolower(i) : i;
-#else
-		if (i >= 'A' && i <= 'Z')
-		    i += 'a' - 'A';
-#endif
 		*cp++ = i;
 	    }
 	}
