@@ -1,10 +1,13 @@
 
 /*
  * getanswer.c -- get a yes/no answer from the user
+ *
+ * This code is Copyright (c) 2002, by the authors of nmh.  See the
+ * COPYRIGHT file in the root directory of the nmh distribution for
+ * complete copyright information.
  */
 
 #include <h/mh.h>
-#include <stdio.h>
 
 
 int
