@@ -6,8 +6,6 @@
  * OF GETCPY WITH STRDUP.  BUT THIS WILL REQUIRE
  * CHANGING PARTS OF THE CODE TO DEAL WITH NULL VALUES.
  *
- * $Id: getcpy.c,v 1.3 2006/01/02 03:17:42 bress Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -18,7 +16,7 @@
 
 
 char *
-getcpy (char *str)
+getcpy (const char *str)
 {
     char *cp;
     size_t len;
