@@ -4,8 +4,6 @@
  *          -- return the numeric value of the message.  If the string
  *          -- contains any non-digit characters, then return 0.
  *
- * $Id: m_atoi.c,v 1.3 2007/04/13 11:53:08 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -18,14 +16,10 @@ int
 m_atoi (char *str)
 {
     int i;
-    unsigned char *cp;
+    char *cp;
 
     for (i = 0, cp = str; *cp; cp++) {
-#ifdef LOCALE
-	if (!isdigit(*cp))
-#else
-	if (*cp < '0' || *cp > '9')
-#endif
+	if (!isdigit((unsigned char) *cp))
 	    return 0;
 
 	i *= 10;
