@@ -2,8 +2,6 @@
 /*
  * m_backup.c -- construct a backup file
  *
- * $Id: m_backup.c,v 1.3 2008/04/05 18:41:37 pm215 Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -25,6 +23,6 @@ m_backup (char *file)
 	snprintf(buffer, sizeof(buffer), "%.*s%s%s", (int)(cp - file), file,
 		BACKUP_PREFIX, cp);
 
-    unlink(buffer);
+    (void) m_unlink(buffer);
     return buffer;
 }
