@@ -2,13 +2,13 @@
 /*
  * m_draft.c -- construct the name of a draft message
  *
- * $Id: m_draft.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
+ * This code is Copyright (c) 2002, by the authors of nmh.  See the
+ * COPYRIGHT file in the root directory of the nmh distribution for
+ * complete copyright information.
  */
 
 #include <h/mh.h>
-#include <errno.h>
-
-extern int errno;
+#include <h/utils.h>
 
 
 char *
@@ -16,7 +16,6 @@ m_draft (char *folder, char *msg, int use, int *isdf)
 {
     register char *cp;
     register struct msgs *mp;
-    struct stat st;
     static char buffer[BUFSIZ];
 
     if (*isdf == -1 || folder == NULL || *folder == '\0') {
@@ -32,21 +31,13 @@ m_draft (char *folder, char *msg, int use, int *isdf)
     
     chdir (m_maildir (""));
     strncpy (buffer, m_maildir (folder), sizeof(buffer));
-    if (stat (buffer, &st) == -1) {
-	if (errno != ENOENT)
-	    adios (buffer, "error on folder");
-	cp = concat ("Create folder \"", buffer, "\"? ", NULL);
-	if (!getanswer (cp))
-	    done (0);
-	free (cp);
-	if (!makedir (buffer))
-	    adios (NULL, "unable to create folder %s", buffer);
-    }
+
+    create_folder (buffer, 0, done);
 
     if (chdir (buffer) == -1)
 	adios (buffer, "unable to change directory to");
 
-    if (!(mp = folder_read (folder)))
+    if (!(mp = folder_read (folder, 1)))
 	adios (NULL, "unable to read folder %s", folder);
 
     /*
