@@ -8,29 +8,19 @@
  */
 
 #include <h/mh.h>   /* for snprintf() */
-#include <h/nmh.h>
 #include <h/utils.h>
 
 #define nmhetcdir(file) NMHETCDIR#file
 
-#include <ctype.h>
-#include <stdio.h>
 #include <h/mts.h>
 #include <pwd.h>
 #include <sys/socket.h>
 #include <netdb.h>
 
-#ifdef HAVE_SYS_UTSNAME_H
-# include <sys/utsname.h>
-#endif
-
-#define	NOTOK   (-1)
-#define	OK        0
-
 /*
  * static prototypes
  */
-static char *tailor_value (unsigned char *);
+static char *tailor_value (char *);
 static void getuserinfo (void);
 static const char *get_mtsconf_pathname(void);
 static const char *get_mtsuserconf_pathname(void);
@@ -61,31 +51,25 @@ char *uucplfil = "";
 char *mmdlm1 = "\001\001\001\001\n";
 char *mmdlm2 = "\001\001\001\001\n";
 
-/* Cache the username and fullname of the user */
+char *spoollocking = DEFAULT_LOCKING;
+
+/* Cache the username, fullname, and mailbox of the user */
 static char username[BUFSIZ];
 static char fullname[BUFSIZ];
-
-/* Variables for username masquerading: */
-       boolean  draft_from_masquerading = FALSE;  /* also used from post.c */
-static boolean  mmailid_masquerading = FALSE;
-       boolean  username_extension_masquerading = FALSE;  /* " from addrsbr.c */
-static char*    masquerade = "";
+static char localmbox[BUFSIZ];
 
 /*
  * MTS specific variables
  */
-#if defined(SMTPMTS)
-static char *sm_method = "smtp";
-int  sm_mts    = MTS_SMTP;
-char *hostable = nmhetcdir(/hosts);
+static char *mts_method = "smtp";
+int  sm_mts    = MTS_SENDMAIL_SMTP;
 char *sendmail = SENDMAILPATH;
-#endif
 
 /*
  * SMTP/POP stuff
  */
 char *clientname = NULL;
-char *servers    = "localhost \01localnet";
+char *servers    = "localhost";
 char *pophost    = "";
 
 /*
@@ -117,18 +101,13 @@ static struct bind binds[] = {
     { "systemname", &systemname },
     { "mmdfldir", &mmdfldir },
     { "mmdflfil", &mmdflfil },
+    { "spoollocking", &spoollocking },
     { "uucpldir", &uucpldir },
     { "uucplfil", &uucplfil },
     { "mmdelim1", &mmdlm1 },
     { "mmdelim2", &mmdlm2 },
-    { "masquerade", &masquerade },
-
-#if defined(SMTPMTS)
-    { "mts",      &sm_method },
-    { "hostable", &hostable  },
+    { "mts",      &mts_method },
     { "sendmail", &sendmail  },
-#endif
-
     { "clientname",  &clientname },
     { "servers", &servers },
     { "pophost", &pophost },
@@ -140,6 +119,25 @@ static struct bind binds[] = {
 };
 
 
+/* Convert name of mts method to integer value and store it. */
+void
+save_mts_method (const char *value) {
+    if (! strcasecmp (value, "smtp")) {
+        mts_method = "smtp";
+        sm_mts = MTS_SMTP;
+    } else if (! strcasecmp (value, "sendmail/smtp")  ||
+               ! strcasecmp (value, "sendmail")) {
+        mts_method = "sendmail/smtp";
+        sm_mts = MTS_SENDMAIL_SMTP;
+    } else if (! strcasecmp (value, "sendmail/pipe")) {
+        mts_method = "sendmail/pipe";
+        sm_mts = MTS_SENDMAIL_PIPE;
+    } else {
+        adios (NULL, "unsupported mts selection \"%s\"", value);
+    }
+}
+
+
 /*
  * Read the configuration file for the nmh interface
  * to the mail transport system (MTS).
@@ -151,6 +149,7 @@ mts_init (char *name)
     const char *cp;
     FILE *fp;
     static int inited = 0;
+    NMH_UNUSED (name);
 
     if (inited++ || (fp = fopen (get_mtsconf_pathname(), "r")) == NULL)
 	return;
@@ -166,25 +165,7 @@ mts_init (char *name)
 
     Everyone = atoi (everyone);
 
-    if (strstr(masquerade, "draft_from") != NULL)
-	draft_from_masquerading = TRUE;
-
-    if (strstr(masquerade, "mmailid") != NULL)
-	mmailid_masquerading = TRUE;
-
-    if (strstr(masquerade, "username_extension") != NULL)
-	username_extension_masquerading = TRUE;
-
-#ifdef SMTPMTS
-    if (strcmp(sm_method, "smtp") == 0)
-        sm_mts = MTS_SMTP;
-    else if (strcmp(sm_method, "sendmail") == 0)
-        sm_mts = MTS_SENDMAIL;
-    else {
-        advise(NULL, "unsupported \"mts\" value in mts.conf: %s", sm_method);
-        sm_mts = MTS_SMTP;
-    }
-#endif
+    save_mts_method (mts_method);
 }
 
 
@@ -196,7 +177,7 @@ mts_init (char *name)
  */
 
 static char *
-tailor_value (unsigned char *s)
+tailor_value (char *s)
 {
     int i, r;
     char *bp;
@@ -219,13 +200,13 @@ tailor_value (unsigned char *s)
 		    break;
 
 		default: 
-		    if (!isdigit (*s)) {
+		    if (!isdigit ((unsigned char) *s)) {
 			*bp++ = QUOTE;
 			*bp = *s;
 		    }
-		    r = *s != '0' ? 10 : 8;
-		    for (i = 0; isdigit (*s); s++)
-			i = i * r + *s - '0';
+		    r = ((unsigned char) *s) != '0' ? 10 : 8;
+		    for (i = 0; isdigit ((unsigned char) *s); s++)
+			i = i * r + ((unsigned char) *s) - '0';
 		    s--;
 		    *bp = toascii (i);
 		    break;
@@ -243,43 +224,45 @@ tailor_value (unsigned char *s)
 
 /*
  * Get the fully qualified name of the local host.
+ *
+ * If flag is 0, then use anything out of mts.conf (like localname).
+ * If flag is 1, then only use the "proper" local hostname.
  */
 
 char *
-LocalName (void)
+LocalName (int flag)
 {
-    static char buffer[BUFSIZ] = "";
+    static char buffer0[BUFSIZ] = "";
+    static char buffer1[BUFSIZ] = "";
+    static char *buffer[] = { buffer0, buffer1 };
+    char *buf;
     struct addrinfo hints, *res;
-#ifdef HAVE_UNAME
-    struct utsname name;
-#endif
+
+    if (flag < 0 || flag > 1)
+    	return NULL;
+
+    buf = buffer[flag];
 
     /* check if we have cached the local name */
-    if (buffer[0])
-	return buffer;
+    if (buf[0])
+	return buf;
 
     mts_init ("mts");
 
     /* check if the mts.conf file specifies a "localname" */
-    if (*localname) {
-	strncpy (buffer, localname, sizeof(buffer));
+    if (*localname && flag == 0) {
+	strncpy (buf, localname, sizeof(buffer0));
     } else {
-	memset(buffer, 0, sizeof(buffer));
-#ifdef HAVE_UNAME
+	memset(buf, 0, sizeof(buffer0));
 	/* first get our local name */
-	uname (&name);
-	strncpy (buffer, name.nodename, sizeof(buffer) - 1);
-#else
-	/* first get our local name */
-	gethostname (buffer, sizeof(buffer) - 1);
-#endif
+	gethostname (buf, sizeof(buffer0) - 1);
 	/* now fully qualify our name */
 
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_flags = AI_CANONNAME;
 	hints.ai_family = PF_UNSPEC;
-	if (getaddrinfo(buffer, NULL, &hints, &res) == 0) {
-	    strncpy(buffer, res->ai_canonname, sizeof(buffer) - 1);
+	if (getaddrinfo(buf, NULL, &hints, &res) == 0) {
+	    strncpy(buf, res->ai_canonname, sizeof(buffer0) - 1);
 	    freeaddrinfo(res);
 	}
     }
@@ -289,11 +272,11 @@ LocalName (void)
      * we append that now.  This should rarely be needed.
      */
     if (*localdomain) {
-	strcat (buffer, ".");
-	strcat (buffer, localdomain);
+	strcat (buf, ".");
+	strcat (buf, localdomain);
     }
 
-    return buffer;
+    return buf;
 }
 
 
@@ -307,10 +290,6 @@ SystemName (void)
 {
     static char buffer[BUFSIZ] = "";
 
-#ifdef HAVE_UNAME
-    struct utsname name;
-#endif
-
     /* check if we have cached the system name */
     if (buffer[0])
 	return buffer;
@@ -323,12 +302,7 @@ SystemName (void)
 	return buffer;
     }
 
-#ifdef HAVE_UNAME
-    uname (&name);
-    strncpy (buffer, name.nodename, sizeof(buffer));
-#else
     gethostname (buffer, sizeof(buffer));
-#endif
 
     return buffer;
 }
@@ -363,18 +337,30 @@ getfullname (void)
 }
 
 
+/*
+ * Get the full local mailbox name.  This is in the form:
+ *
+ * User Name <user@name.com>
+ */
+
+char *
+getlocalmbox (void)
+{
+    if (username[0] == '\0')
+    	getuserinfo();
+
+    return localmbox;
+}
+
 /*
  * Find the user's username and full name, and cache them.
- * Also, handle "mmailid" username masquerading controlled from the GECOS field
- * of the passwd file. 
  */
 
 static void
 getuserinfo (void)
 {
-    register unsigned char *cp;
-    register char *np;
-    register struct passwd *pw;
+    char *cp, *np;
+    struct passwd *pw;
 
     if ((pw = getpwuid (getuid ())) == NULL
 	    || pw->pw_name == NULL
@@ -385,105 +371,76 @@ getuserinfo (void)
 	return;
     }
 
+
+    /* username */
+    /* If there's a Local-Mailbox profile component, try to extract
+       the username from it.  But don't try very hard, this assumes
+       the very simple User Name <user@name.com> form.
+       Note that post(8) and whom(1) use context_foil (), so they
+       won't see the profile component. */
+    if ((np = context_find("Local-Mailbox")) != NULL) {
+	char *left_angle_bracket = strchr (np, '<');
+	char *at_sign = strchr (np, '@');
+	char *right_angle_bracket = strchr (np, '>');
+
+	strncpy(localmbox, np, sizeof(localmbox));
+
+	if (left_angle_bracket	&&  at_sign  &&	 right_angle_bracket) {
+	    if (at_sign > left_angle_bracket  &&
+		at_sign - left_angle_bracket < BUFSIZ) {
+		strncpy(username, left_angle_bracket + 1,
+			at_sign - left_angle_bracket - 1);
+	    }
+	}
+    }
+
+    if (username[0] == '\0') {
+	strncpy (username, pw->pw_name, sizeof(username));
+    }
+
+    username[sizeof(username) - 1] = '\0';
+
+    escape_local_part(username, sizeof(username));
+
+
+    /* fullname */
     np = pw->pw_gecos;
 
     /* Get the user's real name from the GECOS field.  Stop once we hit a ',',
        which some OSes use to separate other 'finger' information in the GECOS
-       field, like phone number.  Also, if mmailid masquerading is turned on due
-       to "mmailid" appearing on the "masquerade:" line of mts.conf, stop if we
-       hit a '<' (which should precede any ','s). */
-#ifndef BSD42
-    if (mmailid_masquerading)
-	/* Stop at ',' or '<'. */
-	for (cp = fullname; *np != '\0' && *np != ',' && *np != '<';
-	     *cp++ = *np++)
-	    continue;
-    else
-	/* Allow '<' as a legal character of the user's name.  This code is
-	   basically a duplicate of the code above the "else" -- we don't
-	   collapse it down to one copy and put the mmailid_masquerading check
-	   inside the loop with "(x ? y : z)" because that's inefficient and the
-	   value'll never change while it's in there. */
-	for (cp = fullname; *np != '\0' && *np != ',';
-	     *cp++ = *np++)
-	    continue;
-#else /* BSD42 */
-    /* On BSD(-derived) systems, the system utilities that deal with the GECOS
-       field (finger, mail, sendmail, etc.) translate any '&' character in it to
-       the login name, with the first letter capitalized.  So, for instance,
-       fingering a user "bob" with the GECOS field "& Jones" would reveal him to
-       be "In real life: Bob Jones".  Surprisingly, though, the OS doesn't do
-       the translation for you, so we have to do it manually here. */
-    if (mmailid_masquerading)
-	/* Stop at ',' or '<'. */
-	for (cp = fullname;
-	     *np != '\0' && *np != ',' && *np != '<';) {
-	    if (*np == '&')	{	/* blech! */
-		strcpy (cp, pw->pw_name);
-		*cp = toupper(*cp);
-		while (*cp)
-		    cp++;
-		np++;
-	    } else {
-		*cp++ = *np++;
-	    }
-	}
-    else
-	/* Allow '<' as a legal character of the user's name.  This code is
-	   basically a duplicate of the code above the "else" -- we don't
-	   collapse it down to one copy and put the mmailid_masquerading check
-	   inside the loop with "(x ? y : z)" because that's inefficient and the
-	   value'll never change while it's in there. */
-	for (cp = fullname;
-	     *np != '\0' && *np != ',';) {
-	    if (*np == '&')	{	/* blech! */
-		strcpy (cp, pw->pw_name);
-		*cp = toupper(*cp);
-		while (*cp)
-		    cp++;
-		np++;
-	    } else {
-		*cp++ = *np++;
-	    }
-	}
-#endif /* BSD42 */
+       field, like phone number. */
+    for (cp = fullname; *np != '\0' && *np != ','; *cp++ = *np++)
+	continue;
     *cp = '\0';
 
-    if (mmailid_masquerading) {
-	/* Do mmailid processing.  The GECOS field should have the form
-	   "Full Name <fakeusername>".  For instance,
-	   "Dan Harkless <Dan.Harkless>".  Naturally, you'll want your MTA to
-	   have an alias (e.g. in /etc/aliases) from "fakeusername" to your
-	   account name.  */ 
-	if (*np)
-	    np++;
-	for (cp = username; *np && *np != '>'; *cp++ = *np++)
-	    continue;
-	*cp = '\0';
-    }
-    if (!mmailid_masquerading || *np == '\0')
-	strncpy (username, pw->pw_name, sizeof(username));
-
     /* The $SIGNATURE environment variable overrides the GECOS field's idea of
-       your real name. */
+       your real name. If SIGNATURE isn't set, use the Signature profile
+       setting if it exists.
+       Note that post(8) and whom(1) use context_foil (), so they
+       won't see the profile component. */
     if ((cp = getenv ("SIGNATURE")) && *cp)
 	strncpy (fullname, cp, sizeof(fullname));
+    else if ((cp = context_find("Signature")))
+    	strncpy (fullname, cp, sizeof(fullname));
 
-    if (strchr(fullname, '.')) {		/*  quote any .'s */
-	char tmp[BUFSIZ];
+    fullname[sizeof(fullname) - 1] = '\0';
 
-	/* should quote "'s too */
-	snprintf (tmp, sizeof(tmp), "\"%s\"", fullname);
-	strncpy (fullname, tmp, sizeof(fullname));
+    escape_display_name(fullname, sizeof(fullname));
+
+
+    /* localmbox, if not using Local-Mailbox */
+    if (localmbox[0] == '\0') {
+	snprintf(localmbox, sizeof(localmbox), "%s <%s@%s>", fullname,
+		 username, LocalName(0));
     }
 
-    return;
+    localmbox[sizeof(localmbox) - 1] = '\0';
 }
 
 static const char*
 get_mtsconf_pathname (void)
 {
-    const char *cp = getenv ( "MHMTSCONF ");
+    const char *cp = getenv ( "MHMTSCONF" );
     if (cp != NULL && *cp != '\0') {
         return cp;
     }
@@ -503,8 +460,7 @@ get_mtsuserconf_pathname (void)
 static void
 mts_read_conf_file (FILE *fp)
 {
-    unsigned char *bp;
-    char *cp, buffer[BUFSIZ];
+    char *bp, *cp, buffer[BUFSIZ];
     struct bind *b;
 
     while (fgets (buffer, sizeof(buffer), fp)) {
@@ -516,7 +472,7 @@ mts_read_conf_file (FILE *fp)
 	if (!(bp = strchr(buffer, ':')))
 	    break;
 	*bp++ = 0;
-	while (isspace (*bp))
+	while (isspace ((unsigned char) *bp))
 	    *bp++ = 0;
 
 	for (b = binds; b->keyword; b++)
