@@ -2,14 +2,16 @@
 /*
  * print_sw.c -- print switches
  *
- * $Id: print_sw.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
+ * This code is Copyright (c) 2002, by the authors of nmh.  See the
+ * COPYRIGHT file in the root directory of the nmh distribution for
+ * complete copyright information.
  */
 
 #include <h/mh.h>
 
 
 void
-print_sw (char *substr, struct swit *swp, char *prefix)
+print_sw (char *substr, struct swit *swp, char *prefix, FILE *fp)
 {
     int len, optno;
     register int i;
@@ -41,10 +43,11 @@ print_sw (char *substr, struct swit *swp, char *prefix)
 		    *cp++ = *cp1++;
 		*cp++ = ')';
 		while ((*cp++ = *cp1++));
-		printf ("  %s%s\n", prefix, buf);
+		fprintf (fp, "  %s%s\n", prefix, buf);
 	    } else {
 		if (!swp->minchars)
-		    printf(optno ? "  %s[no]%s\n" : "  %s%s\n", prefix, swp->sw);
+		    fprintf(fp, optno ? "  %s[no]%s\n" : "  %s%s\n",
+                            prefix, swp->sw);
 	    }
 	    if (optno)
 		swp++;	/* skip -noswitch */
