@@ -2,8 +2,6 @@
 /*
  * push.c -- push a fork into the background
  *
- * $Id: push.c,v 1.2 2002/07/02 22:09:14 kenh Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -11,7 +9,6 @@
 
 #include <h/mh.h>
 #include <h/signals.h>
-#include <signal.h>
 
 
 void
@@ -40,6 +37,9 @@ push(void)
 	    SIGNAL (SIGTTIN, SIG_IGN);
 	    SIGNAL (SIGTTOU, SIG_IGN);
 #endif
+
+	    unregister_for_removal(0);
+
 	    freopen ("/dev/null", "r", stdin);
 	    freopen ("/dev/null", "w", stdout);
 	    break;
