@@ -3,7 +3,9 @@
  * refile.c -- call the "fileproc" to refile the
  *          -- msg or draft into another folder
  *
- * $Id: refile.c,v 1.1.1.1 1999/04/30 18:08:34 doug Exp $
+ * This code is Copyright (c) 2002, by the authors of nmh.  See the
+ * COPYRIGHT file in the root directory of the nmh distribution for
+ * complete copyright information.
  */
 
 #include <h/mh.h>
@@ -13,15 +15,16 @@ int
 refile (char **arg, char *file)
 {
     pid_t pid;
-    register int vecp;
-    char *vec[MAXARGS];
+    int vecp;
+    char **vec;
+    char *program;
 
-    vecp = 0;
-    vec[vecp++] = r1bindex (fileproc, '/');
-    vec[vecp++] = "-nolink";	/* override bad .mh_profile defaults */
-    vec[vecp++] = "-nopreserve";
-    vec[vecp++] = "-file";
-    vec[vecp++] = file;
+    vec = argsplit(fileproc, &program, &vecp);
+
+    vec[vecp++] = getcpy("-nolink");	/* override bad .mh_profile defaults */
+    vec[vecp++] = getcpy("-nopreserve");
+    vec[vecp++] = getcpy("-file");
+    vec[vecp++] = getcpy(file);
 
     if (arg) {
 	while (*arg)
@@ -32,18 +35,19 @@ refile (char **arg, char *file)
     context_save();	/* save the context file */
     fflush(stdout);
 
-    switch (pid = vfork()) {
+    switch (pid = fork()) {
 	case -1: 
 	    advise ("fork", "unable to");
 	    return -1;
 
 	case 0: 
-	    execvp (fileproc, vec);
+	    execvp (program, vec);
 	    fprintf (stderr, "unable to exec ");
 	    perror (fileproc);
 	    _exit (-1);
 
 	default: 
+	    arglist_free(program, vec);
 	    return (pidwait (pid, -1));
     }
 }
