@@ -2,8 +2,6 @@
 /*
  * smatch.c -- match a switch (option)
  *
- * $Id: smatch.c,v 1.2 2002/07/02 22:09:14 kenh Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -31,16 +29,16 @@ smatch(char *string, struct swit *swp)
 	    continue;			/* no match */
 	for (sp = string; *sp == *tcp++;) {
 	    if (*sp++ == '\0')
-		return (tp - swp);	/* exact match */
+		return tp->swret;	/* exact match */
 	}
 	if (*sp) {
 	    if (*sp != ' ')
 		continue;		/* no match */
 	    if (*--tcp == '\0')
-		return (tp - swp);	/* exact match */
+		return tp->swret;	/* exact match */
 	}
 	if (firstone == UNKWNSW)
-	    firstone = tp - swp;
+	    firstone = tp->swret;
 	else
 	    firstone = AMBIGSW;
     }
