@@ -2,8 +2,6 @@
 /*
  * uprf.c -- "unsigned" lexical prefix
  *
- * $Id: uprf.c,v 1.2 2002/07/02 22:09:14 kenh Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -11,9 +9,6 @@
 
 #include <h/mh.h>
 
-#define TO_LOWER 040
-#define NO_MASK  000
-
 
 int
 uprf (char *c1, char *c2)
@@ -25,16 +20,11 @@ uprf (char *c1, char *c2)
 
     while ((c = *c2++))
     {
-#ifdef LOCALE
 	c &= 0xff;
 	mask = *c1 & 0xff;
 	c = (isalpha(c) && isupper(c)) ? tolower(c) : c;
 	mask = (isalpha(mask) && isupper(mask)) ? tolower(mask) : mask;
 	if (c != mask)
-#else
-	mask = (isalpha(c) && isalpha(*c1)) ?  TO_LOWER : NO_MASK;
-	if ((c | mask) != (*c1 | mask))
-#endif
 	    return 0;
 	else
 	    c1++;
