@@ -9,6 +9,7 @@
 
 #include <h/mh.h>
 #include <h/aliasbr.h>
+#include <h/addrsbr.h>
 #include <h/utils.h>
 #include <grp.h>
 #include <pwd.h>
@@ -25,7 +26,7 @@ struct home *hometail = NULL;
 /*
  * prototypes
  */
-int alias (char *); 
+int alias (char *);
 int akvisible (void);
 void init_pw (void);
 char *akresult (struct aka *);
@@ -34,7 +35,7 @@ char *akerror (int);
 
 static  char *akval (struct aka *, char *);
 static int aleq (char *, char *);
-static char *scanp (unsigned char *);
+static char *scanp (char *);
 static char *getp (char *);
 static char *seekp (char *, char *, char **);
 static int addfile (struct aka *, char *);
@@ -45,7 +46,6 @@ static char *getalias (char *);
 static void add_aka (struct aka *, char *);
 static struct aka *akalloc (char *);
 static struct home *hmalloc (struct passwd *);
-struct home *seek_home (char *);
 
 
 /* Do mh alias substitution on 's' and return the results. */
@@ -104,9 +104,51 @@ akval (struct aka *ak, char *s)
     if (!s)
 	return s;			/* XXX */
 
-    for (; ak; ak = ak->ak_next)
-	if (aleq (s, ak->ak_name))
+    /* It'd be tempting to check for a trailing semicolon and remove
+       it.  But that would break the EXMH alias parser on what would
+       then be valid expressions:
+       http://lists.gnu.org/archive/html/nmh-workers/2012-10/msg00039.html
+     */
+
+    for (; ak; ak = ak->ak_next) {
+	if (aleq (s, ak->ak_name)) {
 	    return akresult (ak);
+	} else if (strchr (s, ':')) {
+	    /* The first address in a blind list will contain the
+	       alias name, so try to match, but just with just the
+	       address (not including the list name).  If there's a
+	       match, then replace the alias part with its
+	       expansion. */
+
+	    char *name = getname (s);
+	    char *cp = NULL;
+
+	    if (name) {
+		/* s is of the form "Blind list: address".  If address
+		   is an alias, expand it. */
+		struct mailname *mp = getm (name, NULL, 0, NULL, 0);
+
+		if (mp	&&  mp->m_ingrp) {
+		    char *gname = add (mp->m_gname, NULL);
+
+		    if (gname  &&  aleq (name, ak->ak_name)) {
+			/* Will leak cp. */
+			cp = concat (gname, akresult (ak), NULL);
+			free (gname);
+		    }
+		}
+
+		mnfree (mp);
+	    }
+
+	    /* Need to flush getname after use. */
+	    while (getname ("")) continue;
+
+	    if (cp) {
+		return cp;
+	    }
+	}
+    }
 
     return getcpy (s);
 }
@@ -277,9 +319,9 @@ akerror (int i)
 
 
 static char *
-scanp (unsigned char *p)
+scanp (char *p)
 {
-    while (isspace (*p))
+    while (isspace ((unsigned char) *p))
 	p++;
     return p;
 }
@@ -288,10 +330,10 @@ scanp (unsigned char *p)
 static char *
 getp (char *p)
 {
-    register unsigned char  *cp = scanp (p);
+    char  *cp = scanp (p);
 
     p = cp;
-    while (!isspace (*cp) && *cp)
+    while (!isspace ((unsigned char) *cp) && *cp)
 	cp++;
     *cp = 0;
 
@@ -302,10 +344,10 @@ getp (char *p)
 static char *
 seekp (char *p, char *c, char **a)
 {
-    register unsigned char *cp;
+    char *cp;
 
     p = cp = scanp (p);
-    while (!isspace (*cp) && *cp && *cp != ':' && *cp != ';')
+    while (!isspace ((unsigned char) *cp) && *cp && *cp != ':' && *cp != ';')
 	cp++;
     *c = *cp;
     *cp++ = 0;
@@ -350,29 +392,20 @@ addgroup (struct aka *ak, char *grp)
 	return 0;
     }
 
-#ifndef DBMPWD
-    if (homehead == NULL)
-	init_pw ();
-#endif /* DBMPWD */
-
     while ((gp = *gr->gr_mem++))
-#ifdef DBMPWD
     {
 	struct passwd *pw;
-#endif /* DBMPWD */
 	for (hm = homehead; hm; hm = hm->h_next)
 	    if (!strcmp (hm->h_name, gp)) {
 		add_aka (ak, hm->h_name);
 		break;
 	    }
-#ifdef DBMPWD
         if ((pw = getpwnam(gp)))
 	{
 		hmalloc(pw);
 		add_aka (ak, gp);
 	}
     }
-#endif /* DBMPWD */
 
     return 1;
 }
@@ -396,10 +429,7 @@ addmember (struct aka *ak, char *grp)
 	return 0;
     }
 
-#ifndef DBMPWD
-    if (homehead == NULL)
-#endif /* DBMPWD */
-	init_pw ();
+    init_pw ();
 
     for (hm = homehead; hm; hm = hm->h_next)
 	if (hm->h_gid == gid)
@@ -415,15 +445,13 @@ addall (struct aka *ak)
     int noshell = NoShell == NULL || *NoShell == 0;
     register struct home *hm;
 
-#ifndef DBMPWD
-    if (homehead == NULL)
-#endif /* DBMPWD */
-	init_pw ();
+    init_pw ();
+
     if (Everyone < 0)
 	Everyone = EVERYONE;
 
     for (hm = homehead; hm; hm = hm->h_next)
-	if (hm->h_uid > Everyone
+        if ((int) hm->h_uid > Everyone
 		&& (noshell || strcmp (hm->h_shell, NoShell)))
 	    add_aka (ak, hm->h_name);
 
@@ -434,7 +462,7 @@ addall (struct aka *ak)
 static char *
 getalias (char *addrs)
 {
-    register unsigned char *pp, *qp;
+    char *pp, *qp;
     static char *cp = NULL;
 
     if (cp == NULL)
@@ -443,17 +471,20 @@ getalias (char *addrs)
 	if (*cp == 0)
 	    return (cp = NULL);
 
-    for (pp = cp; isspace (*pp); pp++)
+    /* Remove leading any space from the address. */
+    for (pp = cp; isspace ((unsigned char) *pp); pp++)
 	continue;
     if (*pp == 0)
 	return (cp = NULL);
+    /* Find the end of the address. */
     for (qp = pp; *qp != 0 && *qp != ','; qp++)
 	continue;
+    /* Set cp to point to the remainder of the addresses. */
     if (*qp == ',')
 	*qp++ = 0;
     for (cp = qp, qp--; qp > pp; qp--)
 	if (*qp != 0) {
-	    if (isspace (*qp))
+	    if (isspace ((unsigned char) *qp))
 		*qp = 0;
 	    else
 		break;
@@ -487,28 +518,24 @@ void
 init_pw (void)
 {
     register struct passwd  *pw;
-#ifdef DBMPWD
     static int init;
   
     if (!init)
     {
-          /* if the list has yet to be initialized */
-	    /* zap the list, and rebuild from scratch */
-	    homehead=NULL;
-	    hometail=NULL;
-	    init++;
-#endif /* DBMPWD */
+	/* if the list has yet to be initialized */
+	/* zap the list, and rebuild from scratch */
+	homehead=NULL;
+	hometail=NULL;
+	init++;
 
-    setpwent ();
+	setpwent ();
 
-    while ((pw = getpwent ()))
-	if (!hmalloc (pw))
-	    break;
+	while ((pw = getpwent ()))
+	    if (!hmalloc (pw))
+		break;
 
-    endpwent ();
-#ifdef DBMPWD
+	endpwent ();
     }
-#endif /* DBMPWD */
 }
 
 
@@ -555,42 +582,3 @@ hmalloc (struct passwd *pw)
 
     return p;
 }
-
-
-struct home *
-seek_home (char *name)
-{
-    register struct home *hp;
-#ifdef DBMPWD
-    struct passwd *pw;
-    char lname[32];
-    unsigned char *c;
-    char *c1;
-#else  /* DBMPWD */
-
-    if (homehead == NULL)
-	init_pw ();
-#endif /* DBMPWD */
-
-    for (hp = homehead; hp; hp = hp->h_next)
-	if (!mh_strcasecmp (name, hp->h_name))
-	    return hp;
-
-#ifdef DBMPWD
-    /*
-     * The only place where there might be problems.
-     * This assumes that ALL usernames are kept in lowercase.
-     */
-    for (c = name, c1 = lname; *c && (c1 - lname < sizeof(lname) - 1); c++, c1++) {
-        if (isalpha(*c) && isupper(*c))
-	    *c1 = tolower (*c);
-	else
-	    *c1 = *c;
-    }
-    *c1 = '\0';
-    if ((pw = getpwnam(lname)))
-	return(hmalloc(pw));
-#endif /* DBMPWD */
-	
-    return NULL;
-}
