@@ -19,55 +19,41 @@
 
 #define	FORMAT	"%<{error}%{error}: %{text}%|%(putstr(proper{text}))%>"
 
-static struct swit switches[] = {
-#define	FORMSW	0
-    { "form formatfile", 0 },
-#define	FMTSW	1
-    { "format string", 5 },
-#define	NORMSW	2
-    { "normalize", 0 },
-#define	NNORMSW	3
-    { "nonormalize", 0 },
-#define	WIDTHSW	4
-    { "width columns", 0 },
-#define VERSIONSW 5
-    { "version", 0 },
-#define	HELPSW	6
-    { "help", 0 },
-    { NULL, 0 }
-};
+#define AP_SWITCHES \
+    X("form formatfile", 0, FORMSW) \
+    X("format string", 5, FMTSW) \
+    X("width columns", 0, WIDTHSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(AP);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(AP, switches);
+#undef X
 
 static struct format *fmt;
 
 static int dat[5];
 
-/*
- * prototypes
- */
-int sc_width (void);  /* from termsbr.c */
-
 /*
  * static prototypes
  */
-static int process (char *, int, int);
+static int process (char *, int);
 
 
 int
 main (int argc, char **argv)
 {
-    int addrp = 0, normalize = AD_HOST;
+    int addrp = 0;
     int width = 0, status = 0;
     char *cp, *form = NULL, *format = NULL, *nfs;
     char buf[BUFSIZ], **argp;
     char **arguments, *addrs[NADDRS];
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     mts_init (invo_name);
     arguments = getarguments (invo_name, argc, argv, 1);
@@ -87,10 +73,10 @@ main (int argc, char **argv)
 		    snprintf (buf, sizeof(buf), "%s [switches] addrs ...",
 			invo_name);
 		    print_help (buf, switches, 1);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version (invo_name);
-		    done (1);
+		    done (0);
 
 		case FORMSW: 
 		    if (!(form = *argp++) || *form == '-')
@@ -108,13 +94,6 @@ main (int argc, char **argv)
 			adios (NULL, "missing argument to %s", argp[-2]);
 		    width = atoi (cp);
 		    continue;
-
-		case NORMSW: 
-		    normalize = AD_HOST;
-		    continue;
-		case NNORMSW: 
-		    normalize = AD_NHST;
-		    continue;
 	    }
 	}
 	if (addrp > NADDRS)
@@ -137,8 +116,7 @@ main (int argc, char **argv)
     }
     if (width > WBUFSIZ)
 	width = WBUFSIZ;
-    fmt_norm = normalize;
-    fmt_compile (nfs, &fmt);
+    fmt_compile (nfs, &fmt, 1);
 
     dat[0] = 0;
     dat[1] = 0;
@@ -147,8 +125,9 @@ main (int argc, char **argv)
     dat[4] = 0;
 
     for (addrp = 0; addrs[addrp]; addrp++)
-	status += process (addrs[addrp], width, normalize);
+	status += process (addrs[addrp], width);
 
+    fmt_free (fmt, 1);
     done (status);
     return 1;
 }
@@ -161,7 +140,7 @@ struct pqpair {
 
 
 static int
-process (char *arg, int length, int norm)
+process (char *arg, int length)
 {
     int	status = 0;
     register char *cp;
@@ -175,7 +154,7 @@ process (char *arg, int length, int norm)
     while ((cp = getname (arg))) {
 	if ((p = (struct pqpair *) calloc ((size_t) 1, sizeof(*p))) == NULL)
 	    adios (NULL, "unable to allocate pqpair memory");
-	if ((mp = getm (cp, NULL, 0, norm, error)) == NULL) {
+	if ((mp = getm (cp, NULL, 0, error, sizeof(error))) == NULL) {
 	    p->pq_text = getcpy (cp);
 	    p->pq_error = getcpy (error);
 	    status++;
@@ -188,17 +167,26 @@ process (char *arg, int length, int norm)
     }
 
     for (p = pq.pq_next; p; p = q) {
-	FINDCOMP (cptr, "text");
-	if (cptr)
+	cptr = fmt_findcomp ("text");
+	if (cptr) {
+	    if (cptr->c_text)
+	    	free(cptr->c_text);
 	    cptr->c_text = p->pq_text;
-	FINDCOMP (cptr, "error");
-	if (cptr)
+	    p->pq_text = NULL;
+	}
+	cptr = fmt_findcomp ("error");
+	if (cptr) {
+	    if (cptr->c_text)
+	    	free(cptr->c_text);
 	    cptr->c_text = p->pq_error;
+	    p->pq_error = NULL;
+	}
 
-	fmt_scan (fmt, buffer, length, dat);
+	fmt_scan (fmt, buffer, sizeof buffer - 1, length, dat, NULL);
 	fputs (buffer, stdout);
 
-	free (p->pq_text);
+	if (p->pq_text)
+	    free (p->pq_text);
 	if (p->pq_error)
 	    free (p->pq_error);
 	q = p->pq_next;
