@@ -26,17 +26,19 @@
  */
 #define	NGRPS	100
 
-static struct swit switches[] = {
-#define	MAILSW         0
-    { "mail name", 0 },
-#define	SERCHSW        1
-    { "search directory", 0 },
-#define VERSIONSW      2
-    { "version", 0 },
-#define	HELPSW         3
-    { "help", 0 },
-    { NULL, 0 }
-};
+#define CONFLICT_SWITCHES \
+    X("mail name", 0, MAILSW) \
+    X("search directory", 0, SERCHSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(CONFLICT);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(CONFLICT, switches);
+#undef X
 
 static char *mail = NULL;
 static char *dirs[NDIRS];
@@ -66,14 +68,7 @@ main (int argc, char **argv)
     char *cp, **argp, **arguments;
     char buf[BUFSIZ], *akv[50];
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* foil search of user profile/context */
-    if (context_foil (NULL) == -1)
-	done (1);
+    if (nmh_init(argv[0], 0 /* use context_foil() */)) { return 1; }
 
     mts_init (invo_name);
     arguments = getarguments (invo_name, argc, argv, 0);
@@ -92,10 +87,10 @@ main (int argc, char **argv)
 		    snprintf (buf, sizeof(buf), "%s [switches] [aliasfiles ...]",
 			invo_name);
 		    print_help (buf, switches, 0);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case MAILSW: 
 		    if (!(cp = *argp++) || *cp == '-')
@@ -131,10 +126,6 @@ main (int argc, char **argv)
     grp_names ();
     grp_members ();
     grp_ids ();
-#ifdef	UCI
-    ldr_names ();
-    ldr_ship ();
-#endif	/* UCI */
     maildrops ();
 
     done (0);
@@ -368,7 +359,7 @@ setup (void)
 			close (fd);
 		    }
 		execlp (mailproc, r1bindex (mailproc, '/'),
-			mail, "-subject", invo_name, NULL);
+			mail, "-subject", invo_name, (void *) NULL);
 		adios (mailproc, "unable to exec ");
 
 	    default: 
@@ -379,169 +370,3 @@ setup (void)
 	}
     }
 }
-
-#ifdef UCI
-/*
- * UCI specific stuff for conflict
- */
-
-/* taken from <grpldr.h> */
-
-#define	GLDRS	"/admin/etc/GroupLeaders"
-
-struct grpldr {
-    char *gl_name;
-    char **gl_ldr;
-};
-
-int setglent (), endglent ();
-struct grpldr *getglent (), *getglnam ();
-
-
-/* taken from the getglent() routines */
-
-#define	MAXGLS	100
-
-static FILE *glp = NULL;
-static char line[BUFSIZ+1];
-static struct grpldr grpldr;
-static char *gl_ldr[MAXGLS + 1];
-
-
-setglent() {
-    if (glp == NULL)
-	glp = fopen (GLDRS, "r");
-    else
-	rewind (glp);
-
-    return (glp != NULL);
-}
-
-
-endglent() {
-    if (glp != NULL) {
-	fclose (glp);
-	glp = NULL;
-    }
-
-    return 1;
-}
-
-struct grpldr  *getglent () {
-    register char  *cp,
-                  **q;
-
-    if (glp == NULL && !setglent ())
-	return NULL;
-    if ((cp = fgets (line, BUFSIZ, glp)) == NULL)
-	return NULL;
-
-    grpldr.gl_name = cp;
-    grpldr.gl_ldr = q = gl_ldr;
-
-    while (*cp) {
-	while (*cp && !isspace (*cp))
-	    cp++;
-	while (*cp && isspace (*cp))
-	    *cp++ = '\0';
-	if (*cp == '\0')
-	    break;
-	if (q < gl_ldr + MAXGLS)
-	    *q++ = cp;
-	else
-	    break;
-    }
-    *q = NULL;
-
-    return (&grpldr);
-}
-
-struct grpldr  *getglnam (name)
-char   *name;
-{
-    register struct grpldr  *gl = NULL;
-
-    setglent ();
-    while (gl = getglent ())
-	if (strcmp (name, gl->gl_name) == 0)
-	    break;
-    endglent ();
-
-    return gl;
-}
-
-ldr_names () {
-    register int     gp,
-		     hit = 0;
-    char   *gldrs[NGRPS];
-    register struct grpldr  *gl;
-
-    gldrs[0] = NULL;
-    setglent ();
-    while (gl = getglent ()) {
-	if (getgrnam (gl->gl_name) == NULL) {
-	    setup ();
-	    fprintf (out, "unknown group %s in group leaders file\n",
-		    gl->gl_name);
-	    hit++;
-	}
-	for (gp = 0; gldrs[gp]; gp++)
-	    if (strcmp (gldrs[gp], gl->gl_name) == 0) {
-		setup ();
-		fprintf (out, "duplicate group %s in group leaders file\n",
-			gl->gl_name);
-		hit++;
-		break;
-	    }
-	if (gldrs[gp] == NULL)
-	    if (gp < NGRPS) {
-		gldrs[gp++] = getcpy (gl->gl_name);
-		gldrs[gp] = NULL;
-	    }
-	    else {
-		setup ();
-		fprintf (out, "more than %d groups in group leaders file%s\n",
-			" (time to recompile)", NGRPS - 1);
-		hit++;
-	    }
-    }
-    endglent ();
-
-    for (gp = 0; gldrs[gp]; gp++)
-	free (gldrs[gp]);
-
-    if (!hit && out && !mail)
-	fprintf (out, "all groups in group leaders file accounted for\n");
-}
-
-
-ldr_ship () {
-    register int     hit = 0;
-    register char  **cp,
-		   **dp;
-    register struct grpldr  *gl;
-
-    setglent ();
-    while (gl = getglent ())
-	for (cp = gl->gl_ldr; *cp; cp++) {
-	    if (!check (*cp)) {
-		setup ();
-		fprintf (out, "group %s has unknown leader %s\n",
-			gl->gl_name, *cp);
-		hit++;
-	    }
-
-	    for (dp = cp + 1; *dp; dp++)
-		if (strcmp (*cp, *dp) == 0) {
-		    setup ();
-		    fprintf (out, "group %s had duplicate leader %s\n",
-			    gl->gl_name, *cp);
-		    hit++;
-		}
-	}
-    endglent ();
-
-    if (!hit && out && !mail)
-	fprintf (out, "all group leaders accounted for\n");
-}
-#endif	/* UCI */
