@@ -2,8 +2,6 @@
 /*
  * dropsbr.c -- create/read/manipulate mail drops
  *
- * $Id: dropsbr.c,v 1.11 2008/04/05 18:41:38 pm215 Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -17,10 +15,6 @@
 #include <h/mts.h>
 #include <h/tws.h>
 
-#ifdef HAVE_ERRNO_H
-# include <errno.h>
-#endif
-
 #ifdef NTOHLSWAP
 # include <netinet/in.h>
 #else
@@ -46,32 +40,23 @@ static int map_open (char *, int);
 int
 mbx_open (char *file, int mbx_style, uid_t uid, gid_t gid, mode_t mode)
 {
-    int j, count, fd;
+    int j, count, fd = NOTOK;
     struct stat st;
 
     j = 0;
 
     /* attempt to open and lock file */
     for (count = 4; count > 0; count--) {
-	if ((fd = lkopen (file, O_RDWR | O_CREAT | O_NONBLOCK, mode)) == NOTOK) {
-	    switch (errno) {
-#if defined(FCNTL_LOCKING) || defined(LOCKF_LOCKING)
-		case EACCES:
-		case EAGAIN:
-#endif
-
-#ifdef FLOCK_LOCKING
-		case EWOULDBLOCK:
-#endif
-		case ETXTBSY: 
-		    j = errno;
-		    sleep (5);
-		    break;
-
-		default: 
-		    /* just return error */
-		    return NOTOK;
-	    }
+        int failed_to_lock = 0;
+	if ((fd = lkopenspool (file, O_RDWR | O_CREAT |
+			       O_NONBLOCK, mode, &failed_to_lock)) == NOTOK) {
+            if (failed_to_lock) {
+                j = errno;
+                sleep (5);
+                continue;
+            } else {
+                return NOTOK;
+            }
 	}
 
 	/* good file descriptor */
@@ -145,13 +130,12 @@ mbx_chk_mbox (int fd)
 static int
 mbx_chk_mmdf (int fd)
 {
-    size_t count;
+    ssize_t count;
     char ldelim[BUFSIZ];
 
     count = strlen (mmdlm2);
 
-    /* casting -count to off_t, seem to break FreeBSD 2.2.6 */
-    if (lseek (fd, (long) (-count), SEEK_END) == (off_t) NOTOK)
+    if (lseek (fd, -count, SEEK_END) == (off_t) NOTOK)
 	return NOTOK;
     if (read (fd, ldelim, count) != count)
 	return NOTOK;
@@ -312,6 +296,10 @@ mbx_copy (char *mailbox, int mbx_style, int md, int fd,
 	    }
 		    
 	    while ((i = read (fd, buffer, sizeof(buffer))) > 0) {
+                /* valgrind noticed that stringdex depends on null
+                   termination. */
+                buffer[i] = '\0';
+
 		for (j = 0;
 			(j = stringdex (mmdlm1, buffer)) >= 0;
 			buffer[j]++)
@@ -469,8 +457,9 @@ mbx_size (int md, off_t start, off_t stop)
 int
 mbx_close (char *mailbox, int md)
 {
-    lkclose (md, mailbox);
-    return OK;
+    if (lkclosespool (md, mailbox) == 0)
+        return OK;
+    return NOTOK;
 }
 
 
@@ -522,7 +511,8 @@ map_read (char *file, long pos, struct drop **drops, int noisy)
     memcpy((char *) dp, (char *) mp, sizeof(*dp));
 
     lseek (md, (off_t) sizeof(*mp), SEEK_SET);
-    if ((i = read (md, (char *) (dp + 1), msgp * sizeof(*dp))) < sizeof(*dp)) {
+    if ((i = read (md, (char *) (dp + 1), msgp * sizeof(*dp))) <
+        (int) sizeof(*dp)) {
 	i = 0;
 	free ((char *) dp);
     } else {
@@ -567,7 +557,7 @@ map_write (char *mailbox, int md, int id, long last, off_t start,
 	clear = 1;
 
     if (!clear && map_chk (file, fd, &d1, pos, noisy)) {
-	unlink (file);
+	(void) m_unlink (file);
 	mbx_close (file, fd);
 	if ((fd = map_open (file, md)) == NOTOK)
 	    return NOTOK;
@@ -655,7 +645,7 @@ map_open (char *file, int md)
     mode_t mode;
     struct stat st;
 
-    mode = fstat (md, &st) != NOTOK ? (mode_t) (st.st_mode & 0777) : m_gmprot ();
+    mode = fstat (md, &st) != NOTOK ? (int) (st.st_mode & 0777) : m_gmprot ();
     return mbx_open (file, OTHER_FORMAT, st.st_uid, st.st_gid, mode);
 }
 
@@ -663,7 +653,7 @@ map_open (char *file, int md)
 int
 map_chk (char *file, int fd, struct drop *dp, long pos, int noisy)
 {
-    long count;
+    ssize_t count;
     struct drop d, tmpd;
     register struct drop *dl;
 
@@ -704,7 +694,7 @@ map_chk (char *file, int fd, struct drop *dp, long pos, int noisy)
     }
 
     dl = &d;
-    count = (long) strlen (mmdlm2);
+    count = strlen (mmdlm2);
     lseek (fd, (off_t) (dp->d_id * sizeof(*dp)), SEEK_SET);
     if (read (fd, (char *) dl, sizeof(*dl)) != sizeof(*dl)
 	    || (ntohl(dl->d_stop) != dp->d_stop
