@@ -4,68 +4,48 @@
  *             -- push/pop a folder onto/from the folder stack
  *             -- list the folder stack
  *
- * $Id: folder.c,v 1.13 2007/11/04 11:54:33 jjr Exp $
- *
- * This code is Copyright (c) 2002, by the authors of nmh.  See the
+ * This code is Copyright (c) 2002, 2008, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
  */
 
 #include <h/mh.h>
+#include <h/crawl_folders.h>
 #include <h/utils.h>
-#include <errno.h>
 
-static struct swit switches[] = {
-#define	ALLSW           0
-    { "all", 0 },
-#define NALLSW		1
-    { "noall", 0 },
-#define	CREATSW         2
-    { "create", 0 },
-#define	NCREATSW        3
-    { "nocreate", 0 },
-#define	FASTSW          4
-    { "fast", 0 },
-#define	NFASTSW         5
-    { "nofast", 0 },
-#define	HDRSW           6
-    { "header", 0 },
-#define	NHDRSW          7
-    { "noheader", 0 },
-#define	PACKSW          8
-    { "pack", 0 },
-#define	NPACKSW         9
-    { "nopack", 0 },
-#define	VERBSW         10
-    { "verbose", 0 },
-#define	NVERBSW        11
-    { "noverbose", 0 },
-#define	RECURSW        12
-    { "recurse", 0 },
-#define	NRECRSW        13
-    { "norecurse", 0 },
-#define	TOTALSW        14
-    { "total", 0 },
-#define	NTOTLSW        15
-    { "nototal", 0 },
-#define	LISTSW         16
-    { "list", 0 },
-#define	NLISTSW        17
-    { "nolist", 0 },
-#define	PRNTSW         18
-    { "print", 0 },
-#define	NPRNTSW        19
-    { "noprint", -4 },
-#define	PUSHSW         20
-    { "push", 0 },
-#define	POPSW          21
-    { "pop", 0 },
-#define VERSIONSW      22
-    { "version", 0 },
-#define	HELPSW         23
-    { "help", 0 },
-    { NULL, 0 }
-};
+#define FOLDER_SWITCHES \
+    X("all", 0, ALLSW) \
+    X("noall", 0, NALLSW) \
+    X("create", 0, CREATSW) \
+    X("nocreate", 0, NCREATSW) \
+    X("fast", 0, FASTSW) \
+    X("nofast", 0, NFASTSW) \
+    X("header", 0, HDRSW) \
+    X("noheader", 0, NHDRSW) \
+    X("pack", 0, PACKSW) \
+    X("nopack", 0, NPACKSW) \
+    X("verbose", 0, VERBSW) \
+    X("noverbose", 0, NVERBSW) \
+    X("recurse", 0, RECURSW) \
+    X("norecurse", 0, NRECRSW) \
+    X("total", 0, TOTALSW) \
+    X("nototal", 0, NTOTLSW) \
+    X("list", 0, LISTSW) \
+    X("nolist", 0, NLISTSW) \
+    X("print", 0, PRNTSW) \
+    X("noprint", -4, NPRNTSW) \
+    X("push", 0, PUSHSW) \
+    X("pop", 0, POPSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(FOLDER);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(FOLDER, switches);
+#undef X
 
 static int fshort   = 0;	/* output only folder names                 */
 static int fcreat   = 0;	/* should we ask to create new folders?     */
@@ -78,22 +58,10 @@ static int all      = 0;	/* should we output all folders             */
 
 static int total_folders = 0;	/* total number of folders                  */
 
-static int start = 0;
-static int foldp = 0;
-
 static char *nmhdir;
 static char *stack = "Folder-Stack";
 static char folder[BUFSIZ];
 
-#define NUMFOLDERS 100
-
-/*
- * This is how many folders we currently can hold in the array
- * `folds'.  We increase this amount by NUMFOLDERS at a time.
- */
-static int maxfolders;
-static char **folds;
-
 /*
  * Structure to hold information about
  * folders as we scan them.
@@ -118,13 +86,10 @@ static int maxFolderInfo;
 /*
  * static prototypes
  */
-static void dodir (char *);
 static int get_folder_info (char *, char *);
+static crawl_callback_t get_folder_info_callback;
 static void print_folders (void);
 static int sfold (struct msgs *, char *);
-static void addir (char *);
-static void addfold (char *);
-static int compare (char *, char *);
 static void readonly_folders (void);
 
 
@@ -136,13 +101,7 @@ main (int argc, char **argv)
     char *cp, *dp, *msg = NULL, *argfolder = NULL;
     char **ap, **argp, buf[BUFSIZ], **arguments;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     /*
      * If program was invoked with name ending
@@ -167,10 +126,10 @@ main (int argc, char **argv)
 		    snprintf (buf, sizeof(buf), "%s [+folder] [msg] [switches]",
 			invo_name);
 		    print_help (buf, switches, 1);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case ALLSW: 
 		    all = 1;
@@ -350,17 +309,16 @@ main (int argc, char **argv)
 	    done (0);
     }
 
-    /* Allocate initial space to record folder names */
-    maxfolders = NUMFOLDERS;
-    folds = mh_xmalloc (maxfolders * sizeof(char *));
-
     /* Allocate initial space to record folder information */
-    maxFolderInfo = NUMFOLDERS;
+    maxFolderInfo = CRAWL_NUMFOLDERS;
     fi = mh_xmalloc (maxFolderInfo * sizeof(*fi));
 
     /*
      * Scan the folders
      */
+    /* change directory to base of nmh directory for crawl_folders */
+    if (chdir (nmhdir) == NOTOK)
+	adios (nmhdir, "unable to change directory to");
     if (all || ftotal > 0) {
 	/*
 	 * If no folder is given, do them all
@@ -370,7 +328,7 @@ main (int argc, char **argv)
 		admonish (NULL, "no folder given for message %s", msg);
 	    readonly_folders (); /* do any readonly folders */
 	    strncpy (folder, (cp = context_find (pfolder)) ? cp : "", sizeof(folder));
-	    dodir (".");
+	    crawl_folders (".", get_folder_info_callback, NULL);
 	} else {
 	    strncpy (folder, argfolder, sizeof(folder));
 	    if (get_folder_info (argfolder, msg)) {
@@ -382,7 +340,7 @@ main (int argc, char **argv)
 	     * we still need to list all level-1 sub-folders.
 	     */
 	    if (!frecurse)
-		dodir (folder);
+		crawl_folders (folder, get_folder_info_callback, NULL);
 	}
     } else {
 	strncpy (folder, argfolder ? argfolder : getfolder (1), sizeof(folder));
@@ -409,40 +367,10 @@ main (int argc, char **argv)
     return 1;
 }
 
-/*
- * Base routine for scanning a folder
- */
-
-static void
-dodir (char *dir)
-{
-    int i;
-    int os = start;
-    int of = foldp;
-    char buffer[BUFSIZ];
-
-    start = foldp;
-
-    /* change directory to base of nmh directory */
-    if (chdir (nmhdir) == NOTOK)
-	adios (nmhdir, "unable to change directory to");
-
-    addir (strncpy (buffer, dir, sizeof(buffer)));
-
-    for (i = start; i < foldp; i++) {
-	get_folder_info (folds[i], NULL);
-	fflush (stdout);
-    }
-
-    start = os;
-    foldp = of;
-}
-
 static int
-get_folder_info (char *fold, char *msg)
+get_folder_info_body (char *fold, char *msg, boolean *crawl_children)
 {
     int	i, retval = 1;
-    char *mailfile;
     struct msgs *mp = NULL;
 
     i = total_folders++;
@@ -452,7 +380,7 @@ get_folder_info (char *fold, char *msg)
      * for folder information
      */
     if (total_folders >= maxFolderInfo) {
-	maxFolderInfo += NUMFOLDERS;
+	maxFolderInfo += CRAWL_NUMFOLDERS;
 	fi = mh_xrealloc (fi, maxFolderInfo * sizeof(*fi));
     }
 
@@ -464,46 +392,66 @@ get_folder_info (char *fold, char *msg)
     fi[i].others = 0;
     fi[i].error  = 0;
 
-    mailfile = m_maildir (fold);
-
-    if (!chdir (mailfile)) {
-	if ((ftotal > 0) || !fshort || msg || fpack) {
-	    /*
-	     * create message structure and get folder info
-	     */
-	    if (!(mp = folder_read (fold))) {
-		admonish (NULL, "unable to read folder %s", fold);
-		return 0;
-	    }
-
-	    /* set the current message */
-	    if (msg && !sfold (mp, msg))
-		retval = 0;
-
-	    if (fpack) {
-		if (folder_pack (&mp, fverb) == -1)
-		    done (1);
-		seq_save (mp);		/* synchronize the sequences */
-		context_save ();	/* save the context file     */
-	    }
-
-	    /* record info for this folder */
-	    if ((ftotal > 0) || !fshort) {
-		fi[i].nummsg = mp->nummsg;
-		fi[i].curmsg = mp->curmsg;
-		fi[i].lowmsg = mp->lowmsg;
-		fi[i].hghmsg = mp->hghmsg;
-		fi[i].others = other_files (mp);
-	    }
-
-	    folder_free (mp); /* free folder/message structure */
+    if ((ftotal > 0) || !fshort || msg || fpack) {
+	/*
+	 * create message structure and get folder info
+	 */
+	if (!(mp = folder_read (fold, 1))) {
+	    admonish (NULL, "unable to read folder %s", fold);
+	    return 0;
 	}
-    } else {
-	fi[i].error = 1;
+
+	/* set the current message */
+	if (msg && !sfold (mp, msg))
+	    retval = 0;
+
+	if (fpack) {
+	    if (folder_pack (&mp, fverb) == -1)
+		done (1);
+	    seq_save (mp);		/* synchronize the sequences */
+	    context_save ();	/* save the context file     */
+	}
+
+	/* record info for this folder */
+	if ((ftotal > 0) || !fshort) {
+	    fi[i].nummsg = mp->nummsg;
+	    fi[i].curmsg = mp->curmsg;
+	    fi[i].lowmsg = mp->lowmsg;
+	    fi[i].hghmsg = mp->hghmsg;
+	    fi[i].others = other_files (mp);
+	}
+
+	folder_free (mp); /* free folder/message structure */
+    }
+
+    *crawl_children = (frecurse && (fshort || fi[i].others)
+		       && (fi[i].error == 0));
+    return retval;
+}
+
+static boolean
+get_folder_info_callback (char *fold, void *baton)
+{
+    boolean crawl_children;
+    NMH_UNUSED (baton);
+
+    get_folder_info_body (fold, NULL, &crawl_children);
+    fflush (stdout);
+    return crawl_children;
+}
+
+static int
+get_folder_info (char *fold, char *msg)
+{
+    boolean crawl_children;
+    int retval;
+
+    retval = get_folder_info_body (fold, msg, &crawl_children);
+
+    if (crawl_children) {
+	crawl_folders (fold, get_folder_info_callback, NULL);
     }
 
-    if (frecurse && (fshort || fi[i].others) && (fi[i].error == 0))
-	dodir (fold);
     return retval;
 }
 
@@ -663,106 +611,6 @@ sfold (struct msgs *mp, char *msg)
 }
 
 
-static void
-addir (char *name)
-{
-    int nlink;
-    char *base, *cp;
-    struct stat st;
-    struct dirent *dp;
-    DIR * dd;
-
-    cp = name + strlen (name);
-    *cp++ = '/';
-    *cp = '\0';
-
-    /*
-     * A hack to skip over a leading
-     * "./" in folder names.
-     */
-    base = strcmp (name, "./") ? name : name + 2;
-
-   /* short-cut to see if directory has any sub-directories */
-    if (stat (name, &st) != -1 && st.st_nlink == 2)
-        return;
- 
-    if (!(dd = opendir (name))) {
-	admonish (name, "unable to read directory ");
-	return;
-    }
-
-    /*
-     * Keep track of the number of directories we've seen
-     * so we can quit stat'ing early, if we've seen them all.
-     */
-    nlink = st.st_nlink;
-
-    while (nlink && (dp = readdir (dd))) {
-	if (!strcmp (dp->d_name, ".") || !strcmp (dp->d_name, "..")) {
-	    nlink--;
-	    continue;
-	}
-	if (cp + NLENGTH(dp) + 2 >= name + BUFSIZ)
-	    continue;
-	strcpy (cp, dp->d_name);
-	if (stat (name, &st) != -1 && S_ISDIR(st.st_mode)) {
-	    /*
-	     * Check if this was really a symbolic link pointing at
-	     * a directory.  If not, then decrement link count.
-	     */
-	    if (lstat (name, &st) == -1)
-		nlink--;
-	    addfold (base);
-	}
-    }
-
-    closedir (dd);
-    *--cp = '\0';
-}
-
-/*
- * Add the folder name into the
- * list in a sorted fashion.
- */
-
-static void
-addfold (char *fold)
-{
-    register int i, j;
-    register char *cp;
-
-    /* if necessary, reallocate the space for folder names */
-    if (foldp >= maxfolders) {
-	maxfolders += NUMFOLDERS;
-	folds = mh_xrealloc (folds, maxfolders * sizeof(char *));
-    }
-
-    cp = getcpy (fold);
-    for (i = start; i < foldp; i++)
-	if (compare (cp, folds[i]) < 0) {
-	    for (j = foldp - 1; j >= i; j--)
-		folds[j + 1] = folds[j];
-	    foldp++;
-	    folds[i] = cp;
-	    return;
-	}
-
-    folds[foldp++] = cp;
-}
-
-
-static int
-compare (char *s1, char *s2)
-{
-    register int i;
-
-    while (*s1 || *s2)
-	if ((i = *s1++ - *s2++))
-	    return i;
-
-    return 0;
-}
-
 /*
  * Do the read only folders
  */
