@@ -29,25 +29,13 @@
 #include <h/mh.h>
 #include <h/utils.h>
 #include <fcntl.h>
-
-#ifdef POP
-# include <h/dropsbr.h>
-# include <h/popsbr.h>
-#endif
-
+#include <h/dropsbr.h>
+#include <h/popsbr.h>
 #include <h/fmt_scan.h>
 #include <h/scansbr.h>
 #include <h/signals.h>
 #include <h/tws.h>
 #include <h/mts.h>
-#include <errno.h>
-#include <signal.h>
-
-#ifndef	POP
-# define POPminc(a) (a)
-#else
-# define POPminc(a)  0
-#endif
 
 #ifndef CYRUS_SASL
 # define SASLminc(a) (a)
@@ -55,55 +43,39 @@
 # define SASLminc(a)  0
 #endif
 
-static struct swit switches[] = {
-#define	AUDSW                      0
-    { "audit audit-file", 0 },
-#define	NAUDSW                     1
-    { "noaudit", 0 },
-#define	CHGSW                      2
-    { "changecur", 0 },
-#define	NCHGSW                     3
-    { "nochangecur", 0 },
-#define	FILESW                     4
-    { "file name", 0 },
-#define	FORMSW                     5
-    { "form formatfile", 0 },
-#define	FMTSW                      6
-    { "format string", 5 },
-#define	HOSTSW                     7
-    { "host hostname", POPminc (-4) },
-#define	USERSW                     8
-    { "user username", POPminc (-4) },
-#define	PACKSW                     9
-    { "pack file", POPminc (-4) },
-#define	NPACKSW                   10
-    { "nopack", POPminc (-6) },
-#define PORTSW			  11
-    { "port name/number", POPminc (-4) },
-#define	SILSW                     12
-    { "silent", 0 },
-#define	NSILSW                    13
-    { "nosilent", 0 },
-#define	TRNCSW                    14
-    { "truncate", 0 },
-#define	NTRNCSW                   15
-    { "notruncate", 0 },
-#define	WIDTHSW                   16
-    { "width columns", 0 },
-#define VERSIONSW                 17
-    { "version", 0 },
-#define	HELPSW                    18
-    { "help", 0 },
-#define SNOOPSW                   19
-    { "snoop", -5 },
-#define SASLSW                    20
-    { "sasl", SASLminc(-4) },
-#define SASLMECHSW                21
-    { "saslmech", SASLminc(-8) },
-#define PROXYSW                   22
-    { "proxy command", POPminc(-5) },
-    { NULL, 0 }
-};
+#define INC_SWITCHES \
+    X("audit audit-file", 0, AUDSW) \
+    X("noaudit", 0, NAUDSW) \
+    X("changecur", 0, CHGSW) \
+    X("nochangecur", 0, NCHGSW) \
+    X("file name", 0, FILESW) \
+    X("form formatfile", 0, FORMSW) \
+    X("format string", 5, FMTSW) \
+    X("host hostname", 0, HOSTSW) \
+    X("user username", 0, USERSW) \
+    X("pack file", 0, PACKSW) \
+    X("nopack", 0, NPACKSW) \
+    X("port name/number", 0, PORTSW) \
+    X("silent", 0, SILSW) \
+    X("nosilent", 0, NSILSW) \
+    X("truncate", 0, TRNCSW) \
+    X("notruncate", 0, NTRNCSW) \
+    X("width columns", 0, WIDTHSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+    X("snoop", -5, SNOOPSW) \
+    X("sasl", SASLminc(-4), SASLSW) \
+    X("nosasl", SASLminc(-6), NOSASLSW) \
+    X("saslmech", SASLminc(-8), SASLMECHSW) \
+    X("proxy command", 0, PROXYSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(INC);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(INC, switches);
+#undef X
 
 /*
  * flags for the mail source
@@ -119,7 +91,6 @@ static struct Maildir_entry {
 static int num_maildir_entries = 0;
 static int snoop = 0;
 
-#ifdef POP
 extern char response[];
 
 static int size;
@@ -127,7 +98,6 @@ static long pos;
 
 static int mbx_style = MMDF_FORMAT;
 static int pd = NOTOK;
-#endif /* POP */
 
 static long start;
 static long stop;
@@ -139,13 +109,9 @@ static FILE *pf = NULL;
  * privilege ops into macros.
  * *GROUPPRIVS() is related to handling the setgid MAIL property,
  * and only applies if MAILGROUP is defined.
- * *USERPRIVS() is related to handling the setuid root property,
- * and only applies if POP is defined [why does POP => setuid root?]
  * Basically, SAVEGROUPPRIVS() is called right at the top of main()
  * to initialise things, and then DROPGROUPPRIVS() and GETGROUPPRIVS()
- * do the obvious thing. TRYDROPGROUPPRIVS() has to be safe to call
- * before DROPUSERPRIVS() is called [this is needed because setgid()
- * sets both effective and real uids if euid is root.]
+ * do the obvious thing.
  *
  * There's probably a better implementation if we're allowed to use
  * BSD-style setreuid() rather than using POSIX saved-ids.
@@ -162,15 +128,7 @@ static FILE *pf = NULL;
  */
 #ifdef MAILGROUP
 static int return_gid;
-#ifndef POP
-/* easy case; we're not setuid root, so can drop group privs
- * immediately.
- */
 #define TRYDROPGROUPPRIVS() DROPGROUPPRIVS()
-#else /* POP ie we are setuid root */
-#define TRYDROPGROUPPRIVS() \
-if (geteuid() != 0) DROPGROUPPRIVS()
-#endif
 #define DROPGROUPPRIVS() setgid(getgid())
 #define GETGROUPPRIVS() setgid(return_gid)
 #define SAVEGROUPPRIVS() return_gid = getegid()
@@ -193,11 +151,9 @@ static FILE *in;
 char *map_name(char *);
 
 static void inc_done(int) NORETURN;
-#ifdef POP
 static int pop_action(char *);
 static int pop_pack(char *);
 static int map_count(void);
-#endif
 
 int
 maildir_srt(const void *va, const void *vb)
@@ -227,18 +183,11 @@ main (int argc, char **argv)
     struct msgs *mp = NULL;
     struct stat st, s1;
     FILE *aud = NULL;
-    char b[MAXPATHLEN + 1];
+    char b[PATH_MAX + 1];
     char *maildir_copy = NULL;	/* copy of mail directory because the static gets overwritten */
 
-#ifdef POP
     int nmsgs, nbytes;
-    char *pass = NULL;
     char *MAILHOST_env_variable;
-#endif
-
-#ifdef MHE
-    FILE *mhe = NULL;
-#endif
 
     done=inc_done;
 
@@ -248,19 +197,12 @@ main (int argc, char **argv)
     SAVEGROUPPRIVS();
     TRYDROPGROUPPRIVS();
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     mts_init (invo_name);
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
-#ifdef POP
     /*
      * Scheme is:
      *        use MAILHOST environment variable if present,
@@ -277,10 +219,6 @@ main (int argc, char **argv)
     if (pophost && *pophost)
 	host = pophost;
 
-    if ((cp = getenv ("MHPOPDEBUG")) && *cp)
-	snoop++;
-#endif /* POP */
-
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -293,10 +231,10 @@ main (int argc, char **argv)
 	    case HELPSW: 
 		snprintf (buf, sizeof(buf), "%s [+folder] [switches]", invo_name);
 		print_help (buf, switches, 1);
-		done (1);
+		done (0);
 	    case VERSIONSW:
 		print_version(invo_name);
-		done (1);
+		done (0);
 
 	    case AUDSW: 
 		if (!(cp = *argp++) || *cp == '-')
@@ -371,7 +309,7 @@ main (int argc, char **argv)
 		continue;
 
 	    case PORTSW:
-		if (!(host = *argp++) || *port == '-')
+		if (!(port = *argp++) || *port == '-')
 		    adios (NULL, "missing argument to %s", argp[-2]);
 		continue;
 
@@ -381,18 +319,11 @@ main (int argc, char **argv)
 		continue;
 
 	    case PACKSW:
-#ifndef	POP
-		if (!(cp = *argp++) || *cp == '-')
-		    adios (NULL, "missing argument to %s", argp[-2]);
-#else /* POP */
 		if (!(packfile = *argp++) || *packfile == '-')
 		    adios (NULL, "missing argument to %s", argp[-2]);
-#endif /* POP */
 		continue;
 	    case NPACKSW:
-#ifdef POP
 		packfile = NULL;
-#endif /* POP */
 		continue;
 
 	    case SNOOPSW:
@@ -402,6 +333,9 @@ main (int argc, char **argv)
 	    case SASLSW:
 		sasl++;
 		continue;
+	    case NOSASLSW:
+	    	sasl = 0;
+		continue;
 	
 	    case SASLMECHSW:
 		if (!(saslmech = *argp++) || *saslmech == '-')
@@ -426,10 +360,8 @@ main (int argc, char **argv)
     /* NOTE: above this point you should use TRYDROPGROUPPRIVS(),
      * not DROPGROUPPRIVS().
      */
-#ifdef POP
     if (host && !*host)
 	host = NULL;
-#endif /* POP */
 
     /* guarantee dropping group priveleges; we might not have done so earlier */
     DROPGROUPPRIVS();
@@ -439,31 +371,24 @@ main (int argc, char **argv)
      */
     if (from)
 	inc_type = INC_FILE;
-#ifdef POP
     else if (host)
 	inc_type = INC_POP;
-#endif
     else
 	inc_type = INC_FILE;
 
-#ifdef POP
     /*
      * Are we getting the mail from
      * a POP server?
      */
     if (inc_type == INC_POP) {
-	if (user == NULL)
-	    user = getusername ();
-	if (sasl)
-	    pass = getusername ();
-	else
-	    ruserpass (host, &user, &pass);
+	struct nmh_creds creds = { 0, 0, 0 };
 
 	/*
 	 * initialize POP connection
 	 */
-	if (pop_init (host, port, user, pass, proxy, snoop, sasl,
-		      saslmech) == NOTOK)
+	nmh_get_credentials (host, user, sasl, &creds);
+	if (pop_init (host, port, creds.user, creds.password, proxy, snoop,
+		      sasl, saslmech) == NOTOK)
 	    adios (NULL, "%s", response);
 
 	/* Check if there are any messages */
@@ -475,7 +400,6 @@ main (int argc, char **argv)
 	    adios (NULL, "no mail to incorporate");
 	}
     }
-#endif /* POP */
 
     /*
      * We will get the mail from a file
@@ -550,11 +474,9 @@ main (int argc, char **argv)
 	newmail = cp;
     }
 
-#ifdef POP
     /* skip the folder setup */
     if ((inc_type == INC_POP) && packfile)
 	goto go_to_it;
-#endif /* POP */
 
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
@@ -579,12 +501,10 @@ main (int argc, char **argv)
 	adios (maildir, "unable to change directory to");
 
     /* read folder and create message structure */
-    if (!(mp = folder_read (folder)))
+    if (!(mp = folder_read (folder, 0)))
 	adios (NULL, "unable to read folder %s", folder);
 
-#ifdef POP
 go_to_it:
-#endif /* POP */
 
     if (inc_type == INC_FILE && Maildir == NULL) {
 	if (access (newmail, W_OK) != NOTOK) {
@@ -597,7 +517,7 @@ go_to_it:
 	    }
 
             GETGROUPPRIVS();       /* Reset gid to lock mail file */
-            in = lkfopen (newmail, "r");
+            in = lkfopenspool (newmail, "r");
             DROPGROUPPRIVS();
             if (in == NULL)
 		adios (NULL, "unable to lock and fopen %s", newmail);
@@ -621,31 +541,17 @@ go_to_it:
 	else if (i == NOTOK)
 	    chmod (audfile, m_gmprot ());
 
-#ifdef POP
-	fprintf (aud, from ? "<<inc>> %s -ms %s\n"
-		 : host ? "<<inc>> %s -host %s -user %s\n"
-		 : "<<inc>> %s\n",
-		 dtimenow (0), from ? from : host, user);
-#else /* POP */
-	fprintf (aud, from ? "<<inc>> %s  -ms %s\n" : "<<inc>> %s\n",
-		 dtimenow (0), from);
-#endif /* POP */
+	if (from)
+	    fprintf (aud, "<<inc>> %s -ms %s\n", dtimenow(0), from);
+	else {
+	    if (host)
+	    	fprintf (aud, "<<inc>> %s -host %s -user %s\n", dtimenow(0),
+			 host, user);
+	    else
+		fprintf (aud, "<<inc>> %s\n", dtimenow (0));
+	}
     }
 
-#ifdef MHE
-    if (context_find ("mhe")) {
-	int i;
-	cp = concat (maildir, "/++", NULL);
-	i = stat (cp, &st);
-	if ((mhe = fopen (cp, "a")) == NULL)
-	    admonish (cp, "unable to append to");
-	else
-	    if (i == NOTOK)
-		chmod (cp, m_gmprot ());
-	free (cp);
-    }
-#endif /* MHE */
-
     /* Get new format string */
     nfs = new_fs (form, format, FORMAT);
 
@@ -654,7 +560,7 @@ go_to_it:
 	fflush (stdout);
     }
 
-#ifdef POP
+
     /*
      * Get the mail from a POP server
      */
@@ -678,14 +584,6 @@ go_to_it:
 		adios (NULL, "unable to fdopen %s", packfile);
 	} else {
 	    hghnum = msgnum = mp->hghmsg;
-	    /*
-	     * Check if we have enough message space for all the new
-	     * messages.  If not, then realloc the folder and add enough
-	     * space for all new messages plus 10 additional slots.
-	     */
-	    if (mp->hghmsg + nmsgs >= mp->hghoff
-		&& !(mp = folder_realloc (mp, mp->lowoff, mp->hghmsg + nmsgs + 10)))
-		adios (NULL, "unable to allocate folder storage");
 	}
 
 	for (i = 1; i <= nmsgs; i++) {
@@ -741,18 +639,8 @@ go_to_it:
 	    default: 
 		if (aud)
 		    fputs (scanl, aud);
-# ifdef MHE
-		if (mhe)
-		    fputs (scanl, mhe);
-# endif /* MHE */
 		if (noisy)
 		    fflush (stdout);
-		if (!packfile) {
-		    clear_msg_flags (mp, msgnum);
-		    set_exists (mp, msgnum);
-		    set_unseen (mp, msgnum);
-		    mp->msgflags |= SEQMOD;
-		}
 		break;
 	    }
 	    if (packfile) {
@@ -768,7 +656,7 @@ go_to_it:
 	    } else {
 		if (ferror(pf) || fclose (pf)) {
 		    int e = errno;
-		    unlink (cp);
+		    (void) m_unlink (cp);
 		    pop_quit ();
 		    errno = e;
 		    adios (cp, "write error on");
@@ -778,6 +666,8 @@ go_to_it:
 
 	    if (trnflag && pop_dele (i) == NOTOK)
 		adios (NULL, "%s", response);
+
+	    scan_finished();
 	}
 
 	if (pop_quit () == NOTOK)
@@ -787,39 +677,14 @@ go_to_it:
 	    pd = NOTOK;
 	}
     }
-#endif /* POP */
 
     /*
      * Get the mail from file (usually mail spool)
      */
     if (inc_type == INC_FILE && Maildir == NULL) {
-	m_unknown (in);		/* the MAGIC invocation... */
+	scan_detect_mbox_style (in);		/* the MAGIC invocation... */
 	hghnum = msgnum = mp->hghmsg;
 	for (;;) {
-	    /*
-	     * Check if we need to allocate more space for message status.
-	     * If so, then add space for an additional 100 messages.
-	     */
-	    if (msgnum >= mp->hghoff
-		&& !(mp = folder_realloc (mp, mp->lowoff, mp->hghoff + 100))) {
-		advise (NULL, "unable to allocate folder storage");
-		incerr = NOTOK;
-		break;
-	    }
-
-#if 0
-	    /* copy file from spool to tmp file */
-	    tmpfilenam = m_scratch ("", invo_name);
-	    if ((fd = creat (tmpfilenam, m_gmprot ())) == NOTOK)
-		adios (tmpfilenam, "unable to create");
-	    chmod (tmpfilenam, m_gmprot ());
-	    if (!(in2 = fdopen (fd, "r+")))
-		adios (tmpfilenam, "unable to access");
-	    cpymsg (in, in2);
-
-	    /* link message into folder */
-	    newmsg = folder_addmsg(mp, tmpfilenam);
-#endif
 	    /* create scanline for new message */
 	    switch (incerr = scan (in, msgnum + 1, msgnum + 1, nfs, width,
 			      msgnum == hghnum && chgflag, 1, NULL, 0L, noisy)) {
@@ -852,22 +717,10 @@ go_to_it:
 
 		if (aud)
 		    fputs (scanl, aud);
-#ifdef MHE
-		if (mhe)
-		    fputs (scanl, mhe);
-#endif /* MHE */
 		if (noisy)
 		    fflush (stdout);
 
 		msgnum++;
-		mp->hghmsg++;
-		mp->nummsg++;
-		if (mp->lowmsg == 0) mp->lowmsg = 1;
-
-		clear_msg_flags (mp, msgnum);
-		set_exists (mp, msgnum);
-		set_unseen (mp, msgnum);
-		mp->msgflags |= SEQMOD;
 		continue;
 	    }
 	    /* If we get here there was some sort of error from scan(),
@@ -883,16 +736,6 @@ go_to_it:
 	hghnum = msgnum = mp->hghmsg;
 	for (i = 0; i < num_maildir_entries; i++) {
 	    msgnum++;
-	    /*
-	     * Check if we need to allocate more space for message status.
-	     * If so, then add space for an additional 100 messages.
-	     */
-	    if (msgnum >= mp->hghoff
-		&& !(mp = folder_realloc (mp, mp->lowoff, mp->hghoff + 100))) {
-		advise (NULL, "unable to allocate folder storage");
-		incerr = NOTOK;
-		break;
-	    }
 
 	    sp = Maildir[i].filename;
 	    cp = getcpy (m_name (msgnum));
@@ -910,7 +753,7 @@ go_to_it:
 			break;
 		if (ferror(sf) || fflush(pf) || ferror(pf)) {
 			int e = errno;
-			fclose(pf); fclose(sf); unlink(cp);
+			fclose(pf); fclose(sf); (void) m_unlink(cp);
 			errno = e;
 			adios(cp, "copy error %s -> %s", sp, cp);
 		}
@@ -951,40 +794,34 @@ go_to_it:
 
 		if (aud)
 		    fputs (scanl, aud);
-# ifdef MHE
-		if (mhe)
-		    fputs (scanl, mhe);
-# endif /* MHE */
 		if (noisy)
 		    fflush (stdout);
-		if (!packfile) {
-		    clear_msg_flags (mp, msgnum);
-		    set_exists (mp, msgnum);
-		    set_unseen (mp, msgnum);
-		    mp->msgflags |= SEQMOD;
-		}
 		break;
 	    }
 	    if (ferror(pf) || fclose (pf)) {
 		int e = errno;
-		unlink (cp);
+		(void) m_unlink (cp);
 		errno = e;
 		adios (cp, "write error on");
 	    }
 	    pf = NULL;
 	    free (cp);
 
-	    if (trnflag && unlink (sp) == NOTOK)
+	    if (trnflag && m_unlink (sp) == NOTOK)
 		adios (sp, "couldn't unlink");
 	    free (sp); /* Free Maildir[i]->filename */
+
+	    scan_finished();
 	}
 	free (Maildir); /* From now on Maildir is just a flag - don't dref! */
     }
 
+    scan_finished ();
+
     if (incerr < 0) {		/* error */
 	if (locked) {
             GETGROUPPRIVS();      /* Be sure we can unlock mail file */
-            (void) lkfclose (in, newmail); in = NULL;
+            (void) lkfclosespool (in, newmail); in = NULL;
             DROPGROUPPRIVS();    /* And then return us to normal privileges */
 	} else {
 	    fclose (in); in = NULL;
@@ -995,18 +832,11 @@ go_to_it:
     if (aud)
 	fclose (aud);
 
-#ifdef MHE
-    if (mhe)
-	fclose (mhe);
-#endif /* MHE */
-
     if (noisy)
 	fflush (stdout);
 
-#ifdef POP
     if ((inc_type == INC_POP) && packfile)
 	done (0);
-#endif /* POP */
 
     /*
      * truncate file we are incorporating from
@@ -1021,7 +851,7 @@ go_to_it:
 		    close (newfd);
 		else
 		    admonish (newmail, "error zero'ing");
-		unlink(map_name(newmail));
+		(void) m_unlink(map_name(newmail));
 	    }
 	} else {
 	    if (noisy)
@@ -1032,15 +862,51 @@ go_to_it:
     if (msgnum == hghnum) {
 	admonish (NULL, "no messages incorporated");
     } else {
+    	/*
+	 * Lock the sequence file now, and loop to set the right flags
+	 * in the folder structure
+	 */
+
+	struct msgs *mp2;
+	int i;
+
 	context_replace (pfolder, folder);	/* update current folder */
+
+	if ((mp2 = folder_read(folder, 1)) == NULL) {
+	    admonish(NULL, "Unable to reread folder %s", folder);
+	    goto skip;
+	}
+
+	/*
+	 * Shouldn't happen, but just in case ...
+	 */
+
+	if (msgnum >= mp2->hghoff
+		&& !(mp2 = folder_realloc (mp2, mp2->lowoff, msgnum + 1))) {
+	    advise (NULL, "unable to reallocate folder storage");
+	    goto skip;
+	}
+
 	if (chgflag)
-	    mp->curmsg = hghnum + 1;
-	mp->hghmsg = msgnum;
-	if (mp->lowmsg == 0)
-	    mp->lowmsg = 1;
+	    mp2->curmsg = hghnum + 1;
+	mp2->hghmsg = msgnum;
+
+	if (mp2->lowmsg == 0)
+	    mp2->lowmsg = 1;
 	if (chgflag)		/* sigh... */
-	    seq_setcur (mp, mp->curmsg);
+	    seq_setcur (mp2, mp2->curmsg);
+
+	for (i = hghnum + 1; i <= msgnum; i++) {
+	    clear_msg_flags (mp2, i);
+	    set_exists (mp2, i);
+	    set_unseen (mp2, i);
+	}
+	mp2->msgflags |= SEQMOD;
+	seq_setunseen(mp2, 0);	/* Set the Unseen-Sequence */
+	seq_save(mp2);		/* Save the sequence file */
+	folder_free(mp2);
     }
+skip:
 
     /*
      * unlock the mail spool
@@ -1048,71 +914,33 @@ go_to_it:
     if (inc_type == INC_FILE && Maildir == NULL) {
 	if (locked) {
            GETGROUPPRIVS();        /* Be sure we can unlock mail file */
-           (void) lkfclose (in, newmail); in = NULL;
+           (void) lkfclosespool (in, newmail); in = NULL;
            DROPGROUPPRIVS();       /* And then return us to normal privileges */
 	} else {
 	    fclose (in); in = NULL;
 	}
     }
 
-    seq_setunseen (mp, 0);	/* set the Unseen-Sequence */
-    seq_save (mp);		/* synchronize sequences   */
     context_save ();		/* save the context file   */
     done (0);
     return 1;
 }
 
 
-#if 0
-
-/*
- * Copy message message from spool into
- * temporary file.  Massage the "From " line
- * while copying.
- */
-
-cpymsg (FILE *in, FILE *out)
-{
-    int state;
-    char *tmpbuf, name[NAMESZ];
-
-    for (;;) {
-	state = m_getfld (state, name, tmpbuf, rlwidth, in);
-	switch (state) {
-	case FLD:
-	case FLDPLUS:
-	    break;
-	case BODY:
-	    break;
-	case LENERR:
-	case FMTERR:
-	    break;
-	case FILEEOF:
-	    break;
-	default:
-	}
-    }
-}
-#endif /* if 0 */
-
-
 static void
 inc_done (int status)
 {
-#ifdef POP
     if (packfile && pd != NOTOK)
 	mbx_close (packfile, pd);
-#endif /* POP */
     if (locked)
     {
         GETGROUPPRIVS();
-        lkfclose(in, newmail);
+        lkfclosespool(in, newmail);
         DROPGROUPPRIVS();
     }
     exit (status);
 }
 
-#ifdef POP
 static int
 pop_action (char *s)
 {
@@ -1156,4 +984,3 @@ map_count (void)
     close (md);
     return (d.d_id);
 }
-#endif /* POP */
