@@ -11,8 +11,6 @@
 #include <fcntl.h>
 #include <h/signals.h>
 #include <h/md5.h>
-#include <errno.h>
-#include <signal.h>
 #include <h/mts.h>
 #include <h/tws.h>
 #include <h/mime.h>
@@ -20,59 +18,56 @@
 #include <h/mhcachesbr.h>
 #include <h/utils.h>
 
-#ifdef HAVE_SYS_WAIT_H
-# include <sys/wait.h>
-#endif
+#define MHBUILD_SWITCHES \
+    X("auto", 0, AUTOSW) \
+    X("noauto", 0, NAUTOSW) \
+    X("check", 0, CHECKSW) \
+    X("nocheck", 0, NCHECKSW) \
+    X("directives", 0, DIRECTIVES) \
+    X("nodirectives", 0, NDIRECTIVES) \
+    X("headers", 0, HEADSW) \
+    X("noheaders", 0, NHEADSW) \
+    X("list", 0, LISTSW) \
+    X("nolist", 0, NLISTSW) \
+    X("realsize", 0, SIZESW) \
+    X("norealsize", 0, NSIZESW) \
+    X("rfc934mode", 0, RFC934SW) \
+    X("norfc934mode", 0, NRFC934SW) \
+    X("verbose", 0, VERBSW) \
+    X("noverbose", 0, NVERBSW) \
+    X("disposition", 0, DISPOSW) \
+    X("nodisposition", 0, NDISPOSW) \
+    X("rcache policy", 0, RCACHESW) \
+    X("wcache policy", 0, WCACHESW) \
+    X("contentid", 0, CONTENTIDSW) \
+    X("nocontentid", 0, NCONTENTIDSW) \
+    X("headerencoding encoding-algorithm", 0, HEADERENCSW) \
+    X("autoheaderencoding", 0, AUTOHEADERENCSW) \
+    X("maxunencoded", 0, MAXUNENCSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+    X("debug", -5, DEBUGSW) \
+    X("dist", -4, DISTSW) \
 
-static struct swit switches[] = {
-#define	CHECKSW                 0
-    { "check", 0 },
-#define	NCHECKSW                1
-    { "nocheck", 0 },
-#define	EBCDICSW                2
-    { "ebcdicsafe", 0 },
-#define	NEBCDICSW               3
-    { "noebcdicsafe", 0 },
-#define	HEADSW                  4
-    { "headers", 0 },
-#define	NHEADSW                 5
-    { "noheaders", 0 },
-#define	LISTSW                  6
-    { "list", 0 },
-#define	NLISTSW                 7
-    { "nolist", 0 },
-#define	SIZESW                  8
-    { "realsize", 0 },
-#define	NSIZESW                 9
-    { "norealsize", 0 },
-#define	RFC934SW               10
-    { "rfc934mode", 0 },
-#define	NRFC934SW              11
-    { "norfc934mode", 0 },
-#define	VERBSW                 12
-    { "verbose", 0 },
-#define	NVERBSW                13
-    { "noverbose", 0 },
-#define	RCACHESW               14
-    { "rcache policy", 0 },
-#define	WCACHESW               15
-    { "wcache policy", 0 },
-#define	CONTENTIDSW            16
-    { "contentid", 0 },
-#define	NCONTENTIDSW           17
-    { "nocontentid", 0 },
-#define VERSIONSW              18
-    { "version", 0 },
-#define	HELPSW                 19
-    { "help", 0 },
-#define	DEBUGSW                20
-    { "debug", -5 },
-    { NULL, 0 }
-};
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(MHBUILD);
+#undef X
 
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(MHBUILD, switches);
+#undef X
 
-/* mhbuildsbr.c */
-extern char *tmp;	/* directory to place temp files */
+#define MIMEENCODING_SWITCHES \
+    X("base64", 0, BASE64SW) \
+    X("quoted-printable", 0, QUOTEDPRINTSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(MIMEENCODING);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(MIMEENCODING, encodingswitches);
+#undef X
 
 /* mhcachesbr.c */
 extern int rcachesw;
@@ -81,9 +76,7 @@ extern char *cache_public;
 extern char *cache_private;
 
 int debugsw = 0;
-int verbosw = 0;
 
-int ebcdicsw = 0;
 int listsw   = 0;
 int rfc934sw = 0;
 int contentidsw = 1;
@@ -100,24 +93,16 @@ static int unlink_outfile = 0;
 static void unlink_done (int) NORETURN;
 
 /* mhbuildsbr.c */
-CT build_mime (char *);
 int output_message (CT, char *);
 int output_message_fp (CT, FILE *, char*);
 
-/* mhlistsbr.c */
-int list_all_messages (CT *, int, int, int, int);
-
-/* mhmisc.c */
-void set_endian (void);
-
-/* mhfree.c */
-void free_content (CT);
-
 
 int
 main (int argc, char **argv)
 {
-    int sizesw = 1, headsw = 1;
+    int sizesw = 1, headsw = 1, directives = 1, autobuild = 0, dist = 0;
+    int verbosw = 0, dispo = 0;
+    size_t maxunencoded = MAXTEXTPERLN;
     int *icachesw;
     char *cp, buf[BUFSIZ];
     char buffer[BUFSIZ], *compfile = NULL;
@@ -125,17 +110,12 @@ main (int argc, char **argv)
     CT ct, cts[2];
     FILE *fp = NULL;
     FILE *fp_out = NULL;
+    int header_encoding = CE_UNKNOWN;
+
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     done=unlink_done;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
-
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
@@ -160,10 +140,24 @@ main (int argc, char **argv)
 	    case HELPSW: 
 		snprintf (buf, sizeof(buf), "%s [switches] file", invo_name);
 		print_help (buf, switches, 1);
-		done (1);
+		done (0);
 	    case VERSIONSW:
 		print_version(invo_name);
-		done (1);
+		done (0);
+
+	    case AUTOSW:
+	    	/* -auto implies -nodirectives */
+		autobuild = 1;
+		directives = 0;
+		continue;
+	    case NAUTOSW:
+	    	/*
+		 * We're turning directives back on since this is likely here
+		 * to override a profile entry
+		 */
+		autobuild = 0;
+		directives = 1;
+		continue;
 
 	    case RCACHESW:
 		icachesw = &rcachesw;
@@ -191,13 +185,6 @@ main (int argc, char **argv)
 		checksw = 0;
 		continue;
 
-	    case EBCDICSW:
-		ebcdicsw++;
-		continue;
-	    case NEBCDICSW:
-		ebcdicsw = 0;
-		continue;
-
 	    case HEADSW:
 		headsw++;
 		continue;
@@ -205,6 +192,13 @@ main (int argc, char **argv)
 		headsw = 0;
 		continue;
 
+	    case DIRECTIVES:
+		directives = 1;
+		continue;
+	    case NDIRECTIVES:
+		directives = 0;
+		continue;
+
 	    case LISTSW:
 		listsw++;
 		continue;
@@ -233,15 +227,60 @@ main (int argc, char **argv)
 		contentidsw = 0;
 		continue;
 
+	    case HEADERENCSW: {
+	    	int encoding;
+
+		if (!(cp = *argp++) || *cp == '-')
+		    adios (NULL, "missing argument to %s", argp[-2]);
+		switch (encoding = smatch (cp, encodingswitches)) {
+		case AMBIGSW:
+		    ambigsw (cp, encodingswitches);
+		    done (1);
+		case UNKWNSW:
+		    adios (NULL, "%s unknown encoding algorithm", cp);
+		case BASE64SW:
+		    header_encoding = CE_BASE64;
+		    break;
+		case QUOTEDPRINTSW:
+		    header_encoding = CE_QUOTED;
+		    break;
+		default:
+		    adios (NULL, "Internal error: algorithm %s", cp);
+		}
+		continue;
+	    }
+
+	    case AUTOHEADERENCSW:
+	    	header_encoding = CE_UNKNOWN;
+		continue;
+
+	    case MAXUNENCSW:
+		if (!(cp = *argp++) || *cp == '-')
+		    adios (NULL, "missing argument to %s", argp[-2]);
+		if ((maxunencoded = atoi(cp)) < 1)
+		    adios (NULL, "Invalid argument for %s: %s", argp[-2], cp);
+		if (maxunencoded > 998)
+		    adios (NULL, "limit of -maxunencoded is 998");
+		continue;
+
 	    case VERBSW: 
 		verbosw++;
 		continue;
 	    case NVERBSW: 
 		verbosw = 0;
 		continue;
+	    case DISPOSW:
+		dispo = 1;
+		continue;
+	    case NDISPOSW:
+		dispo = 0;
+		continue;
 	    case DEBUGSW:
 		debugsw = 1;
 		continue;
+	    case DISTSW:
+		dist = 1;
+		continue;
 	    }
 	}
 	if (compfile)
@@ -250,11 +289,6 @@ main (int argc, char **argv)
 	    compfile = cp;
     }
 
-    set_endian ();
-
-    if ((cp = getenv ("MM_NOASK")) && !strcmp (cp, "1"))
-	listsw  = 0;
-
     /*
      * Check if we've specified an additional profile
      */
@@ -284,16 +318,6 @@ main (int argc, char **argv)
 	cache_private = ".cache";
     cache_private = getcpy (m_maildir (cache_private));
 
-    /*
-     * Check for storage directory.  If defined, we
-     * will store temporary files there.  Else we
-     * store them in standard nmh directory.
-     */
-    if ((cp = context_find (nmhstorage)) && *cp)
-	tmp = concat (cp, "/", invo_name, NULL);
-    else
-	tmp = add (m_maildir (invo_name), NULL);
-
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
 
@@ -305,40 +329,42 @@ main (int argc, char **argv)
      * Process the composition file from standard input.
      */
     if (compfile[0] == '-' && compfile[1] == '\0') {
+	if ((cp = m_mktemp2(NULL, invo_name, NULL, &fp)) == NULL) {
+	    adios(NULL, "unable to create temporary file in %s",
+		  get_temp_dir());
+	}
+
 	/* copy standard input to temporary file */
-	strncpy (infile, m_mktemp(invo_name, NULL, &fp), sizeof(infile));
+	strncpy (infile, cp, sizeof(infile));
 	while (fgets (buffer, BUFSIZ, stdin))
 	    fputs (buffer, fp);
 	fclose (fp);
 	unlink_infile = 1;
 
 	/* build the content structures for MIME message */
-	ct = build_mime (infile);
-	cts[0] = ct;
-	cts[1] = NULL;
+	ct = build_mime (infile, autobuild, dist, directives, header_encoding,
+			 maxunencoded, verbosw);
 
-	/* output MIME message to this temporary file */
-	strncpy (outfile, m_mktemp(invo_name, NULL, &fp_out), sizeof(outfile));
-	unlink_outfile = 1;
+	/*
+	 * If ct == NULL, that means that -auto was set and a MIME version
+	 * header was already seen.  Just use the input file as the output
+	 */
 
-	/* output the message */
-	output_message_fp (ct, fp_out, outfile);
-        fclose(fp_out);
+	if (!ct) {
+	    if (! (fp = fopen(infile, "r"))) {
+		adios(NULL, "Unable to open %s for reading", infile);
+	    }
+	    while (fgets(buffer, BUFSIZ, fp))
+		fputs(buffer, stdout);
+	} else {
+	    /* output the message */
+	    output_message_fp (ct, stdout, NULL);
+	    free_content (ct);
+	}
 
-	/* output the temp file to standard output */
-	if ((fp = fopen (outfile, "r")) == NULL)
-	    adios (outfile, "unable to open");
-	while (fgets (buffer, BUFSIZ, fp))
-	    fputs (buffer, stdout);
-	fclose (fp);
-
-	unlink (infile);
+	(void) m_unlink (infile);
 	unlink_infile = 0;
 
-	unlink (outfile);
-	unlink_outfile = 0;
-
-	free_content (ct);
 	done (0);
     }
 
@@ -347,13 +373,25 @@ main (int argc, char **argv)
      */
 
     /* build the content structures for MIME message */
-    ct = build_mime (compfile);
+    ct = build_mime (compfile, autobuild, dist, directives, header_encoding,
+    		     maxunencoded, verbosw);
+
+    /*
+     * If ct == NULL, that means -auto was set and we found a MIME version
+     * header.  Simply exit and do nothing.
+     */
+
+    if (! ct)
+	done(0);
+
     cts[0] = ct;
     cts[1] = NULL;
 
     /* output MIME message to this temporary file */
-    strncpy(outfile, m_mktemp2(compfile, invo_name, NULL, &fp_out),
-            sizeof(outfile));
+    if ((cp = m_mktemp2(compfile, invo_name, NULL, &fp_out)) == NULL) {
+	adios(NULL, "unable to create temporary file in %s", get_temp_dir());
+    }
+    strncpy(outfile, cp, sizeof(outfile));
     unlink_outfile = 1;
 
     /* output the message */
@@ -364,7 +402,7 @@ main (int argc, char **argv)
      * List the message info
      */
     if (listsw)
-	list_all_messages (cts, headsw, sizesw, verbosw, debugsw);
+	list_all_messages (cts, headsw, sizesw, verbosw, debugsw, dispo);
 
     /* Rename composition draft */
     snprintf (buffer, sizeof(buffer), "%s.orig", m_backup (compfile));
@@ -394,9 +432,9 @@ unlink_done (int status)
      * temporary files.
      */
     if (unlink_infile)
-	unlink (infile);
+	(void) m_unlink (infile);
     if (unlink_outfile)
-	unlink (outfile);
+	(void) m_unlink (outfile);
 
     exit (status);
 }
