@@ -11,9 +11,6 @@
 #include <fcntl.h>
 #include <h/signals.h>
 #include <h/md5.h>
-#include <errno.h>
-#include <setjmp.h>
-#include <signal.h>
 #include <h/mts.h>
 #include <h/tws.h>
 #include <h/mime.h>
@@ -21,25 +18,13 @@
 #include <h/mhcachesbr.h>
 #include <h/utils.h>
 
-#ifdef TIME_WITH_SYS_TIME
+#ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
-# include <time.h>
-#else
-# ifdef TM_IN_SYS_TIME
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-
-#ifdef HAVE_SYS_WAIT_H
-# include <sys/wait.h>
 #endif
+#include <time.h>
 
 extern int debugsw;
 
-extern pid_t xpid;	/* mhshowsbr.c or mhbuildsbr.c */
-
 /* cache policies */
 int rcachesw = CACHE_ASK;
 int wcachesw = CACHE_ASK;
@@ -52,13 +37,9 @@ char *cache_public;
 char *cache_private;
 
 
-/* mhparse.c (OR) mhbuildsbr.c */
-int pidcheck (int);
-
 /* mhmisc.c */
 int part_ok (CT, int);
 int type_ok (CT, int);
-int make_intermediates (char *);
 void content_error (char *, CT, char *, ...);
 void flush_errors (void);
 
@@ -111,7 +92,7 @@ cache_content (CT ct)
 {
     int	cachetype;
     char *file, cachefile[BUFSIZ];
-    CE ce = ct->c_cefile;
+    CE ce = &ct->c_cefile;
 
     if (!ct->c_id) {
 	advise (NULL, "no %s: field in %s", ID_FIELD, ct->c_file);
@@ -123,14 +104,6 @@ cache_content (CT ct)
 	return;
     }
 
-/* THIS NEEDS TO BE FIXED */
-#if 0
-    if (ct->c_ceopenfnx == openMail) {
-	advise (NULL, "a radish may no know Greek, but I do...");
-	return;
-    }
-#endif
-
     if (find_cache (NULL, wcachesw != CACHE_NEVER ? wcachesw : CACHE_ASK,
 		    &cachetype, ct->c_id, cachefile, sizeof(cachefile))
 	    == NOTOK) {
@@ -168,11 +141,11 @@ cache_content (CT ct)
 
 	    if (ferror (gp)) {
 		admonish (ce->ce_file, "error reading");
-		unlink (cachefile);
+		(void) m_unlink (cachefile);
 	    } else {
 		if (ferror (fp)) {
 		    admonish (cachefile, "error writing");
-		    unlink (cachefile);
+		    (void) m_unlink (cachefile);
 		}
 	    }
 	    fclose (fp);
@@ -251,13 +224,6 @@ got_it:
 	int len, buflen;
 	char *bp, query[BUFSIZ];
 
-	if (xpid) {
-	    if (xpid < 0)
-		xpid = -xpid;
-	    pidcheck (pidwait (xpid, NOTOK));
-	    xpid = 0;
-	}
-
 	/* Get buffer ready to go */
 	bp = query;
 	buflen = sizeof(query);
@@ -298,7 +264,7 @@ got_it:
     if (status == OK && writing) {
 	if (*writing && strchr(buffer, '/'))
 	    make_intermediates (buffer);
-	unlink (buffer);
+	(void) m_unlink (buffer);
     }
 
     free (id);
@@ -313,14 +279,11 @@ find_cache_aux (int writing, char *directory, char *id,
     int	mask, usemap;
     char mapfile[BUFSIZ], mapname[BUFSIZ];
     FILE *fp;
+    int failed_to_lock = 0;
     static int partno, pid;
     static time_t clock = 0;
 
-#ifdef BSD42
-    usemap = strchr (id, '/') ? 1 : 0;
-#else
     usemap = 1;
-#endif
 
     if (debugsw)
 	fprintf (stderr, "find_cache_aux %s usemap=%d\n", directory, usemap);
@@ -371,19 +334,22 @@ use_raw:
 
     make_intermediates (mapfile);
     mask = umask (writing == 2 ? 0077 : 0);
-    if (!(fp = lkfopen (mapfile, "a")) && errno == ENOENT) {
+    if (!(fp = lkfopendata (mapfile, "a", &failed_to_lock)) && errno == ENOENT) {
 	int fd;
 
 	if ((fd = creat (mapfile, 0666)) != NOTOK) {
 	    close (fd);
-	    fp = lkfopen (mapfile, "a");
+	    fp = lkfopendata (mapfile, "a", &failed_to_lock);
+            if (failed_to_lock) {
+		adios (mapfile, "failed to lock");
+            }
 	}
     }
     umask (mask);
     if (!fp)
 	return NOTOK;
     fprintf (fp, "%s: %s\n", mapname, id);
-    lkfclose (fp, mapfile);
+    lkfclosedata (fp, mapfile);
 
 done_map:
     if (*mapname == '/')
@@ -403,25 +369,28 @@ find_cache_aux2 (char *mapfile, char *id, char *mapname, int namelen)
     int	state;
     char buf[BUFSIZ], name[NAMESZ];
     FILE *fp;
+    m_getfld_state_t gstate = 0;
+    int failed_to_lock = 0;
 
-    if (!(fp = lkfopen (mapfile, "r")))
+    if (!(fp = lkfopendata (mapfile, "r", &failed_to_lock)))
 	return NOTOK;
 
-    for (state = FLD;;) {
+    for (;;) {
 	int result;
 	char *cp, *dp;
+	int bufsz = sizeof buf;
 
-	switch (state = m_getfld (state, name, buf, sizeof(buf), fp)) {
+	switch (state = m_getfld (&gstate, name, buf, &bufsz, fp)) {
 	    case FLD:
 	    case FLDPLUS:
-	    case FLDEOF:
 	        strncpy (mapname, name, namelen);
 		if (state != FLDPLUS)
 		    cp = buf;
 		else {
 		    cp = add (buf, NULL);
 		    while (state == FLDPLUS) {
-			state = m_getfld (state, name, buf, sizeof(buf), fp);
+			bufsz = sizeof buf;
+			state = m_getfld (&gstate, name, buf, &bufsz, fp);
 			cp = add (buf, cp);
 		    }
 		}
@@ -434,22 +403,20 @@ find_cache_aux2 (char *mapfile, char *id, char *mapname, int namelen)
 		result = strcmp (id, dp);
 		free (dp);
 		if (result == 0) {
-		    lkfclose (fp, mapfile);
+		    lkfclosedata (fp, mapfile);
 		    return OK;
 		}
-		if (state != FLDEOF)
-		    continue;
-		/* else fall... */
+		continue;
 
 	    case BODY:
-	    case BODYEOF:
 	    case FILEEOF:
 	    default:
 		break;
 	}
 	break;
     }
+    m_getfld_state_destroy (&gstate);
 
-    lkfclose (fp, mapfile);
+    lkfclosedata (fp, mapfile);
     return NOTOK;
 }
