@@ -11,8 +11,6 @@
 #include <fcntl.h>
 #include <h/signals.h>
 #include <h/md5.h>
-#include <errno.h>
-#include <signal.h>
 #include <h/mts.h>
 #include <h/tws.h>
 #include <h/mime.h>
@@ -20,111 +18,60 @@
 #include <h/mhcachesbr.h>
 #include <h/utils.h>
 
-#ifdef HAVE_SYS_WAIT_H
-# include <sys/wait.h>
-#endif
+#define MHN_SWITCHES \
+    X("auto", 0, AUTOSW) \
+    X("noauto", 0, NAUTOSW) \
+    X("cache", 0, CACHESW) \
+    X("nocache", 0, NCACHESW) \
+    X("check", 0, CHECKSW) \
+    X("nocheck", 0, NCHECKSW) \
+    X("headers", 0, HEADSW) \
+    X("noheaders", 0, NHEADSW) \
+    X("list", 0, LISTSW) \
+    X("nolist", 0, NLISTSW) \
+    X("realsize", 0, SIZESW) \
+    X("norealsize", 0, NSIZESW) \
+    X("show", 0, SHOWSW) \
+    X("noshow", 0, NSHOWSW) \
+    X("store", 0, STORESW) \
+    X("nostore", 0, NSTORESW) \
+    X("verbose", 0, VERBSW) \
+    X("noverbose", 0, NVERBSW) \
+    X("file file", 0, FILESW) \
+    X("form formfile", 0, FORMSW) \
+    X("part number", 0, PARTSW) \
+    X("type content", 0, TYPESW) \
+    X("rcache policy", 0, RCACHESW) \
+    X("wcache policy", 0, WCACHESW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+    /*					\
+     * for debugging			\
+     */					\
+    X("debug", -5, DEBUGSW) \
+    /*					\
+     * switches for moreproc/mhlproc	\
+     */					\
+    X("moreproc program", -4, PROGSW) \
+    X("nomoreproc", -3, NPROGSW) \
+    X("length lines", -4, LENSW) \
+    X("width columns", -4, WIDTHSW) \
+    /*					\
+     * switches for mhbuild		\
+     */					\
+    X("build", -5, BUILDSW) \
+    X("nobuild", -7, NBUILDSW) \
+    X("rfc934mode", -10, RFC934SW) \
+    X("norfc934mode", -12, NRFC934SW) \
 
-static struct swit switches[] = {
-#define	AUTOSW                  0
-    { "auto", 0 },
-#define	NAUTOSW                 1
-    { "noauto", 0 },
-#define	CACHESW                 2
-    { "cache", 0 },
-#define	NCACHESW                3
-    { "nocache", 0 },
-#define	CHECKSW                 4
-    { "check", 0 },
-#define	NCHECKSW                5
-    { "nocheck", 0 },
-#define	HEADSW                  6
-    { "headers", 0 },
-#define	NHEADSW                 7
-    { "noheaders", 0 },
-#define	LISTSW                  8
-    { "list", 0 },
-#define	NLISTSW                 9
-    { "nolist", 0 },
-#define	PAUSESW                10
-    { "pause", 0 },
-#define	NPAUSESW               11
-    { "nopause", 0 },
-#define	SIZESW                 12
-    { "realsize", 0 },
-#define	NSIZESW                13
-    { "norealsize", 0 },
-#define	SERIALSW               14
-    { "serialonly", 0 },
-#define	NSERIALSW              15
-    { "noserialonly", 0 },
-#define	SHOWSW                 16
-    { "show", 0 },
-#define	NSHOWSW                17
-    { "noshow", 0 },
-#define	STORESW                18
-    { "store", 0 },
-#define	NSTORESW               19
-    { "nostore", 0 },
-#define	VERBSW                 20
-    { "verbose", 0 },
-#define	NVERBSW                21
-    { "noverbose", 0 },
-#define	FILESW                 22	/* interface from show */
-    { "file file", 0 },
-#define	FORMSW                 23
-    { "form formfile", 0 },
-#define	PARTSW                 24
-    { "part number", 0 },
-#define	TYPESW                 25
-    { "type content", 0 },
-#define	RCACHESW               26
-    { "rcache policy", 0 },
-#define	WCACHESW               27
-    { "wcache policy", 0 },
-#define VERSIONSW              28
-    { "version", 0 },
-#define	HELPSW                 29
-    { "help", 0 },
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(MHN);
+#undef X
 
-/*
- * switches for debugging
- */
-#define	DEBUGSW                30
-    { "debug", -5 },
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(MHN, switches);
+#undef X
 
-/*
- * switches for moreproc/mhlproc
- */
-#define	PROGSW                 31
-    { "moreproc program", -4 },
-#define	NPROGSW                32
-    { "nomoreproc", -3 },
-#define	LENSW                  33
-    { "length lines", -4 },
-#define	WIDTHSW                34
-    { "width columns", -4 },
-
-/*
- * switches for mhbuild
- */
-#define BUILDSW                35
-    { "build", -5 },
-#define NBUILDSW               36
-    { "nobuild", -7 },
-#define	EBCDICSW               37
-    { "ebcdicsafe", -10 },
-#define	NEBCDICSW              38
-    { "noebcdicsafe", -12 },
-#define	RFC934SW               39
-    { "rfc934mode", -10 },
-#define	NRFC934SW              40
-    { "norfc934mode", -12 },
-    { NULL, 0 }
-};
-
-
-/* mhparse.c */
-extern char *tmp;	/* directory to place temp files */
 
 /* mhcachesbr.c */
 extern int rcachesw;
@@ -133,17 +80,11 @@ extern char *cache_public;
 extern char *cache_private;
 
 /* mhshowsbr.c */
-extern int pausesw;
-extern int serialsw;
 extern char *progsw;
 extern int nolist;
 extern int nomore;	/* flags for moreproc/header display */
 extern char *formsw;
 
-/* mhstoresbr.c */
-extern int autosw;
-extern char *cwd;	/* cache current working directory */
-
 /* mhmisc.c */
 extern int npart;
 extern int ntype;
@@ -158,7 +99,6 @@ int verbosw = 0;
  * variables for mhbuild (mhn -build)
  */
 static int buildsw  = 0;
-static int ebcdicsw = 0;
 static int rfc934sw = 0;
 
 /*
@@ -177,55 +117,46 @@ CT parse_mime (char *);
 /* mhmisc.c */
 int part_ok (CT, int);
 int type_ok (CT, int);
-void set_endian (void);
 void flush_errors (void);
 
-/* mhshowsbr.c */
-void show_all_messages (CT *);
-
-/* mhlistsbr.c */
-void list_all_messages (CT *, int, int, int, int);
-
 /* mhstoresbr.c */
-void store_all_messages (CT *);
+typedef struct mhstoreinfo *mhstoreinfo_t;
+mhstoreinfo_t mhstoreinfo_create(CT *, char *, const char *, int, int);
+void mhstoreinfo_free(mhstoreinfo_t);
+void store_all_messages (mhstoreinfo_t);
 
 /* mhcachesbr.c */
 void cache_all_messages (CT *);
 
 /* mhfree.c */
-void free_content (CT);
 extern CT *cts;
 void freects_done (int) NORETURN;
 
 /*
  * static prototypes
  */
-static RETSIGTYPE pipeser (int);
+static void pipeser (int);
 
 
 int
 main (int argc, char **argv)
 {
-    int sizesw = 1, headsw = 1;
+    int sizesw = 1, headsw = 1, autosw = 0;
     int msgnum, *icachesw;
     char *cp, *file = NULL, *folder = NULL;
     char *maildir, buf[100], **argp;
     char **arguments;
+    char *cwd;
     struct msgs_array msgs = { 0, 0, NULL };
     struct msgs *mp = NULL;
     CT ct, *ctp;
     FILE *fp;
+    mhstoreinfo_t info;
+
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     done=freects_done;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
-
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
@@ -245,10 +176,10 @@ main (int argc, char **argv)
 		snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
 			invo_name);
 		print_help (buf, switches, 1);
-		done (1);
+		done (0);
 	    case VERSIONSW:
 		print_version(invo_name);
-		done (1);
+		done (0);
 
 	    case AUTOSW:
 		autosw++;
@@ -304,20 +235,6 @@ do_cache:
 		listsw = 0;
 		continue;
 
-	    case PAUSESW:
-		pausesw = 1;
-		continue;
-	    case NPAUSESW:
-		pausesw = 0;
-		continue;
-
-	    case SERIALSW:
-		serialsw = 1;
-		continue;
-	    case NSERIALSW:
-		serialsw = 0;
-		continue;
-
 	    case SHOWSW:
 		showsw = 1;
 		continue;
@@ -403,12 +320,6 @@ do_cache:
 	    case NRFC934SW:
 		rfc934sw = -1;
 		continue;
-	    case EBCDICSW:
-		ebcdicsw = 1;
-		continue;
-	    case NEBCDICSW:
-		ebcdicsw = -1;
-		continue;
 
 	    case VERBSW: 
 		verbosw = 1;
@@ -434,14 +345,6 @@ do_cache:
     parts[npart] = NULL;
     types[ntype] = NULL;
 
-    set_endian ();
-
-    if ((cp = getenv ("MM_NOASK")) && !strcmp (cp, "1")) {
-	nolist  = 1;
-	listsw  = 0;
-	pausesw = 0;
-    }
-
     /*
      * Check if we've specified an additional profile
      */
@@ -474,17 +377,7 @@ do_cache:
     /*
      * Cache the current directory before we do any chdirs()'s.
      */
-    cwd = getcpy (pwd());
-
-    /*
-     * Check for storage directory.  If specified,
-     * then store temporary files there.  Else we
-     * store them in standard nmh directory.
-     */
-    if ((cp = context_find (nmhstorage)) && *cp)
-	tmp = concat (cp, "/", invo_name, NULL);
-    else
-	tmp = add (m_maildir (invo_name), NULL);
+    cwd = add(pwd(), NULL);
 
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
@@ -506,11 +399,6 @@ do_cache:
 	vecp = 0;
 	vec[vecp++] = "mhbuild";
 
-	if (ebcdicsw == 1)
-	    vec[vecp++] = "-ebcdicsafe";
-	else if (ebcdicsw == -1)
-	    vec[vecp++] = "-noebcdicsafe";
-
 	if (rfc934sw == 1)
 	    vec[vecp++] = "-rfc934mode";
 	else if (rfc934sw == -1)
@@ -538,11 +426,6 @@ do_cache:
 	vecp = 0;
 	vec[vecp++] = "mhbuild";
 
-	if (ebcdicsw == 1)
-	    vec[vecp++] = "-ebcdicsafe";
-	else if (ebcdicsw == -1)
-	    vec[vecp++] = "-noebcdicsafe";
-
 	if (rfc934sw == 1)
 	    vec[vecp++] = "-rfc934mode";
 	else if (rfc934sw == -1)
@@ -567,7 +450,7 @@ do_cache:
 	    adios (NULL, "out of memory");
 	ctp = cts;
 
-	if ((ct = parse_mime (file)));
+	if ((ct = parse_mime (file)))
 	    *ctp++ = ct;
     } else {
 	/*
@@ -583,7 +466,7 @@ do_cache:
 	    adios (maildir, "unable to change directory to");
 
 	/* read folder and create message structure */
-	if (!(mp = folder_read (folder)))
+	if (!(mp = folder_read (folder, 1)))
 	    adios (NULL, "unable to read folder %s", folder);
 
 	/* check for empty folder */
@@ -648,13 +531,24 @@ do_cache:
      * List the message content
      */
     if (listsw)
-	list_all_messages (cts, headsw, sizesw, verbosw, debugsw);
+	list_all_messages (cts, headsw, sizesw, verbosw, debugsw, 0);
 
     /*
      * Store the message content
      */
-    if (storesw)
-	store_all_messages (cts);
+    if (storesw) {
+	info = mhstoreinfo_create (cts, cwd, "always", autosw, verbosw);;
+	store_all_messages (info);
+	mhstoreinfo_free (info);
+    }
+
+    /* If reading from a folder, do some updating */
+    if (mp) {
+	context_replace (pfolder, folder);/* update current folder  */
+	seq_setcur (mp, mp->hghsel);	  /* update current message */
+	seq_save (mp);			  /* synchronize sequences  */
+	context_save ();		  /* save the context file  */
+    }
 
     /*
      * Cache the message content
@@ -666,33 +560,24 @@ do_cache:
      * Show the message content
      */
     if (showsw)
-	show_all_messages (cts);
+	show_all_messages (cts, 0, 0, 0, NULL);
 
     /* Now free all the structures for the content */
     for (ctp = cts; *ctp; ctp++)
 	free_content (*ctp);
 
-    free ((char *) cts);
+    free (cts);
     cts = NULL;
 
-    /* If reading from a folder, do some updating */
-    if (mp) {
-	context_replace (pfolder, folder);/* update current folder  */
-	seq_setcur (mp, mp->hghsel);	  /* update current message */
-	seq_save (mp);			  /* synchronize sequences  */
-	context_save ();		  /* save the context file  */
-    }
-
     done (0);
     return 1;
 }
 
 
-static RETSIGTYPE
+static void
 pipeser (int i)
 {
     if (i == SIGQUIT) {
-	unlink ("core");
 	fflush (stdout);
 	fprintf (stderr, "\n");
 	fflush (stderr);
