@@ -2,8 +2,6 @@
 /*
  * mhshow.c -- display the contents of MIME messages
  *
- * $Id: mhshow.c,v 1.15 2007/11/04 11:54:35 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -13,8 +11,6 @@
 #include <fcntl.h>
 #include <h/signals.h>
 #include <h/md5.h>
-#include <errno.h>
-#include <signal.h>
 #include <h/mts.h>
 #include <h/tws.h>
 #include <h/mime.h>
@@ -22,68 +18,46 @@
 #include <h/mhcachesbr.h>
 #include <h/utils.h>
 
-#ifdef HAVE_SYS_WAIT_H
-# include <sys/wait.h>
-#endif
+#define MHSHOW_SWITCHES \
+    X("check", 0, CHECKSW) \
+    X("nocheck", 0, NCHECKSW) \
+    X("verbose", 0, VERBSW) \
+    X("noverbose", 0, NVERBSW) \
+    X("concat", 0, CONCATSW) \
+    X("noconcat", 0, NCONCATSW) \
+    X("textonly", 0, TEXTONLYSW) \
+    X("notextonly", 0, NTEXTONLYSW) \
+    X("inlineonly", 0, INLINESW) \
+    X("noinlineonly", 0, NINLINESW) \
+    X("file file", 0, FILESW) \
+    X("form formfile", 0, FORMSW) \
+    X("markform formfile", 0, MARKFORMSW) \
+    X("part number", 0, PARTSW) \
+    X("type content", 0, TYPESW) \
+    X("rcache policy", 0, RCACHESW) \
+    X("wcache policy", 0, WCACHESW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+    /*					\
+     * switches for moreproc/mhlproc	\
+     */					\
+    X("moreproc program", -4, PROGSW) \
+    X("nomoreproc", -3, NPROGSW) \
+    X("length lines", -4, LENSW) \
+    X("width columns", -4, WIDTHSW) \
+    /*				\
+     * switches for debugging	\
+     */				\
+    X("debug", -5, DEBUGSW) \
 
-static struct swit switches[] = {
-#define	CHECKSW                 0
-    { "check", 0 },
-#define	NCHECKSW                1
-    { "nocheck", 0 },
-#define	PAUSESW                 2
-    { "pause", 0 },
-#define	NPAUSESW                3
-    { "nopause", 0 },
-#define	SERIALSW                4
-    { "serialonly", 0 },
-#define	NSERIALSW               5
-    { "noserialonly", 0 },
-#define	VERBSW                  6
-    { "verbose", 0 },
-#define	NVERBSW                 7
-    { "noverbose", 0 },
-#define	FILESW                  8	/* interface from show */
-    { "file file", 0 },
-#define	FORMSW                  9
-    { "form formfile", 0 },
-#define	PARTSW                 10
-    { "part number", 0 },
-#define	TYPESW                 11
-    { "type content", 0 },
-#define	RCACHESW               12
-    { "rcache policy", 0 },
-#define	WCACHESW               13
-    { "wcache policy", 0 },
-#define VERSIONSW              14
-    { "version", 0 },
-#define	HELPSW                 15
-    { "help", 0 },
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(MHSHOW);
+#undef X
 
-/*
- * switches for moreproc/mhlproc
- */
-#define	PROGSW                 16
-    { "moreproc program", -4 },
-#define	NPROGSW                17
-    { "nomoreproc", -3 },
-#define	LENSW                  18
-    { "length lines", -4 },
-#define	WIDTHSW                19
-    { "width columns", -4 },
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(MHSHOW, switches);
+#undef X
 
-/*
- * switches for debugging
- */
-#define	DEBUGSW                20
-    { "debug", -5 },
-    { NULL, 0 }
-};
-
-
-/* mhparse.c */
-extern int checksw;
-extern char *tmp;	/* directory to place temp files */
 
 /* mhcachesbr.c */
 extern int rcachesw;
@@ -92,10 +66,7 @@ extern char *cache_public;
 extern char *cache_private;
 
 /* mhshowsbr.c */
-extern int pausesw;
-extern int serialsw;
 extern char *progsw;
-extern int nolist;
 extern int nomore;	/* flags for moreproc/header display */
 extern char *formsw;
 
@@ -117,28 +88,23 @@ CT parse_mime (char *);
 /* mhmisc.c */
 int part_ok (CT, int);
 int type_ok (CT, int);
-void set_endian (void);
 void flush_errors (void);
 
-/* mhshowsbr.c */
-void show_all_messages (CT *);
-
 /* mhfree.c */
-void free_content (CT);
 extern CT *cts;
-void freects_done (int);
+void freects_done (int) NORETURN;
 
 /*
  * static prototypes
  */
-static RETSIGTYPE pipeser (int);
+static void pipeser (int);
 
 
 int
 main (int argc, char **argv)
 {
-    int msgnum, *icachesw;
-    char *cp, *file = NULL, *folder = NULL;
+    int msgnum, *icachesw, concatsw = -1, textonly = -1, inlineonly = -1;
+    char *cp, *file = NULL, *folder = NULL, *markform = NULL;
     char *maildir, buf[100], **argp;
     char **arguments;
     struct msgs_array msgs = { 0, 0, NULL };
@@ -146,16 +112,10 @@ main (int argc, char **argv)
     CT ct, *ctp;
     FILE *fp;
 
+    if (nmh_init(argv[0], 1)) { return 1; }
+
     done=freects_done;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
-
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
@@ -175,10 +135,10 @@ main (int argc, char **argv)
 		snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
 			invo_name);
 		print_help (buf, switches, 1);
-		done (1);
+		done (0);
 	    case VERSIONSW:
 		print_version(invo_name);
-		done (1);
+		done (0);
 
 	    case RCACHESW:
 		icachesw = &rcachesw;
@@ -206,18 +166,23 @@ do_cache:
 		checksw = 0;
 		continue;
 
-	    case PAUSESW:
-		pausesw = 1;
+	    case CONCATSW:
+		concatsw = 1;
 		continue;
-	    case NPAUSESW:
-		pausesw = 0;
+	    case NCONCATSW:
+		concatsw = 0;
 		continue;
-
-	    case SERIALSW:
-		serialsw = 1;
+	    case TEXTONLYSW:
+		textonly = 1;
 		continue;
-	    case NSERIALSW:
-		serialsw = 0;
+	    case NTEXTONLYSW:
+		textonly = 0;
+		continue;
+	    case INLINESW:
+		inlineonly = 1;
+		continue;
+	    case NINLINESW:
+		inlineonly = 0;
 		continue;
 
 	    case PARTSW:
@@ -252,6 +217,11 @@ do_cache:
 		formsw = getcpy (etcpath (cp));
 		continue;
 
+	    case MARKFORMSW:
+		if (!(markform = *argp++) || *markform == '-')
+		    adios (NULL, "missing argument to %s", argp[-2]);
+		continue;
+
 	    /*
 	     * Switches for moreproc/mhlproc
 	     */
@@ -293,11 +263,16 @@ do_cache:
     parts[npart] = NULL;
     types[ntype] = NULL;
 
-    set_endian ();
+    /*
+     * If we had any specific parts or types specified, turn off text only
+     * content.
+     */
 
-    if ((cp = getenv ("MM_NOASK")) && !strcmp (cp, "1")) {
-	nolist  = 1;
-	pausesw = 0;
+    if (npart > 0 || ntype > 0) {
+	if (textonly == -1)
+	    textonly = 0;
+	if (inlineonly == -1)
+	    inlineonly = 0;
     }
 
     /*
@@ -329,16 +304,6 @@ do_cache:
 	cache_private = ".cache";
     cache_private = getcpy (m_maildir (cache_private));
 
-    /*
-     * Check for storage directory.  If specified,
-     * then store temporary files there.  Else we
-     * store them in standard nmh directory.
-     */
-    if ((cp = context_find (nmhstorage)) && *cp)
-	tmp = concat (cp, "/", invo_name, NULL);
-    else
-	tmp = add (m_maildir (invo_name), NULL);
-
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
 
@@ -353,7 +318,7 @@ do_cache:
 	    adios (NULL, "out of memory");
 	ctp = cts;
 
-	if ((ct = parse_mime (file)));
+	if ((ct = parse_mime (file)))
 	    *ctp++ = ct;
     } else {
 	/*
@@ -369,7 +334,7 @@ do_cache:
 	    adios (maildir, "unable to change directory to");
 
 	/* read folder and create message structure */
-	if (!(mp = folder_read (folder)))
+	if (!(mp = folder_read (folder, 1)))
 	    adios (NULL, "unable to read folder %s", folder);
 
 	/* check for empty folder */
@@ -433,18 +398,6 @@ do_cache:
 	}
     }
 
-    /*
-     * Show the message content
-     */
-    show_all_messages (cts);
-
-    /* Now free all the structures for the content */
-    for (ctp = cts; *ctp; ctp++)
-	free_content (*ctp);
-
-    free ((char *) cts);
-    cts = NULL;
-
     /* If reading from a folder, do some updating */
     if (mp) {
 	context_replace (pfolder, folder);/* update current folder  */
@@ -453,16 +406,33 @@ do_cache:
 	context_save ();		  /* save the context file  */
     }
 
+    if (concatsw)
+	m_popen(moreproc, 0);
+
+    /*
+     * Show the message content
+     */
+    show_all_messages (cts, concatsw, textonly, inlineonly, markform);
+
+    /* Now free all the structures for the content */
+    for (ctp = cts; *ctp; ctp++)
+	free_content (*ctp);
+
+    free ((char *) cts);
+    cts = NULL;
+
+    if (concatsw)
+	m_pclose();
+
     done (0);
     return 1;
 }
 
 
-static RETSIGTYPE
+static void
 pipeser (int i)
 {
     if (i == SIGQUIT) {
-	unlink ("core");
 	fflush (stdout);
 	fprintf (stderr, "\n");
 	fflush (stderr);
