@@ -2,8 +2,6 @@
 /*
  * mhtest.c -- test harness for MIME routines
  *
- * $Id: mhtest.c,v 1.15 2007/11/04 11:54:35 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -13,8 +11,6 @@
 #include <fcntl.h>
 #include <h/signals.h>
 #include <h/md5.h>
-#include <errno.h>
-#include <signal.h>
 #include <h/mts.h>
 #include <h/tws.h>
 #include <h/mime.h>
@@ -22,51 +18,30 @@
 #include <h/mhcachesbr.h>
 #include <h/utils.h>
 
-#ifdef HAVE_SYS_WAIT_H
-# include <sys/wait.h>
-#endif
+#define MHTEST_SWITCHES \
+    X("check", 0, CHECKSW) \
+    X("nocheck", 0, NCHECKSW) \
+    X("verbose", 0, VERBSW) \
+    X("noverbose", 0, NVERBSW) \
+    X("file file", 0, FILESW) \
+    X("outfile file", 0, OUTFILESW) \
+    X("part number", 0, PARTSW) \
+    X("type content", 0, TYPESW) \
+    X("rcache policy", 0, RCACHESW) \
+    X("wcache policy", 0, WCACHESW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+    X("debug", -5, DEBUGSW) \
 
-static struct swit switches[] = {
-#define	CHECKSW                 0
-    { "check", 0 },
-#define	NCHECKSW                1
-    { "nocheck", 0 },
-#define	VERBSW                  2
-    { "verbose", 0 },
-#define	NVERBSW                 3
-    { "noverbose", 0 },
-#define	FILESW                  4
-    { "file file", 0 },
-#define OUTFILESW               5
-    { "outfile file", 0 },
-#define	PARTSW                  6
-    { "part number", 0 },
-#define	TYPESW                  7
-    { "type content", 0 },
-#define	RCACHESW                8
-    { "rcache policy", 0 },
-#define	WCACHESW                9
-    { "wcache policy", 0 },
-#define VERSIONSW              10
-    { "version", 0 },
-#define	HELPSW                 11
-    { "help", 0 },
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(MHTEST);
+#undef X
 
-/*
- * switches for debugging
- */
-#define	DEBUGSW                12
-    { "debug", -5 },
-    { NULL, 0 }
-};
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(MHTEST, switches);
+#undef X
 
 
-int ebcdicsw = 0;	/* hack for linking purposes */
-
-/* mhparse.c */
-extern int checksw;
-extern char *tmp;	/* directory to place temp files */
-
 /* mhcachesbr.c */
 extern int rcachesw;
 extern int wcachesw;
@@ -84,8 +59,6 @@ extern int userrs;
  * This is currently needed to keep mhparse happy.
  * This needs to be changed.
  */
-pid_t xpid  = 0;
-
 int debugsw = 0;
 int verbosw = 0;
 
@@ -100,19 +73,17 @@ int output_message (CT, char *);
 /* mhmisc.c */
 int part_ok (CT, int);
 int type_ok (CT, int);
-void set_endian (void);
 void flush_errors (void);
 
 /* mhfree.c */
-void free_content (CT);
 extern CT *cts;
-void freects_done (int);
+void freects_done (int) NORETURN;
 
 /*
  * static prototypes
  */
 static int write_content (CT *, char *);
-static RETSIGTYPE pipeser (int);
+static void pipeser (int);
 
 
 int
@@ -126,16 +97,10 @@ main (int argc, char **argv)
     struct msgs *mp = NULL;
     CT ct, *ctp;
 
+    if (nmh_init(argv[0], 1)) { return 1; }
+
     done=freects_done;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
-
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
@@ -155,10 +120,10 @@ main (int argc, char **argv)
 		snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
 			invo_name);
 		print_help (buf, switches, 1);
-		done (1);
+		done (0);
 	    case VERSIONSW:
 		print_version(invo_name);
-		done (1);
+		done (0);
 
 	    case RCACHESW:
 		icachesw = &rcachesw;
@@ -240,8 +205,6 @@ do_cache:
     parts[npart] = NULL;
     types[ntype] = NULL;
 
-    set_endian ();
-
     if (outfile == NULL)
 	adios (NULL, "must specify output file");
 
@@ -254,16 +217,6 @@ do_cache:
 	cache_private = ".cache";
     cache_private = getcpy (m_maildir (cache_private));
 
-    /*
-     * Check for storage directory.  If specified,
-     * then store temporary files there.  Else we
-     * store them in standard nmh directory.
-     */
-    if ((cp = context_find (nmhstorage)) && *cp)
-	tmp = concat (cp, "/", invo_name, NULL);
-    else
-	tmp = add (m_maildir (invo_name), NULL);
-
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
 
@@ -278,7 +231,7 @@ do_cache:
 	    adios (NULL, "out of memory");
 	ctp = cts;
 
-	if ((ct = parse_mime (file)));
+	if ((ct = parse_mime (file)))
 	    *ctp++ = ct;
     } else {
 	/*
@@ -294,7 +247,7 @@ do_cache:
 	    adios (maildir, "unable to change directory to");
 
 	/* read folder and create message structure */
-	if (!(mp = folder_read (folder)))
+	if (!(mp = folder_read (folder, 1)))
 	    adios (NULL, "unable to read folder %s", folder);
 
 	/* check for empty folder */
@@ -384,11 +337,10 @@ write_content (CT *cts, char *outfile)
 }
 
 
-static RETSIGTYPE
+static void
 pipeser (int i)
 {
     if (i == SIGQUIT) {
-	unlink ("core");
 	fflush (stdout);
 	fprintf (stderr, "\n");
 	fflush (stderr);
