@@ -2,8 +2,6 @@
 /*
  * msgchk.c -- check for mail
  *
- * $Id: msgchk.c,v 1.13 2007/11/04 11:54:35 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -14,37 +12,7 @@
 #include <h/tws.h>
 #include <pwd.h>
 
-#ifdef POP
-# include <h/popsbr.h>
-#endif
-
-#ifdef HESIOD
-# include <hesiod.h>
-#endif
-
-#ifndef	POP
-# define POPminc(a) (a)
-#else
-# define POPminc(a)  0
-#endif
-
-#ifndef	RPOP
-# define RPOPminc(a) (a)
-#else
-# define RPOPminc(a)  0
-#endif
-
-#ifndef	APOP
-# define APOPminc(a) (a)
-#else
-# define APOPminc(a)  0
-#endif
-
-#ifndef	KPOP
-# define KPOPminc(a) (a)
-#else
-# define KPOPminc(a)  0
-#endif
+#include <h/popsbr.h>
 
 #ifndef CYRUS_SASL
 # define SASLminc(a) (a)
@@ -52,43 +20,28 @@
 # define SASLminc(a)  0
 #endif
 
-static struct swit switches[] = {
-#define	DATESW                   0
-    { "date", 0 },
-#define	NDATESW                  1
-    { "nodate", 0 },
-#define	NOTESW                   2
-    { "notify type", 0 },
-#define	NNOTESW                  3
-    { "nonotify type", 0 },
-#define	HOSTSW                   4
-    { "host hostname", POPminc (-4) },
-#define	USERSW                   5
-    { "user username", POPminc (-4) },
-#define	APOPSW                   6
-    { "apop", APOPminc (-4) },
-#define	NAPOPSW                  7
-    { "noapop", APOPminc (-6) },
-#define	RPOPSW                   8
-    { "rpop", RPOPminc (-4) },
-#define	NRPOPSW                  9
-    { "norpop", RPOPminc (-6) },
-#define VERSIONSW               10
-    { "version", 0 },
-#define	HELPSW                  11
-    { "help", 0 },
-#define SNOOPSW                 12
-    { "snoop", -5 },
-#define KPOPSW                  13
-    { "kpop", KPOPminc (-4) },
-#define SASLSW                  14
-    { "sasl", SASLminc(-4) },
-#define SASLMECHSW		15
-    { "saslmech", SASLminc(-5) },
-#define PROXYSW 		16
-    { "proxy command", POPminc(-5) },
-    { NULL, 0 }
-};
+#define MSGCHK_SWITCHES \
+    X("date", 0, DATESW) \
+    X("nodate", 0, NDATESW) \
+    X("notify type", 0, NOTESW) \
+    X("nonotify type", 0, NNOTESW) \
+    X("host hostname", 0, HOSTSW) \
+    X("user username", 0, USERSW) \
+    X("port name/number", 0, PORTSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+    X("snoop", -5, SNOOPSW) \
+    X("sasl", SASLminc(-4), SASLSW) \
+    X("saslmech", SASLminc(-5), SASLMECHSW) \
+    X("proxy command", 0, PROXYSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(MSGCHK);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(MSGCHK, switches);
+#undef X
 
 /*
  * Maximum numbers of users we can check (plus
@@ -97,6 +50,8 @@ static struct swit switches[] = {
 #define MAXVEC  51
 
 #define	NT_NONE	0x0
+#ifdef NT_NONE
+#endif /* Use NT_NONE to prevent warning from gcc -Wunused-macros. */
 #define	NT_MAIL	0x1
 #define	NT_NMAI	0x2
 #define	NT_ALL	(NT_MAIL | NT_NMAI)
@@ -115,52 +70,28 @@ static struct swit switches[] = {
  */
 static int donote (char *, int);
 static int checkmail (char *, char *, int, int, int);
-
-#ifdef POP
-static int remotemail (char *, char *, char *, int, int, int, int, int, int, char *);
-#endif
+static int remotemail (char *, char *, char *, char *, int, int, int, int,
+		       char *);
 
 
 int
 main (int argc, char **argv)
 {
     int datesw = 1, notifysw = NT_ALL;
-    int rpop, status = 0;
-    int kpop = 0, sasl = 0;
+    int status = 0, sasl = 0;
     int snoop = 0, vecp = 0;
-    uid_t uid;
-    char *cp, *host = NULL, *user, *proxy = NULL; 
+    char *cp, *host = NULL, *port = NULL, *user = NULL, *proxy = NULL;
     char buf[BUFSIZ], *saslmech = NULL; 
     char **argp, **arguments, *vec[MAXVEC];
     struct passwd *pw;
 
-#ifdef HESIOD
-    struct hes_postoffice *po;
-    char *tmphost;
-#endif
-
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     mts_init (invo_name);
-    uid = getuid ();
-    user = getusername();
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
-#ifdef POP
-    if ((cp = getenv ("MHPOPDEBUG")) && *cp)
-	snoop++;
-#endif
-
-    rpop = 0;
-
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -174,10 +105,10 @@ main (int argc, char **argv)
 		    snprintf (buf, sizeof(buf), "%s [switches] [users ...]",
 			invo_name);
 		    print_help (buf, switches, 1);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case DATESW:
 		    datesw++;
@@ -201,31 +132,19 @@ main (int argc, char **argv)
 		    if (!(host = *argp++) || *host == '-')
 			adios (NULL, "missing argument to %s", argp[-2]);
 		    continue;
+
+		case PORTSW:
+		    if (!(port = *argp++) || *port == '-')
+			adios (NULL, "missing argument to %s", argp[-2]);
+		continue;
+
 		case USERSW: 
 		    if (!(cp = *argp++) || *cp == '-')
 			adios (NULL, "missing argument to %s", argp[-2]);
 		    if (vecp >= MAXVEC-1)
 			adios (NULL, "you can only check %d users at a time", MAXVEC-1);
 		    else
-			vec[vecp++] = cp;
-		    continue;
-
-		case APOPSW: 
-		    rpop = -1;
-		    continue;
-		case NAPOPSW:
-		    rpop = 0;
-		    continue;
-
-		case RPOPSW: 
-		    rpop = 1;
-		    continue;
-		case NRPOPSW: 
-		    rpop = 0;
-		    continue;
-
-		case KPOPSW:
-		    kpop = 1;
+			user = vec[vecp++] = cp;
 		    continue;
 
 		case SNOOPSW:
@@ -253,25 +172,10 @@ main (int argc, char **argv)
 	    vec[vecp++] = cp;
     }
 
-#ifdef POP
     /*
      * If -host is not specified by user
      */
     if (!host || !*host) {
-# ifdef HESIOD
-	/*
-	 * Scheme is:
-	 *        use MAILHOST environment variable if present,
-	 *  else try Hesiod.
-	 *  If that fails, use the default (if any)
-	 *  provided by mts.conf in mts_init()
-	 */
-	if ((tmphost = getenv("MAILHOST")) != NULL)
-	    pophost = tmphost;
-	else if ((po = hes_getmailhost(vecp ? vec[0] : user)) != NULL &&
-		strcmp(po->po_type, "POP") == 0)
-	    pophost = po->po_host;
-# endif /* HESIOD */
 	/*
 	 * If "pophost" is specified in mts.conf,
 	 * use it as default value.
@@ -281,66 +185,59 @@ main (int argc, char **argv)
     }
     if (!host || !*host)
 	host = NULL;
-    if (!host || rpop <= 0)
-	setuid (uid);
-#endif /* POP */
 
     if (vecp != 0)
 	vec[vecp] = NULL;
 
-#ifdef POP
     if (host) {
-	if ( strcmp( POPSERVICE, "kpop" ) == 0 ) {
-	    kpop = 1;
-	}
 	if (vecp == 0) {
-	    status = remotemail (host, user, proxy, rpop, kpop, notifysw, 1,
+	    status = remotemail (host, port, user, proxy, notifysw, 1,
 				 snoop, sasl, saslmech);
 	} else {
 	    for (vecp = 0; vec[vecp]; vecp++)
-		status += remotemail (host, vec[vecp], proxy, rpop, kpop,
-		                      notifysw, 0, snoop, sasl, saslmech);
+		status += remotemail (host, port, vec[vecp], proxy, notifysw, 0,
+				      snoop, sasl, saslmech);
 	}
     } else {
-#endif /* POP */
+	if (user == NULL) user = getusername ();
+	if (vecp == 0) {
+	    char *home;
 
-    if (vecp == 0) {
-	char *home;
-
-        /* Not sure this check makes sense... */
-	if (!geteuid() || NULL == (home = getenv("HOME"))) {
-	    pw = getpwnam (user);
-	    if (pw == NULL)
-		adios (NULL, "unable to get information about user");
-	    home = pw->pw_dir;
+	    /* Not sure this check makes sense... */
+	    if (!geteuid() || NULL == (home = getenv("HOME"))) {
+		pw = getpwnam (user);
+		if (pw == NULL)
+		    adios (NULL, "unable to get information about user");
+		home = pw->pw_dir;
+	    }
+	    status = checkmail (user, home, datesw, notifysw, 1);
+	} else {
+	    for (vecp = 0; vec[vecp]; vecp++) {
+		if ((pw = getpwnam (vec[vecp])))
+		    status += checkmail (pw->pw_name, pw->pw_dir, datesw, notifysw, 0);
+		else
+		    advise (NULL, "no such user as %s", vec[vecp]);
+	    }
 	}
-	status = checkmail (user, home, datesw, notifysw, 1);
-    } else {
-	for (vecp = 0; vec[vecp]; vecp++) {
-	    if ((pw = getpwnam (vec[vecp])))
-		status += checkmail (pw->pw_name, pw->pw_dir, datesw, notifysw, 0);
-	    else
-		advise (NULL, "no such user as %s", vec[vecp]);
-	}
-    }
-#ifdef POP
     }		/* host == NULL */
-#endif
 
     done (status);
     return 1;
 }
 
 
-static struct swit ntswitches[] = {
-#define	NALLSW     0
-    { "all", 0 },
-#define	NMAISW     1
-    { "mail", 0 },
-#define	NNMAISW	   2
-    { "nomail", 0 },
-    { NULL, 0 }
-};
+#define NOTE_SWITCHES \
+    X("all", 0, NALLSW) \
+    X("mail", 0, NMAISW) \
+    X("nomail", 0, NNMAISW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(NOTE);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(NOTE, ntswitches);
+#undef X
 
 
 static int
@@ -382,7 +279,10 @@ checkmail (char *user, char *home, int datesw, int notifysw, int personal)
 
     if ((mf & UUCPOK) || (mf & MMDFOK)) {
 	if (notifysw & NT_MAIL) {
-	    printf (personal ? "You have " : "%s has ", user);
+	    if (personal)
+		printf ("You have ");
+	    else
+		printf ("%s has ", user);
 	    if (mf & UUCPOK)
 		printf ("%s old-style bell", mf & UUCPOLD ? "old" : "new");
 	    if ((mf & UUCPOK) && (mf & MMDFOK))
@@ -416,34 +316,32 @@ checkmail (char *user, char *home, int datesw, int notifysw, int personal)
 }
 
 
-#ifdef POP
 extern char response[];
 
 static int
-remotemail (char *host, char *user, char *proxy, int rpop, int kpop, int notifysw, int personal, int snoop, int sasl, char *saslmech)
+remotemail (char *host, char *port, char *user, char *proxy, int notifysw,
+	    int personal, int snoop, int sasl, char *saslmech)
 {
     int nmsgs, nbytes, status;
-    char *pass = NULL;
-
-    if (user == NULL)
-	user = getusername ();
-    if (kpop || sasl || (rpop > 0))
-	pass = getusername ();
-    else
-	ruserpass (host, &user, &pass);
+    struct nmh_creds creds = { 0, 0, 0 };
 
     /* open the POP connection */
-    if (pop_init (host, user, pass, proxy, snoop, kpop ? 1 : rpop, kpop,
-		  sasl, saslmech) == NOTOK
-	    || pop_stat (&nmsgs, &nbytes) == NOTOK	/* check for messages  */
-	    || pop_quit () == NOTOK) {			/* quit POP connection */
+    nmh_get_credentials (host, user, sasl, &creds);
+    if (pop_init (host, port, creds.user, creds.password, proxy, snoop, sasl,
+		  saslmech) == NOTOK
+	    || pop_stat (&nmsgs, &nbytes) == NOTOK     /* check for messages  */
+	    || pop_quit () == NOTOK) {                 /* quit POP connection */
 	advise (NULL, "%s", response);
 	return 1;
     }
 
     if (nmsgs) {
 	if (notifysw & NT_MAIL) {
-	    printf (personal ? "You have " : "%s has ", user);
+	    if (personal)
+		printf ("You have ");
+	    else
+		printf ("%s has ", user);
+
 	    printf ("%d message%s (%d bytes)",
 		    nmsgs, nmsgs != 1 ? "s" : "", nbytes);
 	}
@@ -464,4 +362,3 @@ remotemail (char *host, char *user, char *proxy, int rpop, int kpop, int notifys
 
     return status;
 }
-#endif /* POP */
