@@ -2,8 +2,6 @@
 /*
  * packf.c -- pack a nmh folder into a file
  *
- * $Id: packf.c,v 1.11 2007/11/04 11:54:35 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -13,27 +11,27 @@
 #include <fcntl.h>
 #include <h/dropsbr.h>
 #include <h/utils.h>
-#include <errno.h>
 
-static struct swit switches[] = {
-#define FILESW         0
-    { "file name", 0 },
-#define MBOXSW         1
-    { "mbox", 0 },
-#define MMDFSW         2
-    { "mmdf", 0 },
-#define VERSIONSW      3
-    { "version", 0 },
-#define	HELPSW         4
-    { "help", 0 },
-    { NULL, 0 }
-};
+#define PACKF_SWITCHES \
+    X("file name", 0, FILESW) \
+    X("mbox", 0, MBOXSW) \
+    X("mmdf", 0, MMDFSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(PACKF);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(PACKF, switches);
+#undef X
 
 static int md = NOTOK;
 static int mbx_style = MBOX_FORMAT;
 static int mapping = 0;
 
-static void mbxclose_done(int);
+static void mbxclose_done(int) NORETURN;
 
 char *file = NULL;
 
@@ -48,16 +46,10 @@ main (int argc, char **argv)
     struct msgs *mp;
     struct stat st;
 
+    if (nmh_init(argv[0], 1)) { return 1; }
+
     done=mbxclose_done;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
-
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
@@ -77,10 +69,10 @@ main (int argc, char **argv)
 		    snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
 			invo_name);
 		    print_help (buf, switches, 1);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case FILESW: 
 		    if (file)
@@ -139,7 +131,7 @@ main (int argc, char **argv)
 	adios (maildir, "unable to change directory to ");
 
     /* read folder and create message structure */
-    if (!(mp = folder_read (folder)))
+    if (!(mp = folder_read (folder, 1)))
 	adios (NULL, "unable to read folder %s", folder);
 
     /* check for empty folder */
