@@ -10,19 +10,9 @@
 #include <h/mh.h>
 #include <fcntl.h>
 #include <h/signals.h>
-#include <errno.h>
-#include <signal.h>
 #include <setjmp.h>
 
-#ifdef HAVE_TERMIOS_H
-# include <termios.h>
-#else
-# ifdef HAVE_TERMIO_H
-#  include <termio.h>
-# else
-#  include <sgtty.h>
-# endif
-#endif
+#include <termios.h>
 
 #define	QUOTE '\\'
 
@@ -34,54 +24,33 @@
 # define CERASE '#'
 #endif
 
-static struct swit switches[] = {
-#define	ERASESW	0
-    { "erase chr", 0 },
-#define	KILLSW	1
-    { "kill chr", 0 },
-#define	PREPSW	2
-    { "prepend", 0 },
-#define	NPREPSW	3
-    { "noprepend", 0 },	
-#define	RAPDSW	4
-    { "rapid", 0 },
-#define	NRAPDSW	5
-    { "norapid", 0 },
-#define	BODYSW	6
-    { "body", -4 },
-#define	NBODYSW	7
-    { "nobody", -6 },
-#define	DOTSW	8
-    { "doteof", 0 },
-#define	NDOTSW	9
-    { "nodoteof", 0 },
-#define VERSIONSW 10
-    { "version", 0 },
-#define	HELPSW	11
-    { "help", 0 },
-    { NULL, 0 }
-};
+#define PROMPTER_SWITCHES \
+    X("erase chr", 0, ERASESW) \
+    X("kill chr", 0, KILLSW) \
+    X("prepend", 0, PREPSW) \
+    X("noprepend", 0, NPREPSW) \
+    X("rapid", 0, RAPDSW) \
+    X("norapid", 0, NRAPDSW) \
+    X("body", -4, BODYSW) \
+    X("nobody", -6, NBODYSW) \
+    X("doteof", 0, DOTSW) \
+    X("nodoteof", 0, NDOTSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(PROMPTER);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(PROMPTER, switches);
+#undef X
 
 
-#ifdef HAVE_TERMIOS_H
 static struct termios tio;
-# define ERASE tio.c_cc[VERASE]
-# define KILL  tio.c_cc[VKILL]
-# define INTR  tio.c_cc[VINTR]
-#else
-# ifdef HAVE_TERMIO_H
-static struct termio tio;
-#  define ERASE tio.c_cc[VERASE]
-#  define KILL  tio.c_cc[VKILL]
-#  define INTR  tio.c_cc[VINTR]
-# else
-static struct sgttyb tio;
-static struct tchars tc;
-#  define ERASE tio.sg_erase
-#  define KILL  tio.sg_kill
-#  define INTR  tc.t_intrc
-# endif
-#endif
+#define ERASE tio.c_cc[VERASE]
+#define KILL  tio.c_cc[VKILL]
+#define INTR  tio.c_cc[VINTR]
 
 static int wtuser = 0;
 static int sigint = 0;
@@ -93,7 +62,7 @@ static jmp_buf sigenv;
 int getln (char *, int);
 static int chrcnv (char *);
 static void chrdsp (char *, char);
-static RETSIGTYPE intrser (int);
+static void intrser (int);
 
 
 int
@@ -103,18 +72,13 @@ main (int argc, char **argv)
     int doteof = 0, fdi, fdo, i, state;
     char *cp, *drft = NULL, *erasep = NULL;
     char *killp = NULL, name[NAMESZ], field[BUFSIZ];
-    char buffer[BUFSIZ], tmpfil[BUFSIZ];
+    char buffer[BUFSIZ];
     char **arguments, **argp;
     FILE *in, *out;
-    char *tfile = NULL;
+    char *tmpfil;
+    m_getfld_state_t gstate = 0;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -132,10 +96,10 @@ main (int argc, char **argv)
 		    snprintf (buffer, sizeof(buffer), "%s [switches] file",
 			invo_name);
 		    print_help (buffer, switches, 1);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case ERASESW: 
 		    if (!(erasep = *argp++) || *erasep == '-')
@@ -184,32 +148,18 @@ main (int argc, char **argv)
     if ((in = fopen (drft, "r")) == NULL)
 	adios (drft, "unable to open");
 
-    tfile = m_mktemp2(NULL, invo_name, NULL, &out);
-    if (tfile == NULL) adios("prompter", "unable to create temporary file");
-    chmod (tmpfil, 0600);
-    strncpy (tmpfil, tfile, sizeof(tmpfil));
+    if ((tmpfil = m_mktemp2(NULL, invo_name, NULL, &out)) == NULL) {
+	adios(NULL, "unable to create temporary file in %s", get_temp_dir());
+    }
 
     /*
      * Are we changing the kill or erase character?
      */
     if (killp || erasep) {
-#ifdef HAVE_TERMIOS_H
 	cc_t save_erase, save_kill;
-#else
-	int save_erase, save_kill;
-#endif
 
 	/* get the current terminal attributes */
-#ifdef HAVE_TERMIOS_H
 	tcgetattr(0, &tio);
-#else
-# ifdef HAVE_TERMIO_H
-	ioctl(0, TCGETA, &tio);
-# else
-	ioctl (0, TIOCGETP, (char *) &tio);
-	ioctl (0, TIOCGETC, (char *) &tc);
-# endif
-#endif
 
 	/* save original kill, erase character for later */
 	save_kill = KILL;
@@ -220,15 +170,7 @@ main (int argc, char **argv)
 	ERASE = erasep ? chrcnv (erasep) : save_erase;
 
 	/* set the new terminal attributes */
-#ifdef HAVE_TERMIOS_H
 	 tcsetattr(0, TCSADRAIN, &tio);
-#else
-# ifdef HAVE_TERMIO_H
-	ioctl(0, TCSETAW, &tio);
-# else
-	ioctl (0, TIOCSETN, (char *) &tio);
-# endif
-#endif
 
 	/* print out new kill erase characters */
 	chrdsp ("erase", ERASE);
@@ -252,10 +194,10 @@ main (int argc, char **argv)
     /*
      * Loop through the lines of the draft skeleton.
      */
-    for (state = FLD;;) {
-	switch (state = m_getfld (state, name, field, sizeof(field), in)) {
+    for (;;) {
+	int fieldsz = sizeof field;
+	switch (state = m_getfld (&gstate, name, field, &fieldsz, in)) {
 	    case FLD: 
-	    case FLDEOF: 
 	    case FLDPLUS: 
 		/*
 		 * Check if the value of field contains anything
@@ -270,8 +212,8 @@ main (int argc, char **argv)
 		    printf ("%s:%s", name, field);
 		    fprintf (out, "%s:%s", name, field);
 		    while (state == FLDPLUS) {
-			state =
-			    m_getfld (state, name, field, sizeof(field), in);
+			fieldsz = sizeof field;
+			state = m_getfld (&gstate, name, field, &fieldsz, in);
 			printf ("%s", field);
 			fprintf (out, "%s", field);
 		    }
@@ -283,17 +225,9 @@ main (int argc, char **argv)
 		    if (i == -1) {
 abort:
 			if (killp || erasep) {
-#ifdef HAVE_TERMIOS_H
 			    tcsetattr(0, TCSADRAIN, &tio);
-#else
-# ifdef HAVE_TERMIO
-			    ioctl (0, TCSETA, &tio);
-# else
-			    ioctl (0, TIOCSETN, (char *) &tio);
-# endif
-#endif
 			}
-			unlink (tmpfil);
+			(void) m_unlink (tmpfil);
 			done (1);
 		    }
 		    if (i != 0 || (field[0] != '\n' && field[0] != 0)) {
@@ -309,17 +243,9 @@ abort:
 		    }
 		}
 
-		if (state == FLDEOF) {	/* moby hack */
-		    fprintf (out, "--------\n");
-		    printf ("--------\n");
-		    if (!body)
-			break;
-		    goto no_body;
-		}
 		continue;
 
 	    case BODY: 
-	    case BODYEOF:
 	    case FILEEOF: 
 	        if (!body)
 	            break;
@@ -345,13 +271,14 @@ abort:
 			if (!rapid && !sigint)
 			    printf ("%s", field);
 		    } while (state == BODY &&
-			    (state = m_getfld (state, name, field, sizeof(field), in)));
+			    (fieldsz = sizeof field,
+			     state = m_getfld (&gstate, name, field, &fieldsz, in)));
 		    if (prepend || !body)
 			break;
 		    else
 			printf ("\n--------Enter additional text\n\n");
 		}
-no_body:
+
 		fflush (stdout);
 		for (;;) {
 		    getln (field, sizeof(field));
@@ -368,6 +295,7 @@ no_body:
 	}
 	break;
     }
+    m_getfld_state_destroy (&gstate);
 
     if (body)
 	printf ("--------\n");
@@ -378,15 +306,7 @@ no_body:
     SIGNAL (SIGINT, SIG_IGN);
 
     if (killp || erasep) {
-#ifdef HAVE_TERMIOS_H
 	 tcsetattr(0, TCSADRAIN, &tio);
-#else
-# ifdef HAVE_TERMIO_H
-	ioctl (0, TCSETAW, &tio);
-# else
-	ioctl (0, TIOCSETN, (char *) &tio);
-# endif
-#endif
     }
 
     if ((fdi = open (tmpfil, O_RDONLY)) == NOTOK)
@@ -396,7 +316,7 @@ no_body:
     cpydata (fdi, fdo, tmpfil, drft);
     close (fdi);
     close (fdo);
-    unlink (tmpfil);
+    (void) m_unlink (tmpfil);
 
     context_save ();	/* save the context file */
     done (0);
@@ -409,6 +329,7 @@ getln (char *buffer, int n)
 {
     int c;
     char *cp;
+    static int quoting = 0;
 
     cp = buffer;
     *cp = 0;
@@ -430,12 +351,14 @@ getln (char *buffer, int n)
     for (;;) {
 	switch (c = getchar ()) {
 	    case EOF: 
+		quoting = 0;
 		clearerr (stdin);
 		longjmp (sigenv, DONE);
 
 	    case '\n': 
-		if (cp[-1] == QUOTE) {
-		    cp[-1] = c;
+		if (quoting) {
+		    *(cp - 1) = c;
+		    quoting = 0;
 		    wtuser = 0;
 		    return 1;
 		}
@@ -445,6 +368,11 @@ getln (char *buffer, int n)
 		return 0;
 
 	    default: 
+		if (c == QUOTE) {
+		    quoting = 1;
+		} else {
+		    quoting = 0;
+		}
 		if (cp < buffer + n)
 		    *cp++ = c;
 		*cp = 0;
@@ -453,12 +381,10 @@ getln (char *buffer, int n)
 }
 
 
-static RETSIGTYPE
+static void
 intrser (int i)
 {
-#ifndef	RELIABLE_SIGNALS
-    SIGNAL (SIGINT, intrser);
-#endif
+    NMH_UNUSED (i);
 
     if (wtuser)
 	longjmp (sigenv, NOTOK);
