@@ -2,8 +2,6 @@
 /*
  * rcvpack.c -- append message to a file
  *
- * $Id: rcvpack.c,v 1.8 2007/11/04 11:54:35 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -15,17 +13,19 @@
 #include <h/tws.h>
 #include <h/mts.h>
 
-static struct swit switches[] = {
-#define MBOXSW       0
-    { "mbox", 0 },
-#define MMDFSW       1
-    { "mmdf", 0 },
-#define VERSIONSW    2
-    { "version", 0 },
-#define	HELPSW       3
-    { "help", 0 },
-    { NULL, 0 }
-};
+#define RCVPACK_SWITCHES \
+    X("mbox", 0, MBOXSW) \
+    X("mmdf", 0, MMDFSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(RCVPACK);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(RCVPACK, switches);
+#undef X
 
 /*
  * default format in which to save messages
@@ -40,13 +40,7 @@ main (int argc, char **argv)
     char *cp, *file = NULL, buf[BUFSIZ];
     char **argp, **arguments;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     mts_init (invo_name);
     arguments = getarguments (invo_name, argc, argv, 1);
@@ -65,10 +59,10 @@ main (int argc, char **argv)
 		case HELPSW: 
 		    snprintf (buf, sizeof(buf), "%s [switches] file", invo_name);
 		    print_help (buf, switches, 1);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case MBOXSW:
 		    mbx_style = MBOX_FORMAT;
