@@ -14,53 +14,41 @@
 
 #include <h/mh.h>
 #include <h/signals.h>
+#include <setjmp.h>
 #include <h/rcvmail.h>
 #include <h/scansbr.h>
 #include <h/tws.h>
 #include <h/mts.h>
-#include <signal.h>
 #include <fcntl.h>
 
-#include <utmp.h>
-
-#ifndef HAVE_GETUTENT
-# ifndef UTMP_FILE
-#  ifdef _PATH_UTMP
-#   define UTMP_FILE _PATH_UTMP
-#  else
-#   define UTMP_FILE "/etc/utmp"
-#  endif
-# endif
-#endif
+#ifdef HAVE_GETUTXENT
+#include <utmpx.h>
+#endif /* HAVE_GETUTXENT */
 
 #define	SCANFMT	\
 "%2(hour{dtimenow}):%02(min{dtimenow}): %<(size)%5(size) %>%<{encrypted}E%>\
 %<(mymbox{from})%<{to}To:%14(friendly{to})%>%>%<(zero)%17(friendly{from})%>  \
 %{subject}%<{body}<<%{body}>>%>"
 
-static struct swit switches[] = {
-#define	BIFFSW	0
-    { "biff", 0 },
-#define	FORMSW	1
-    { "form formatfile", 0 },
-#define	FMTSW	2
-    { "format string", 5 },
-#define WIDTHSW 3
-    { "width columns", 0 },
-#define NLSW    4
-    { "newline", 0 },
-#define NNLSW   5
-    { "nonewline", 0 },
-#define BELSW	6
-    { "bell", 0 },
-#define	NBELSW	7
-    { "nobell", 0 },
-#define VERSIONSW 8
-    { "version", 0 },
-#define	HELPSW	9
-    { "help", 0 },
-    { NULL, 0 }
-};
+#define RCVTTY_SWITCHES \
+    X("biff", 0, BIFFSW) \
+    X("form formatfile", 0, FORMSW) \
+    X("format string", 5, FMTSW) \
+    X("width columns", 0, WIDTHSW) \
+    X("newline", 0, NLSW) \
+    X("nonewline", 0, NNLSW) \
+    X("bell", 0, BELSW) \
+    X("nobell", 0, NBELSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(RCVTTY);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(RCVTTY, switches);
+#undef X
 
 static jmp_buf myctx;
 static int bell = 1;
@@ -78,10 +66,12 @@ char *getusername(void);
 /*
  * static prototypes
  */
-static RETSIGTYPE alrmser (int);
+static void alrmser (int);
 static int message_fd (char **);
 static int header_fd (void);
+#if HAVE_GETUTXENT
 static void alert (char *, int);
+#endif /* HAVE_GETUTXENT */
 
 
 int
@@ -90,20 +80,9 @@ main (int argc, char **argv)
     int md, vecp = 0;
     char *cp, *user, buf[BUFSIZ], tty[BUFSIZ];
     char **argp, **arguments, *vec[MAXARGS];
-#ifdef HAVE_GETUTENT
-    struct utmp * utp;
-#else
-    struct utmp ut;
-    register FILE *uf;
-#endif
+    struct utmpx *utp;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     mts_init (invo_name);
     arguments = getarguments (invo_name, argc, argv, 1);
@@ -122,10 +101,10 @@ main (int argc, char **argv)
 		case HELPSW: 
 		    snprintf (buf, sizeof(buf), "%s [command ...]", invo_name);
 		    print_help (buf, switches, 1);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case BIFFSW:
 		    biff = 1;
@@ -171,45 +150,30 @@ main (int argc, char **argv)
 
     user = getusername();
 
-#ifdef HAVE_GETUTENT
-    setutent();
-    while ((utp = getutent()) != NULL) {
-        if (
-#ifdef HAVE_STRUCT_UTMP_UT_TYPE
-	       utp->ut_type == USER_PROCESS 
-	       &&
-#endif
-               utp->ut_name[0] != 0
+#if HAVE_GETUTXENT
+    setutxent();
+    while ((utp = getutxent()) != NULL) {
+        if (utp->ut_type == USER_PROCESS && utp->ut_user[0] != 0
                && utp->ut_line[0] != 0
-               && strncmp (user, utp->ut_name, sizeof(utp->ut_name)) == 0) {
+               && strncmp (user, utp->ut_user, sizeof(utp->ut_user)) == 0) {
             strncpy (tty, utp->ut_line, sizeof(utp->ut_line));
 	    alert (tty, md);
         }
     }
-    endutent();
+    endutxent();
 #else
-    if ((uf = fopen (UTMP_FILE, "r")) == NULL)
-	exit (RCV_MBX);
-    while (fread ((char *) &ut, sizeof(ut), 1, uf) == 1)
-	if (ut.ut_name[0] != 0
-		&& strncmp (user, ut.ut_name, sizeof(ut.ut_name)) == 0) {
-	    strncpy (tty, ut.ut_line, sizeof(ut.ut_line));
-	    alert (tty, md);
-	}
-    fclose (uf);
-#endif
+    NMH_UNUSED (tty);
+    NMH_UNUSED (utp);
+#endif /* HAVE_GETUTXENT */
 
     exit (RCV_MOK);
-    return 0;  /* dead code to satisfy the compiler */
 }
 
 
-static RETSIGTYPE
+static void
 alrmser (int i)
 {
-#ifndef RELIABLE_SIGNALS
-    SIGNAL (SIGALRM, alrmser);
-#endif
+    NMH_UNUSED (i);
 
     longjmp (myctx, 1);
 }
@@ -219,20 +183,18 @@ static int
 message_fd (char **vec)
 {
     pid_t child_id;
-    int bytes, fd, seconds;
-    char tmpfil[BUFSIZ];
+    int bytes, seconds;
+    int fd;
+    char *tfile;
     struct stat st;
 
-#ifdef HAVE_MKSTEMP
-    fd = mkstemp (strncpy (tmpfil, "/tmp/rcvttyXXXXX", sizeof(tmpfil)));
-#else
-    unlink (mktemp (strncpy (tmpfil, "/tmp/rcvttyXXXXX", sizeof(tmpfil))));
-    if ((fd = open (tmpfil, O_RDWR | O_CREAT | O_TRUNC, 0600)) == NOTOK)
-	return header_fd ();
-#endif
-    unlink (tmpfil);
+    if ((tfile = m_mktemp2(NULL, invo_name, &fd, NULL)) == NULL) {
+	advise(NULL, "unable to create temporary file in %s", get_temp_dir());
+	return NOTOK;
+    }
+    (void) m_unlink(tfile);  /* Use fd, no longer need the file name. */
 
-    if ((child_id = vfork()) == NOTOK) {
+    if ((child_id = fork()) == NOTOK) {
 	/* fork error */
 	close (fd);
 	return header_fd ();
@@ -263,7 +225,7 @@ message_fd (char **vec)
 	     * Ruthlessly kill the child and anything
 	     * else in its process group.
 	     */
-	    KILLPG(child_id, SIGKILL);
+	    killpg(child_id, SIGKILL);
 	}
 	close (fd);
 	return header_fd ();
@@ -275,9 +237,11 @@ message_fd (char **vec)
 	_exit (-1);
     closefds (3);
     setpgid ((pid_t) 0, getpid ());	/* put in own process group */
-    execvp (vec[0], vec);
-    _exit (-1);
-    return 1;  /* dead code to satisfy compiler */
+    if (execvp (vec[0], vec) == NOTOK) {
+        _exit (-1);
+    }
+
+    return NOTOK;
 }
 
 
@@ -288,15 +252,18 @@ header_fd (void)
     char *nfs;
     char *tfile = NULL;
 
-    tfile = m_mktemp2(NULL, invo_name, &fd, NULL);
-    if (tfile == NULL) return NOTOK;
-    unlink (tfile);
+    if ((tfile = m_mktemp2(NULL, invo_name, &fd, NULL)) == NULL) {
+	advise(NULL, "unable to create temporary file in %s", get_temp_dir());
+        return NOTOK;
+    }
+    (void) m_unlink(tfile);  /* Use fd, no longer need the file name. */
 
     rewind (stdin);
 
     /* get new format string */
     nfs = new_fs (form, format, SCANFMT);
     scan (stdin, 0, 0, nfs, width, 0, 0, NULL, 0L, 0);
+    scan_finished ();
     if (newline)
         write (fd, "\n\r", 2);
     write (fd, scanl, strlen (scanl));
@@ -307,6 +274,7 @@ header_fd (void)
 }
 
 
+#if HAVE_GETUTXENT
 static void
 alert (char *tty, int md)
 {
@@ -344,4 +312,4 @@ alert (char *tty, int md)
 
     close (td);
 }
-
+#endif /* HAVE_GETUTXENT */
