@@ -11,37 +11,31 @@
 #include <h/mh.h>
 #include <h/utils.h>
 #include <fcntl.h>
-#include <errno.h>
 
-static struct swit switches[] = {
-#define	DRAFTSW          0
-    { "draft", 0 },
-#define	LINKSW           1
-    { "link", 0 },
-#define	NLINKSW          2
-    { "nolink", 0 },
-#define	PRESSW           3
-    { "preserve", 0 },
-#define	NPRESSW          4
-    { "nopreserve", 0 },
-#define UNLINKSW         5
-    { "unlink", 0 },
-#define NUNLINKSW        6
-    { "nounlink", 0 },
-#define	SRCSW            7
-    { "src +folder", 0 },
-#define	FILESW           8
-    { "file file", 0 },
-#define	RPROCSW          9
-    { "rmmproc program", 0 },
-#define	NRPRCSW         10
-    { "normmproc", 0 },
-#define VERSIONSW       11
-    { "version", 0 },
-#define	HELPSW          12
-    { "help", 0 },
-    { NULL, 0 }
-};
+#define REFILE_SWITCHES \
+    X("draft", 0, DRAFTSW) \
+    X("link", 0, LINKSW) \
+    X("nolink", 0, NLINKSW) \
+    X("preserve", 0, PRESSW) \
+    X("nopreserve", 0, NPRESSW) \
+    X("retainsequences", 0, RETAINSEQSSW) \
+    X("noretainsequences", 0, NRETAINSEQSSW) \
+    X("unlink", 0, UNLINKSW) \
+    X("nounlink", 0, NUNLINKSW) \
+    X("src +folder", 0, SRCSW) \
+    X("file file", 0, FILESW) \
+    X("rmmproc program", 0, RPROCSW) \
+    X("normmproc", 0, NRPRCSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(REFILE);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(REFILE, switches);
+#undef X
 
 static char maildir[BUFSIZ];
 
@@ -56,13 +50,14 @@ struct st_fold {
 static void opnfolds (struct st_fold *, int);
 static void clsfolds (struct st_fold *, int);
 static void remove_files (int, char **);
-static int m_file (char *, struct st_fold *, int, int, int);
+static int m_file (struct msgs *, char *, int, struct st_fold *, int, int, int);
+static void copy_seqs (struct msgs *, int, struct msgs *, int);
 
 
 int
 main (int argc, char **argv)
 {
-    int	linkf = 0, preserve = 0, filep = 0;
+    int	linkf = 0, preserve = 0, retainseqs = 0, filep = 0;
     int foldp = 0, isdf = 0, unlink_msgs = 0;
     int i, msgnum;
     char *cp, *folder = NULL, buf[BUFSIZ];
@@ -73,13 +68,7 @@ main (int argc, char **argv)
     struct msgs_array msgs = { 0, 0, NULL };
     struct msgs *mp;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -90,35 +79,42 @@ main (int argc, char **argv)
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
-	    case AMBIGSW: 
+	    case AMBIGSW:
 		ambigsw (cp, switches);
 		done (1);
-	    case UNKWNSW: 
+	    case UNKWNSW:
 		adios (NULL, "-%s unknown\n", cp);
 
-	    case HELPSW: 
+	    case HELPSW:
 		snprintf (buf, sizeof(buf), "%s [msgs] [switches] +folder ...",
 			  invo_name);
 		print_help (buf, switches, 1);
-		done (1);
+		done (0);
 	    case VERSIONSW:
 		print_version(invo_name);
-		done (1);
+		done (0);
 
-	    case LINKSW: 
+	    case LINKSW:
 		linkf++;
 		continue;
-	    case NLINKSW: 
+	    case NLINKSW:
 		linkf = 0;
 		continue;
 
-	    case PRESSW: 
+	    case PRESSW:
 		preserve++;
 		continue;
-	    case NPRESSW: 
+	    case NPRESSW:
 		preserve = 0;
 		continue;
 
+	    case RETAINSEQSSW:
+		retainseqs = 1;
+		continue;
+	    case NRETAINSEQSSW:
+		retainseqs = 0;
+		continue;
+
 	    case UNLINKSW:
 		unlink_msgs++;
 		continue;
@@ -126,7 +122,7 @@ main (int argc, char **argv)
 		unlink_msgs = 0;
 		continue;
 
-	    case SRCSW: 
+	    case SRCSW:
 		if (folder)
 		    adios (NULL, "only one source folder at a time!");
 		if (!(cp = *argp++) || *cp == '-')
@@ -140,7 +136,7 @@ main (int argc, char **argv)
 		isdf = 0;
 		files[filep++] = getcpy (m_draft (NULL, NULL, 1, &isdf));
 		continue;
-	    case FILESW: 
+	    case FILESW:
 		if (filep > NFOLDERS)
 		    adios (NULL, "only %d files allowed!", NFOLDERS);
 		if (!(cp = *argp++) || *cp == '-')
@@ -148,11 +144,11 @@ main (int argc, char **argv)
 		files[filep++] = path (cp, TFILE);
 		continue;
 
-	    case RPROCSW: 
+	    case RPROCSW:
 		if (!(rmmproc = *argp++) || *rmmproc == '-')
 		    adios (NULL, "missing argument to %s", argp[-2]);
 		continue;
-	    case NRPRCSW: 
+	    case NRPRCSW:
 		rmmproc = NULL;
 		continue;
 	    }
@@ -171,11 +167,6 @@ main (int argc, char **argv)
     if (foldp == 0)
 	adios (NULL, "no folder specified");
 
-#ifdef WHATNOW
-    if (!msgs.size && !foldp && !filep && (cp = getenv ("mhdraft")) && *cp)
-	files[filep++] = cp;
-#endif /* WHATNOW */
-
     /*
      * We are refiling a file to the folders
      */
@@ -184,7 +175,7 @@ main (int argc, char **argv)
 	    adios (NULL, "use -file or some messages, not both");
 	opnfolds (folders, foldp);
 	for (i = 0; i < filep; i++)
-	    if (m_file (files[i], folders, foldp, preserve, 0))
+	    if (m_file (0, files[i], 0, folders, foldp, preserve, 0))
 		done (1);
 	/* If -nolink, then "remove" files */
 	if (!linkf)
@@ -202,7 +193,7 @@ main (int argc, char **argv)
 	adios (maildir, "unable to change directory to");
 
     /* read source folder and create message structure */
-    if (!(mp = folder_read (folder)))
+    if (!(mp = folder_read (folder, 1)))
 	adios (NULL, "unable to read folder %s", folder);
 
     /* check for empty folder */
@@ -227,23 +218,35 @@ main (int argc, char **argv)
     for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
 	if (is_selected (mp, msgnum)) {
 	    cp = getcpy (m_name (msgnum));
-	    if (m_file (cp, folders, foldp, preserve, !linkf))
+	    if (m_file (mp, cp, retainseqs ? msgnum : 0, folders, foldp,
+                        preserve, !linkf))
 		done (1);
 	    free (cp);
 	}
     }
 
     /*
-     * This is a hack.  If we are using an external rmmproc,
-     * then save the current folder to the context file,
-     * so the external rmmproc will remove files from the correct
-     * directory.  This should be moved to folder_delmsgs().
+     * Update this now, since folder_delmsgs() will save the context
+     * but doesn't have access to the folder name.
      */
-    if (rmmproc) {
-	context_replace (pfolder, folder);
-	context_save ();
-	fflush (stdout);
-    }
+
+    context_replace (pfolder, folder);	/* update current folder   */
+
+    /*
+     * Adjust "cur" if necessary
+     */
+
+    if (mp->hghsel != mp->curmsg
+	&& (mp->numsel != mp->nummsg || linkf))
+	seq_setcur (mp, mp->hghsel);
+
+    /*
+     * Close destination folders now; if we are using private sequences
+     * we need to have all of our calls to seq_save() complete before we
+     * call context_save().
+     */
+
+    clsfolds (folders, foldp);
 
     /* If -nolink, then "remove" messages from source folder.
      *
@@ -252,17 +255,11 @@ main (int argc, char **argv)
      */
     if (!linkf) {
 	folder_delmsgs (mp, unlink_msgs, 1);
+    } else {
+	seq_save (mp);	/* synchronize message sequences */
+	context_save ();			/* save the context file   */
     }
 
-    clsfolds (folders, foldp);
-
-    if (mp->hghsel != mp->curmsg
-	&& (mp->numsel != mp->nummsg || linkf))
-	seq_setcur (mp, mp->hghsel);
-    seq_save (mp);	/* synchronize message sequences */
-
-    context_replace (pfolder, folder);	/* update current folder   */
-    context_save ();			/* save the context file   */
     folder_free (mp);			/* free folder structure   */
     done (0);
     return 1;
@@ -289,7 +286,7 @@ opnfolds (struct st_fold *folders, int nfolders)
 
 	if (chdir (nmaildir) == NOTOK)
 	    adios (nmaildir, "unable to change directory to");
-	if (!(mp = folder_read (fp->f_name)))
+	if (!(mp = folder_read (fp->f_name, 1)))
 	    adios (NULL, "unable to read folder %s", fp->f_name);
 	mp->curmsg = 0;
 
@@ -328,36 +325,39 @@ clsfolds (struct st_fold *folders, int nfolders)
 static void
 remove_files (int filep, char **files)
 {
-    int i;
-    char **vec;
+    int i, vecp;
+    char **vec, *program;
 
     /* If rmmproc is defined, we use that */
     if (rmmproc) {
-	vec = files++;		/* vec[0] = filevec[0] */
-	files[filep] = NULL;	/* NULL terminate list */
+    	vec = argsplit(rmmproc, &program, &vecp);
+	files++;		/* Yes, we need to do this */
+	for (i = 0; i < filep; i++)
+		vec[vecp++] = files[i];
+	vec[vecp] = NULL;	/* NULL terminate list */
 
 	fflush (stdout);
-	vec[0] = r1bindex (rmmproc, '/');
-	execvp (rmmproc, vec);
+	execvp (program, vec);
 	adios (rmmproc, "unable to exec");
     }
 
     /* Else just unlink the files */
     files++;	/* advance past filevec[0] */
     for (i = 0; i < filep; i++) {
-	if (unlink (files[i]) == NOTOK)
+	if (m_unlink (files[i]) == NOTOK)
 	    admonish (files[i], "unable to unlink");
     }
 }
 
 
 /*
- * Link (or copy) the message into each of
- * the destination folders.
+ * Link (or copy) the message into each of the destination folders.
+ * If oldmsgnum is not 0, call copy_seqs().
  */
 
 static int
-m_file (char *msgfile, struct st_fold *folders, int nfolders, int preserve, int refile)
+m_file (struct msgs *mp, char *msgfile, int oldmsgnum,
+	struct st_fold *folders, int nfolders, int preserve, int refile)
 {
     int msgnum;
     struct st_fold *fp, *ep;
@@ -365,6 +365,31 @@ m_file (char *msgfile, struct st_fold *folders, int nfolders, int preserve, int
     for (fp = folders, ep = folders + nfolders; fp < ep; fp++) {
 	if ((msgnum = folder_addmsg (&fp->f_mp, msgfile, 1, 0, preserve, nfolders == 1 && refile, maildir)) == -1)
 	    return 1;
+	if (oldmsgnum) copy_seqs (mp, oldmsgnum, fp->f_mp, msgnum);
     }
     return 0;
 }
+
+
+/*
+ * Copy sequence information for a refiled message to its
+ * new folder.  Skip the cur sequence.
+ */
+static void
+copy_seqs (struct msgs *oldmp, int oldmsgnum, struct msgs *newmp, int newmsgnum)
+{
+    char **seq;
+    size_t seqnum;
+
+    for (seq = svector_strs (oldmp->msgattrs), seqnum = 0;
+	 *seq && seqnum < svector_size (oldmp->msgattrs);
+	 ++seq, ++seqnum) {
+	if (strcmp (current, *seq)) {
+	    assert ((int) seqnum == seq_getnum (oldmp, *seq));
+	    if (in_sequence (oldmp, seqnum, oldmsgnum)) {
+		seq_addmsg (newmp, *seq, newmsgnum,
+			    is_seq_private (oldmp, seqnum) ? 0 : 1, 0);
+	    }
+	}
+    }
+}
