@@ -2,8 +2,6 @@
 /*
  * repl.c -- reply to a message
  *
- * $Id: repl.c,v 1.9 2007/11/04 11:54:36 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -13,104 +11,84 @@
 #include <h/utils.h>
 
 
-static struct swit switches[] = {
-#define GROUPSW                0
-    { "group", 0 },
-#define NGROUPSW               1
-    { "nogroup", 0 },
-#define	ANNOSW                 2
-    { "annotate", 0 },
-#define	NANNOSW                3
-    { "noannotate", 0 },
-#define	CCSW                   4
-    { "cc all|to|cc|me", 0 },
-#define	NCCSW                  5
-    { "nocc type", 0 },
-#define	DFOLDSW                6
-    { "draftfolder +folder", 0 },
-#define	DMSGSW                 7
-    { "draftmessage msg", 0 },
-#define	NDFLDSW                8
-    { "nodraftfolder", 0 },
-#define	EDITRSW                9
-    { "editor editor", 0 },
-#define	NEDITSW               10
-    { "noedit", 0 },
-#define	FCCSW                 11
-    { "fcc folder", 0 },
-#define	FILTSW                12
-    { "filter filterfile", 0 },
-#define	FORMSW                13
-    { "form formfile", 0 },
-#define	FRMTSW                14
-    { "format", 5 },
-#define	NFRMTSW               15
-    { "noformat", 7 },
-#define	INPLSW                16
-    { "inplace", 0 },
-#define	NINPLSW               17
-    { "noinplace", 0 },
-#define MIMESW                18
-    { "mime", 0 },
-#define NMIMESW               19
-    { "nomime", 0 },
-#define	QURYSW                20
-    { "query", 0 },
-#define	NQURYSW               21
-    { "noquery", 0 },
-#define	WHATSW                22
-    { "whatnowproc program", 0 },
-#define	NWHATSW               23
-    { "nowhatnowproc", 0 },
-#define	WIDTHSW               24
-    { "width columns", 0 },
-#define VERSIONSW             25
-    { "version", 0 },
-#define	HELPSW                26
-    { "help", 0 },
-#define	FILESW                27
-    { "file file", 4 },			/* interface from msh */
+#define REPL_SWITCHES \
+    X("group", 0, GROUPSW) \
+    X("nogroup", 0, NGROUPSW) \
+    X("annotate", 0, ANNOSW) \
+    X("noannotate", 0, NANNOSW) \
+    X("cc all|to|cc|me", 0, CCSW) \
+    X("nocc type", 0, NCCSW) \
+    X("draftfolder +folder", 0, DFOLDSW) \
+    X("draftmessage msg", 0, DMSGSW) \
+    X("nodraftfolder", 0, NDFLDSW) \
+    X("editor editor", 0, EDITRSW) \
+    X("noedit", 0, NEDITSW) \
+    X("fcc folder", 0, FCCSW) \
+    X("filter filterfile", 0, FILTSW) \
+    X("form formfile", 0, FORMSW) \
+    X("format", 5, FRMTSW) \
+    X("noformat", 7, NFRMTSW) \
+    X("inplace", 0, INPLSW) \
+    X("noinplace", 0, NINPLSW) \
+    X("mime", 0, MIMESW) \
+    X("nomime", 0, NMIMESW) \
+    X("query", 0, QURYSW) \
+    X("noquery", 0, NQURYSW) \
+    X("whatnowproc program", 0, WHATSW) \
+    X("nowhatnowproc", 0, NWHATSW) \
+    X("width columns", 0, WIDTHSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+    X("file file", 4, FILESW) /* interface from msh */ \
+    X("build", 5, BILDSW) /* interface from mhe */ \
+    X("atfile", 0, ATFILESW) \
+    X("noatfile", 0, NOATFILESW) \
+    X("fmtproc program", 0, FMTPROCSW) \
+    X("nofmtproc", 0, NFMTPROCSW) \
 
-#ifdef MHE
-#define	BILDSW                28
-    { "build", 5 },			/* interface from mhe */
-#endif
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(REPL);
+#undef X
 
-    { NULL, 0 }
-};
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(REPL, switches);
+#undef X
 
-static struct swit ccswitches[] = {
-#define	CTOSW	0
-    { "to", 0 },
-#define	CCCSW	1
-    { "cc", 0 },
-#define	CMESW	2
-    { "me", 0 },
-#define	CALSW	3
-    { "all", 0 },
-    { NULL, 0 }
-};
+#define CC_SWITCHES \
+    X("to", 0, CTOSW) \
+    X("cc", 0, CCCSW) \
+    X("me", 0, CMESW) \
+    X("all", 0, CALSW) \
 
-static struct swit aqrnl[] = {
-#define	NOSW         0
-    { "quit", 0 },
-#define	YESW         1
-    { "replace", 0 },
-#define	LISTDSW      2
-    { "list", 0 },
-#define	REFILSW      3
-    { "refile +folder", 0 },
-#define NEWSW        4
-    { "new", 0 },
-    { NULL, 0 }
-};
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(CC);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(CC, ccswitches);
+#undef X
+
+#define DISPO_SWITCHES \
+    X("quit", 0, NOSW) \
+    X("replace", 0, YESW) \
+    X("list", 0, LISTDSW) \
+    X("refile +folder", 0, REFILSW) \
+    X("new", 0, NEWSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(DISPO);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(DISPO, aqrnl);
+#undef X
 
 static struct swit aqrl[] = {
-    { "quit", 0 },
-    { "replace", 0 },
-    { "list", 0 },
-    { "refile +folder", 0 },
-    { NULL, 0 }
+    { "quit", 0, NOSW },
+    { "replace", 0, YESW },
+    { "list", 0, LISTDSW },
+    { "refile +folder", 0, REFILSW },
+    { NULL, 0, 0 }
 };
 
 short ccto = -1;		/* global for replsbr */
@@ -118,13 +96,13 @@ short cccc = -1;
 short ccme = -1;
 short querysw = 0;
 
-short outputlinelen = OUTPUTLINELEN;
-short groupreply = 0;		/* Is this a group reply?        */
+static short outputlinelen = OUTPUTLINELEN;
+static short groupreply = 0;		/* Is this a group reply?        */
 
-int mime = 0;			/* include original as MIME part */
-char *form   = NULL;		/* form (components) file        */
-char *filter = NULL;		/* message filter file           */
-char *fcc    = NULL;		/* folders to add to Fcc: header */
+static int mime = 0;			/* include original as MIME part */
+static char *form   = NULL;		/* form (components) file        */
+static char *filter = NULL;		/* message filter file           */
+static char *fcc    = NULL;		/* folders to add to Fcc: header */
 
 
 /*
@@ -139,6 +117,8 @@ main (int argc, char **argv)
     int	i, isdf = 0;
     int anot = 0, inplace = 1;
     int nedit = 0, nwhat = 0;
+    int atfile = 0;
+    int fmtproc = -1;
     char *cp, *cwd, *dp, *maildir, *file = NULL;
     char *folder = NULL, *msg = NULL, *dfolder = NULL;
     char *dmsg = NULL, *ed = NULL, drft[BUFSIZ], buf[BUFSIZ];
@@ -147,17 +127,9 @@ main (int argc, char **argv)
     struct stat st;
     FILE *in;
 
-#ifdef MHE
     int buildsw = 0;
-#endif /* MHE */
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -178,7 +150,7 @@ main (int argc, char **argv)
 		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case GROUPSW:
 		    groupreply++;
@@ -219,10 +191,8 @@ main (int argc, char **argv)
 			adios (NULL, "missing argument to %s", argp[-2]);
 		    nwhat = 0;
 		    continue;
-#ifdef MHE
 		case BILDSW: 
 		    buildsw++;	/* fall... */
-#endif /* MHE */
 		case NWHATSW: 
 		    nwhat++;
 		    continue;
@@ -313,6 +283,22 @@ main (int argc, char **argv)
 		    dfolder = NULL;
 		    isdf = NOTOK;
 		    continue;
+
+		case ATFILESW:
+		    atfile++;
+		    continue;
+		case NOATFILESW:
+		    atfile = 0;
+		    continue;
+
+		case FMTPROCSW:
+		    if (!(formatproc = *argp++) || *formatproc == '-')
+			adios (NULL, "missing argument to %s", argp[-2]);
+		    fmtproc = 1;
+		    continue;
+		case NFMTPROCSW:
+		    fmtproc = 0;
+		    continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
@@ -344,18 +330,11 @@ main (int argc, char **argv)
 
 try_it_again:
 
-#ifdef MHE
     strncpy (drft, buildsw ? m_maildir ("reply")
 			  : m_draft (dfolder, NULL, NOUSE, &isdf), sizeof(drft));
 
     /* Check if a draft exists */
     if (!buildsw && stat (drft, &st) != NOTOK) {
-#else
-    strncpy (drft, m_draft (dfolder, dmsg, NOUSE, &isdf), sizeof(drft));
-
-    /* Check if a draft exists */
-    if (stat (drft, &st) != NOTOK) {
-#endif /* MHE */
 	printf ("Draft \"%s\" exists (%ld bytes).", drft, (long) st.st_size);
 	for (i = LISTDSW; i != YESW;) {
 	    if (!(argp = getans ("\nDisposition? ", isdf ? aqrnl : aqrl)))
@@ -401,7 +380,7 @@ try_it_again:
 	    adios (maildir, "unable to change directory to");
 
 	/* read folder and create message structure */
-	if (!(mp = folder_read (folder)))
+	if (!(mp = folder_read (folder, 1)))
 	    adios (NULL, "unable to read folder %s", folder);
 
 	/* check for empty folder */
@@ -435,13 +414,14 @@ try_it_again:
 	    form = etcpath (replcomps);
     }
 
-    replout (in, msg, drft, mp, outputlinelen, mime, form, filter, fcc);
+    replout (in, msg, drft, mp, outputlinelen, mime, form, filter,
+    	     fcc, fmtproc);
     fclose (in);
 
     if (nwhat)
 	done (0);
-    what_now (ed, nedit, NOUSE, drft, msg, 0, mp,
-	    anot ? "Replied" : NULL, inplace, cwd);
+    what_now (ed, nedit, NOUSE, drft, msg, 0, mp, anot ? "Replied" : NULL,
+	    inplace, cwd, atfile);
     done (1);
     return 1;
 }
