@@ -9,17 +9,19 @@
 
 #include <h/mh.h>
 
-static struct swit switches[] = {
-#define	INTRSW            0
-    { "interactive", 0 },
-#define	NINTRSW           1
-    { "nointeractive", 0 },
-#define VERSIONSW         2
-    { "version", 0 },
-#define	HELPSW            3
-    { "help", 0 },
-    { NULL, 0 }
-};
+#define RMF_SWITCHES \
+    X("interactive", 0, INTRSW) \
+    X("nointeractive", 0, NINTRSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(RMF);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(RMF, switches);
+#undef X
 
 /*
  * static prototypes
@@ -35,13 +37,7 @@ main (int argc, char **argv)
     char *cp, *folder = NULL, newfolder[BUFSIZ];
     char buf[BUFSIZ], **argp, **arguments;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -59,10 +55,10 @@ main (int argc, char **argv)
 		    snprintf (buf, sizeof(buf), "%s [+folder] [switches]",
 			invo_name);
 		    print_help (buf, switches, 1);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case INTRSW: 
 		    interactive = 1;
@@ -169,13 +165,6 @@ rmf (char *folder)
 		    continue;	/* else fall */
 
 	    case ',': 
-#ifdef MHE
-	    case '+': 
-#endif /* MHE */
-#ifdef UCI
-	    case '_': 
-	    case '#': 
-#endif /* UCI */
 		break;
 
 	    default: 
@@ -190,7 +179,7 @@ rmf (char *folder)
 		others++;
 		continue;
 	}
-	if (unlink (dp->d_name) == NOTOK) {
+	if (m_unlink (dp->d_name) == NOTOK) {
 	    admonish (dp->d_name, "unable to unlink %s:", folder);
 	    others++;
 	}
