@@ -2,8 +2,6 @@
 /*
  * rmm.c -- remove a message(s)
  *
- * $Id: rmm.c,v 1.10 2007/11/04 11:54:36 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -12,17 +10,21 @@
 #include <h/mh.h>
 #include <h/utils.h>
 
-static struct swit switches[] = {
-#define UNLINKSW      0
-    { "unlink", 0 },
-#define NUNLINKSW    1
-    { "nounlink", 0 },
-#define VERSIONSW     2
-    { "version", 0 },
-#define	HELPSW        3
-    { "help", 0 },
-    { NULL, 0 }
-};
+#define RMM_SWITCHES \
+    X("unlink", 0, UNLINKSW) \
+    X("nounlink", 0, NUNLINKSW) \
+    X("rmmproc program", 0, RPROCSW) \
+    X("normmproc", 0, NRPRCSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(RMM);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(RMM, switches);
+#undef X
 
 
 int
@@ -35,13 +37,7 @@ main (int argc, char **argv)
     struct msgs_array msgs = { 0, 0, NULL };
     struct msgs *mp;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -50,20 +46,20 @@ main (int argc, char **argv)
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
-	    case AMBIGSW: 
+	    case AMBIGSW:
 		ambigsw (cp, switches);
 		done (1);
-	    case UNKWNSW: 
+	    case UNKWNSW:
 		adios (NULL, "-%s unknown\n", cp);
 
-	    case HELPSW: 
+	    case HELPSW:
 		snprintf (buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
 			  invo_name);
 		print_help (buf, switches, 1);
-		done (1);
+		done (0);
 	    case VERSIONSW:
 		print_version(invo_name);
-		done (1);
+		done (0);
 
 	    case UNLINKSW:
 		unlink_msgs++;
@@ -71,6 +67,14 @@ main (int argc, char **argv)
 	    case NUNLINKSW:
 		unlink_msgs = 0;
 		continue;
+
+            case RPROCSW:
+                if (!(rmmproc = *argp++) || *rmmproc == '-')
+                    adios (NULL, "missing argument to %s", argp[-2]);
+                continue;
+            case NRPRCSW:
+                rmmproc = NULL;
+                continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
@@ -94,7 +98,7 @@ main (int argc, char **argv)
 	adios (maildir, "unable to change directory to");
 
     /* read folder and create message structure */
-    if (!(mp = folder_read (folder)))
+    if (!(mp = folder_read (folder, 1)))
 	adios (NULL, "unable to read folder %s", folder);
 
     /* check for empty folder */
@@ -108,24 +112,16 @@ main (int argc, char **argv)
     seq_setprev (mp);		/* set the previous-sequence      */
 
     /*
-     * This is hackish.  If we are using a external rmmproc,
-     * then we need to update the current folder in the
-     * context so the external rmmproc will remove files
-     * from the correct directory.  This should be moved to
-     * folder_delmsgs().
+     * As part of the new world locking order, folder_delmsgs() now updates
+     * the sequence and context for us.  But since folder_delmsgs() doesn't
+     * have access to the folder name, change the context now.
      */
-    if (rmmproc) {
-	context_replace (pfolder, folder);
-	context_save ();
-	fflush (stdout);
-    }
+
+    context_replace (pfolder, folder);
 
     /* "remove" the SELECTED messages */
     folder_delmsgs (mp, unlink_msgs, 0);
 
-    seq_save (mp);		/* synchronize message sequences  */
-    context_replace (pfolder, folder);	/* update current folder   */
-    context_save ();			/* save the context file   */
     folder_free (mp);			/* free folder structure   */
     done (0);
     return 1;
