@@ -9,8 +9,6 @@
 
 #include <h/mh.h>
 #include <fcntl.h>
-#include <errno.h>
-#include <signal.h>
 
 
 #ifndef CYRUS_SASL
@@ -25,107 +23,75 @@
 # define TLSminc(a)   0
 #endif /* TLS_SUPPORT */
 
-static struct swit switches[] = {
-#define	ALIASW                 0
-    { "alias aliasfile", 0 },
-#define	DEBUGSW                1
-    { "debug", -5 },
-#define	DRAFTSW                2
-    { "draft", 0 },
-#define	DFOLDSW                3
-    { "draftfolder +folder", 6 },
-#define	DMSGSW                 4
-    { "draftmessage msg", 6 },
-#define	NDFLDSW                5
-    { "nodraftfolder", 0 },
-#define	FILTSW                 6
-    { "filter filterfile", 0 },
-#define	NFILTSW                7
-    { "nofilter", 0 },
-#define	FRMTSW                 8
-    { "format", 0 },
-#define	NFRMTSW                9
-    { "noformat", 0 },
-#define	FORWSW                10
-    { "forward", 0 },
-#define	NFORWSW               11
-    { "noforward", 0 },
-#define MIMESW                12
-    { "mime", 0 },
-#define NMIMESW               13
-    { "nomime", 0 },
-#define	MSGDSW                14
-    { "msgid", 0 },
-#define	NMSGDSW               15
-    { "nomsgid", 0 },
-#define	PUSHSW                16
-    { "push", 0 },
-#define	NPUSHSW               17
-    { "nopush", 0 },
-#define	SPLITSW               18
-    { "split seconds", 0 },
-#define	UNIQSW                19
-    { "unique", -6 },
-#define	NUNIQSW               20
-    { "nounique", -8 },
-#define	VERBSW                21
-    { "verbose", 0 },
-#define	NVERBSW               22
-    { "noverbose", 0 },
-#define	WATCSW                23
-    { "watch", 0 },
-#define	NWATCSW               24
-    { "nowatch", 0 },
-#define	WIDTHSW               25
-    { "width columns", 0 },
-#define VERSIONSW             26
-    { "version", 0 },
-#define	HELPSW                27
-    { "help", 0 },
-#define BITSTUFFSW            28
-    { "dashstuffing", -12 },
-#define NBITSTUFFSW           29
-    { "nodashstuffing", -14 },
-#define	MAILSW                30
-    { "mail", -4 },
-#define	SAMLSW                31
-    { "saml", -4 },
-#define	SENDSW                32
-    { "send", -4 },
-#define	SOMLSW                33
-    { "soml", -4 },
-#define	CLIESW                34
-    { "client host", -6 },
-#define	SERVSW                35
-    { "server host", 6 },
-#define	SNOOPSW               36
-    { "snoop", 5 },
-#define SASLSW                37
-    { "sasl", SASLminc(4) },
-#define SASLMECHSW            38
-    { "saslmech mechanism", SASLminc(-5) },
-#define USERSW                39
-    { "user username", SASLminc(-4) },
-#define ATTACHSW              40
-    { "attach", 6 },
-#define ATTACHFORMATSW        41
-    { "attachformat", 7 },
-#define PORTSW		      42
-    { "port server-port-name/number" , 4 },
-#define TLSSW		      43
-    { "tls", TLSminc(-3) },
-    { NULL, 0 }
-};
+#define SEND_SWITCHES \
+    X("alias aliasfile", 0, ALIASW) \
+    X("debug", -5, DEBUGSW) \
+    X("draft", 0, DRAFTSW) \
+    X("draftfolder +folder", 6, DFOLDSW) \
+    X("draftmessage msg", 6, DMSGSW) \
+    X("nodraftfolder", 0, NDFLDSW) \
+    X("filter filterfile", 0, FILTSW) \
+    X("nofilter", 0, NFILTSW) \
+    X("format", 0, FRMTSW) \
+    X("noformat", 0, NFRMTSW) \
+    X("forward", 0, FORWSW) \
+    X("noforward", 0, NFORWSW) \
+    X("mime", 0, MIMESW) \
+    X("nomime", 0, NMIMESW) \
+    X("msgid", 0, MSGDSW) \
+    X("nomsgid", 0, NMSGDSW) \
+    X("push", 0, PUSHSW) \
+    X("nopush", 0, NPUSHSW) \
+    X("split seconds", 0, SPLITSW) \
+    X("unique", -6, UNIQSW) \
+    X("nounique", -8, NUNIQSW) \
+    X("verbose", 0, VERBSW) \
+    X("noverbose", 0, NVERBSW) \
+    X("watch", 0, WATCSW) \
+    X("nowatch", 0, NWATCSW) \
+    X("width columns", 0, WIDTHSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+    X("dashstuffing", -12, BITSTUFFSW) \
+    X("nodashstuffing", -14, NBITSTUFFSW) \
+    X("client host", -6, CLIESW) \
+    X("server host", 6, SERVSW) \
+    X("snoop", 5, SNOOPSW) \
+    X("sasl", SASLminc(4), SASLSW) \
+    X("nosasl", SASLminc(-6), NOSASLSW) \
+    X("saslmaxssf", SASLminc(-10), SASLMXSSFSW) \
+    X("saslmech mechanism", SASLminc(-5), SASLMECHSW) \
+    X("user username", SASLminc(-4), USERSW) \
+    X("attach", -6, ATTACHSW) \
+    X("noattach", -8, NOATTACHSW) \
+    X("attachformat", 7, ATTACHFORMATSW) \
+    X("port server-port-name/number", 4, PORTSW) \
+    X("tls", TLSminc(-3), TLSSW) \
+    X("initialtls", TLSminc(-10), INITTLSSW) \
+    X("notls", TLSminc(-5), NTLSSW) \
+    X("mts smtp|sendmail/smtp|sendmail/pipe", 2, MTSSW) \
+    X("messageid localname|random", 2, MESSAGEIDSW) \
 
-static struct swit anyl[] = {
-#define	NOSW     0
-    { "no", 0 },
-#define	YESW     1
-    { "yes", 0 },
-#define	LISTDSW  2
-    { "list", 0 },
-    { NULL, 0 }
-};
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(SEND);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(SEND, switches);
+#undef X
+
+#define USE_SWITCHES \
+    X("no", 0, NOSW) \
+    X("yes", 0, YESW) \
+    X("list", 0, LISTDSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(USE);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(USE, anyl);
+#undef X
 
 extern int debugsw;		/* from sendsbr.c */
 extern int forwsw;
@@ -143,34 +109,41 @@ extern char *distfile;
 int
 main (int argc, char **argv)
 {
-    int msgp = 0, distsw = 0, vecp = 1;
+    int msgp = 0, distsw = 0, vecp;
     int isdf = 0, mime = 0;
     int msgnum, status;
     char *cp, *dfolder = NULL, *maildir = NULL;
-    char buf[BUFSIZ], **ap, **argp, **arguments;
-    char *msgs[MAXARGS], *vec[MAXARGS];
+    char buf[BUFSIZ], **ap, **argp, **arguments, *program;
+    char *msgs[MAXARGS], **vec;
     struct msgs *mp;
     struct stat st;
-    char	*attach = (char *)0;	/* header field name for attachments */
-    int attachformat = 0; /* mhbuild format specifier for attachments */
-#ifdef UCI
-    FILE *fp;
-#endif /* UCI */
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
 
+    vec = argsplit(postproc, &program, &vecp);
+
     vec[vecp++] = "-library";
     vec[vecp++] = getcpy (m_maildir (""));
 
+    if ((cp = context_find ("fileproc"))) {
+	vec[vecp++] = "-fileproc";
+	vec[vecp++] = cp;
+    }
+
+    if ((cp = context_find ("mhlproc"))) {
+	vec[vecp++] = "-mhlproc";
+	vec[vecp++] = cp;
+    }
+
+    if ((cp = context_find ("credentials"))) {
+	/* post doesn't read context so need to pass credentials. */
+	vec[vecp++] = "-credentials";
+	vec[vecp++] = cp;
+    }
+
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -183,10 +156,10 @@ main (int argc, char **argv)
 		case HELPSW: 
 		    snprintf (buf, sizeof(buf), "%s [file] [switches]", invo_name);
 		    print_help (buf, switches, 1);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case DRAFTSW: 
 		    msgs[msgp++] = draft;
@@ -265,13 +238,12 @@ main (int argc, char **argv)
 		case NMSGDSW: 
 		case WATCSW: 
 		case NWATCSW: 
-		case MAILSW: 
-		case SAMLSW: 
-		case SENDSW: 
-		case SOMLSW: 
 		case SNOOPSW: 
 		case SASLSW:
+		case NOSASLSW:
 		case TLSSW:
+		case INITTLSSW:
+		case NTLSSW:
 		    vec[vecp++] = --cp;
 		    continue;
 
@@ -281,8 +253,11 @@ main (int argc, char **argv)
 		case CLIESW: 
 		case SERVSW: 
 		case SASLMECHSW:
+		case SASLMXSSFSW:
 		case USERSW:
 		case PORTSW:
+		case MTSSW:
+		case MESSAGEIDSW:
 		    vec[vecp++] = --cp;
 		    if (!(cp = *argp++) || *cp == '-')
 			adios (NULL, "missing argument to %s", argp[-2]);
@@ -290,23 +265,14 @@ main (int argc, char **argv)
 		    continue;
 		
 		case ATTACHSW:
-		    if (!(attach = *argp++) || *attach == '-')
-			adios (NULL, "missing argument to %s", argp[-2]);
+		    advise(NULL, "The -attach switch is deprecated");
+		    continue;
+		case NOATTACHSW:
+		    advise(NULL, "The -noattach switch is deprecated");
 		    continue;
 
 		case ATTACHFORMATSW:
-		    if (! *argp || **argp == '-')
-			adios (NULL, "missing argument to %s", argp[-1]);
-		    else {
-			attachformat = atoi (*argp);
-			if (attachformat < 0 ||
-			    attachformat > ATTACHFORMATS - 1) {
-			    advise (NULL, "unsupported attachformat %d",
-				    attachformat);
-			    continue;
-			}
-		    }
-		    ++argp;
+		    advise(NULL, "The -attachformat switch is deprecated");
 		    continue;
 	    }
 	} else {
@@ -328,12 +294,6 @@ main (int argc, char **argv)
 
     if (dfolder == NULL) {
 	if (msgp == 0) {
-#ifdef WHATNOW
-	    if ((cp = getenv ("mhdraft")) && *cp) {
-		msgs[msgp++] = cp;
-		goto go_to_it;
-	    }
-#endif /* WHATNOW */
 	    msgs[msgp++] = getcpy (m_draft (NULL, NULL, 1, &isdf));
 	    if (stat (msgs[0], &st) == NOTOK)
 		adios (msgs[0], "unable to stat draft file");
@@ -370,7 +330,7 @@ main (int argc, char **argv)
 	    adios (maildir, "unable to change directory to");
 
 	/* read folder and create message structure */
-	if (!(mp = folder_read (dfolder)))
+	if (!(mp = folder_read (dfolder, 1)))
 	    adios (NULL, "unable to read folder %s", dfolder);
 
 	/* check for empty folder */
@@ -401,18 +361,6 @@ go_to_it:
     if ((cp = getenv ("SIGNATURE")) == NULL || *cp == 0)
 	if ((cp = context_find ("signature")) && *cp)
 	    m_putenv ("SIGNATURE", cp);
-#ifdef UCI
-	else {
-	    snprintf (buf, sizeof(buf), "%s/.signature", mypath);
-	    if ((fp = fopen (buf, "r")) != NULL
-		&& fgets (buf, sizeof buf, fp) != NULL) {
-		    fclose (fp);
-		    if (cp = strchr (buf, '\n'))
-			*cp = 0;
-		    m_putenv ("SIGNATURE", buf);
-	    }
-	}
-#endif /* UCI */
 
     for (msgnum = 0; msgnum < msgp; msgnum++)
 	if (stat (msgs[msgnum], &st) == NOTOK)
@@ -430,16 +378,26 @@ go_to_it:
 	    && (distsw = atoi (cp))
 	    && altmsg) {
 	vec[vecp++] = "-dist";
-	distfile = getcpy (m_mktemp2 (altmsg, invo_name, NULL, NULL));
+	if ((cp = m_mktemp2(altmsg, invo_name, NULL, NULL)) == NULL) {
+	    adios(NULL, "unable to create temporary file in %s",
+		  get_temp_dir());
+	}
+	distfile = getcpy (cp);
+	(void) m_unlink(distfile);
 	if (link (altmsg, distfile) == NOTOK) {
-	    if (errno != EXDEV
+	    /* Cygwin with FAT32 filesystem produces EPERM. */
+	    if (errno != EXDEV  &&  errno != EPERM
 #ifdef EISREMOTE
 		    && errno != EISREMOTE
 #endif /* EISREMOTE */
 		)
 		adios (distfile, "unable to link %s to", altmsg);
 	    free (distfile);
-	    distfile = getcpy (m_mktemp2(NULL, invo_name, NULL, NULL));
+	    if ((cp = m_mktemp2(NULL, invo_name, NULL, NULL)) == NULL) {
+		adios(NULL, "unable to create temporary file in %s",
+		      get_temp_dir());
+	    }
+	    distfile = getcpy (cp);
 	    {
 		int in, out;
 		struct stat st;
@@ -467,12 +425,10 @@ go_to_it:
 	push ();
 
     status = 0;
-    vec[0] = r1bindex (postproc, '/');
     closefds (3);
 
     for (msgnum = 0; msgnum < msgp; msgnum++) {
-	switch (sendsbr (vec, vecp, msgs[msgnum], &st, 1, attach,
-			 attachformat)) {
+	switch (sendsbr (vec, vecp, program, msgs[msgnum], &st, 1)) {
 	    case DONE: 
 		done (++status);
 	    case NOTOK: 
