@@ -2,8 +2,6 @@
 /*
  * slocal.c -- asynchronously filter and deliver new mail
  *
- * $Id: slocal.c,v 1.27 2008/04/11 14:12:55 opk Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -27,24 +25,18 @@
 #include <h/dropsbr.h>
 #include <h/rcvmail.h>
 #include <h/signals.h>
+#include <setjmp.h>
 #include <h/tws.h>
 #include <h/mts.h>
 #include <h/utils.h>
 
 #include <pwd.h>
-#include <signal.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
 
-#ifdef INITGROUPS_HEADER
-#include INITGROUPS_HEADER
-#else
-/* On AIX 4.1, initgroups() is defined and even documented (giving the parameter
-   types as char* and int), but doesn't have a prototype in any of the system
-   header files.  AIX 4.3, SunOS 4.1.3, and ULTRIX 4.2A have the same
-   problem. */
-extern int  initgroups(char*, int);
-#endif
+/* Hopefully, grp.h declares initgroups().  If we run into a platform
+   where it doesn't, we could consider declaring it here as well. */
+#include <grp.h>
 
 /* This define is needed for Berkeley db v2 and above to
  * make the header file expose the 'historical' ndbm APIs.
@@ -52,55 +44,40 @@ extern int  initgroups(char*, int);
  * harmless.
  */
 #define DB_DBM_HSEARCH 1
+#ifdef DB_DBM_HSEARCH
+#endif /* Use DB_DBM_HSEARCH to prevent warning from gcc -Wunused-macros. */
 #ifdef NDBM_HEADER
 #include NDBM_HEADER
 #endif
 
-#include <utmp.h>
+#ifdef HAVE_GETUTXENT
+#include <utmpx.h>
+#endif /* HAVE_GETUTXENT */
 
-#ifndef HAVE_GETUTENT
-# ifndef UTMP_FILE
-#  ifdef _PATH_UTMP
-#   define UTMP_FILE _PATH_UTMP
-#  else
-#   define UTMP_FILE "/etc/utmp"
-#  endif
-# endif
-#endif
+#define SLOCAL_SWITCHES \
+    X("addr address", 0, ADDRSW) \
+    X("user name", 0, USERSW) \
+    X("file file", 0, FILESW) \
+    X("sender address", 0, SENDERSW) \
+    X("mailbox file", 0, MAILBOXSW) \
+    X("home directory", -4, HOMESW) \
+    X("info data", 0, INFOSW) \
+    X("maildelivery file", 0, MAILSW) \
+    X("verbose", 0, VERBSW) \
+    X("noverbose", 0, NVERBSW) \
+    X("suppressdup", 0, SUPPRESSDUP) \
+    X("nosuppressdup", 0, NSUPPRESSDUP) \
+    X("debug", 0, DEBUGSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
 
-static struct swit switches[] = {
-#define	ADDRSW         0
-    { "addr address", 0 },
-#define	USERSW         1
-    { "user name", 0 },
-#define	FILESW         2
-    { "file file", 0 },
-#define	SENDERSW       3
-    { "sender address", 0 },
-#define	MAILBOXSW      4
-    { "mailbox file", 0 },
-#define	HOMESW         5
-    { "home directory", -4 },
-#define	INFOSW         6
-    { "info data", 0 },
-#define	MAILSW         7
-    { "maildelivery file", 0 },
-#define	VERBSW         8
-    { "verbose", 0 },
-#define	NVERBSW        9
-    { "noverbose", 0 },
-#define SUPPRESSDUP   10
-    { "suppressdup", 0 },
-#define NSUPPRESSDUP 11
-    { "nosuppressdup", 0 },
-#define	DEBUGSW       12
-    { "debug", 0 },
-#define VERSIONSW     13
-    { "version", 0 },
-#define	HELPSW        14
-    { "help", 0 },
-    { NULL, 0 }
-};
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(SLOCAL);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(SLOCAL, switches);
+#undef X
 
 static int globbed = 0;		/* have we built "vars" table yet?        */
 static int parsed = 0;		/* have we built header field table yet   */
@@ -192,7 +169,7 @@ static int timely (char *, char *);
 static int usr_file (int, char *, int);
 static int usr_pipe (int, char *, char *, char **, int);
 static int usr_folder (int, char *);
-static RETSIGTYPE alrmser (int);
+static void alrmser (int);
 static void get_sender (char *, char **);
 static int copy_message (int, char *, int);
 static void verbose_printf (char *fmt, ...);
@@ -206,19 +183,12 @@ int
 main (int argc, char **argv)
 {
     int fd, status;
-    FILE *fp = stdin;
+    FILE *fp;
     char *cp, *mdlvr = NULL, buf[BUFSIZ];
     char mailbox[BUFSIZ], tmpfil[BUFSIZ];
     char **argp, **arguments;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (*argv, '/');
-
-    /* foil search of user profile/context */
-    if (context_foil (NULL) == -1)
-	done (1);
+    if (nmh_init(argv[0], 0 /* use context_foil() */)) { return 1; }
 
     mts_init (invo_name);
     arguments = getarguments (invo_name, argc, argv, 0);
@@ -235,13 +205,12 @@ main (int argc, char **argv)
 		    adios (NULL, "-%s unknown", cp);
 
 		case HELPSW: 
-		    snprintf (buf, sizeof(buf),
-			"%s [switches] [address info sender]", invo_name);
+		    snprintf (buf, sizeof(buf), "%s [switches]", invo_name);
 		    print_help (buf, switches, 0);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case ADDRSW: 
 		    if (!(addr = *argp++))/* allow -xyz arguments */
@@ -297,20 +266,8 @@ main (int argc, char **argv)
 		    debug++;
 		    continue;
 	    }
-	}
-
-	switch (argp - (argv + 1)) {
-	    case 1: 
-		addr = cp;
-		break;
-
-	    case 2: 
-		info = cp;
-		break;
-
-	    case 3: 
-		sender = cp;
-		break;
+	} else {
+		adios (NULL, "only switch arguments are supported");
 	}
     }
 
@@ -326,11 +283,15 @@ main (int argc, char **argv)
     umask (0077);
 
     if (geteuid() == 0) {
-	setgid (pw->pw_gid);
+	if (setgid (pw->pw_gid) != 0) {
+	    adios ("setgid", "unable to set group to %ld", (long) pw->pw_gid);
+	}
 	initgroups (pw->pw_name, pw->pw_gid);
-	setuid (pw->pw_uid);
+	if (setuid (pw->pw_uid) != 0) {
+	    adios ("setuid", "unable to set user to %ld", (long) pw->pw_uid);
+	}
     }
-    
+
     if (info == NULL)
 	info = "";
 
@@ -353,14 +314,16 @@ main (int argc, char **argv)
 	if (debug)
 	    debug_printf ("retrieving message from file \"%s\"\n", file);
 	if ((fd = copy_message (tempfd, tmpfil, 1)) == -1)
-	    adios (NULL, "unable to create temporary file");
+	    adios(NULL, "unable to create temporary file in %s",
+                  get_temp_dir());
 	close (tempfd);
     } else {
 	/* getting message from stdin */
 	if (debug)
 	    debug_printf ("retrieving message from stdin\n");
 	if ((fd = copy_message (fileno (stdin), tmpfil, 1)) == -1)
-	    adios (NULL, "unable to create temporary file");
+	    adios(NULL, "unable to create temporary file in %s",
+                  get_temp_dir());
     }
 
     if (debug)
@@ -377,7 +340,7 @@ main (int argc, char **argv)
        thing would be to delay this unlink() until later if debug == 1, but I'll
        leave that for someone who cares about the temp-file-accessing
        functionality (they'll have to watch out for cases where we adios()). */
-    unlink (tmpfil);
+    (void) m_unlink (tmpfil);
 
     if (!(fp = fdopen (fd, "r+")))
 	adios (NULL, "unable to access temporary file");
@@ -441,11 +404,7 @@ localmail (int fd, char *mdlvr)
 	verbose_printf ("(delivering to standard mail spool)\n");
 
     /* last resort - deliver to standard mail spool */
-#ifdef SLOCAL_MBOX
     return usr_file (fd, mbox, MBOX_FORMAT);
-#else
-    return usr_file (fd, mbox, MMDF_FORMAT);
-#endif
 }
 
 
@@ -555,7 +514,7 @@ usr_delivery (int fd, char *delivery, int su)
 	}
 
 	if (vecp > 5) {
-	    if (!mh_strcasecmp (vec[5], "select")) {
+	    if (!strcasecmp (vec[5], "select")) {
 		if (logged_in () != -1)
 		    continue;
 		if (vecp > 7 && timely (vec[6], vec[7]) == -1)
@@ -574,7 +533,7 @@ usr_delivery (int fd, char *delivery, int su)
 	     * "default" matches only if the message hasn't
 	     * been delivered yet.
 	     */
-		if (!mh_strcasecmp (field, "default")) {
+		if (!strcasecmp (field, "default")) {
 		    if (won)
 			continue;
 		    break;
@@ -604,7 +563,7 @@ usr_delivery (int fd, char *delivery, int su)
 	switch (*action) {
 	    case 'q':
 		/* deliver to quoted pipe */
-		if (mh_strcasecmp (action, "qpipe"))
+		if (strcasecmp (action, "qpipe"))
 		    continue;	/* else fall */
 	    case '^':
 		expand (tmpbuf, string, fd);
@@ -615,7 +574,7 @@ usr_delivery (int fd, char *delivery, int su)
 
 	    case 'p': 
 		/* deliver to pipe */
-		if (mh_strcasecmp (action, "pipe"))
+		if (strcasecmp (action, "pipe"))
 		    continue;	/* else fall */
 	    case '|': 
 		vec[2] = "sh";
@@ -628,12 +587,12 @@ usr_delivery (int fd, char *delivery, int su)
 
 	    case 'f': 
 		/* mbox format */
-		if (!mh_strcasecmp (action, "file")) {
+		if (!strcasecmp (action, "file")) {
 		    status = usr_file (fd, string, MBOX_FORMAT);
 		    break;
 		}
 		/* deliver to nmh folder */
-		else if (mh_strcasecmp (action, "folder"))
+		else if (strcasecmp (action, "folder"))
 		    continue;	/* else fall */
 	    case '+':
 		status = usr_folder (fd, string);
@@ -641,12 +600,12 @@ usr_delivery (int fd, char *delivery, int su)
 
 	    case 'm':
 		/* mmdf format */
-		if (!mh_strcasecmp (action, "mmdf")) {
+		if (!strcasecmp (action, "mmdf")) {
 		    status = usr_file (fd, string, MMDF_FORMAT);
 		    break;
 		}
 		/* mbox format */
-		else if (mh_strcasecmp (action, "mbox"))
+		else if (strcasecmp (action, "mbox"))
 		    continue;	/* else fall */
 
 	    case '>': 
@@ -656,7 +615,7 @@ usr_delivery (int fd, char *delivery, int su)
 
 	    case 'd': 
 		/* ignore message */
-		if (mh_strcasecmp (action, "destroy"))
+		if (strcasecmp (action, "destroy"))
 		    continue;
 		status = 0;
 		break;
@@ -684,7 +643,7 @@ static int
 split (char *cp, char **vec)
 {
     int i;
-    unsigned char *s;
+    char *s;
 
     s = cp;
 
@@ -693,7 +652,7 @@ split (char *cp, char **vec)
 	vec[i] = NULL;
 
 	/* zap any whitespace and comma's */
-	while (isspace (*s) || *s == ',')
+	while (isspace ((unsigned char) *s) || *s == ',')
 	    *s++ = 0;
 
 	/* end of buffer, time to leave */
@@ -723,7 +682,7 @@ split (char *cp, char **vec)
 	vec[i++] = s++;
 
 	/* move forward to next field delimiter */
-	while (*s && !isspace (*s) && *s != ',')
+	while (*s && !isspace ((unsigned char) *s) && *s != ',')
 	    s++;
     }
     vec[i] = NULL;
@@ -746,6 +705,7 @@ parse (int fd)
     char name[NAMESZ], field[BUFSIZ];
     struct pair *p, *q;
     FILE  *in;
+    m_getfld_state_t gstate = 0;
 
     if (parsed++)
 	return 0;
@@ -769,18 +729,19 @@ parse (int fd)
      * Scan the headers of the message and build
      * a lookup table.
      */
-    for (i = 0, state = FLD;;) {
-	switch (state = m_getfld (state, name, field, sizeof(field), in)) {
+    for (i = 0;;) {
+	int fieldsz = sizeof field;
+	switch (state = m_getfld (&gstate, name, field, &fieldsz, in)) {
 	    case FLD: 
-	    case FLDEOF: 
 	    case FLDPLUS: 
 		lp = add (field, NULL);
 		while (state == FLDPLUS) {
-		    state = m_getfld (state, name, field, sizeof(field), in);
+		    fieldsz = sizeof field;
+		    state = m_getfld (&gstate, name, field, &fieldsz, in);
 		    lp = add (field, lp);
 		}
 		for (p = hdrs; p->p_name; p++) {
-		    if (!mh_strcasecmp (p->p_name, name)) {
+		    if (!strcasecmp (p->p_name, name)) {
 			if (!(p->p_flags & P_HID)) {
 			    if ((cp = p->p_value)) {
 				if (p->p_flags & P_ADR) {
@@ -805,12 +766,9 @@ parse (int fd)
 		    p++, i++;
 		    p->p_name = NULL;
 		}
-		if (state != FLDEOF)
-		    continue;
-		break;
+		continue;
 
 	    case BODY: 
-	    case BODYEOF: 
 	    case FILEEOF: 
 		break;
 
@@ -826,6 +784,7 @@ parse (int fd)
 	}
 	break;
     }
+    m_getfld_state_destroy (&gstate);
     fclose (in);
 
     if ((p = lookup (vars, "reply-to"))) {
@@ -934,7 +893,7 @@ static struct pair *
 lookup (struct pair *pairs, char *key)
 {
     for (; pairs->p_name; pairs++)
-	if (!mh_strcasecmp (pairs->p_name, key))
+	if (!strcasecmp (pairs->p_name, key))
 	    return pairs;
 
     return NULL;
@@ -946,62 +905,31 @@ lookup (struct pair *pairs, char *key)
  * logged in.
  */
 
-#ifdef HAVE_GETUTENT
 static int
 logged_in (void)
 {
-    struct utmp * utp;
+#if HAVE_GETUTXENT
+    struct utmpx *utp;
 
     if (utmped)
         return utmped;
 
-    setutent();
+    setutxent();
 
-    while ((utp = getutent()) != NULL) {
-        if (
-#ifdef HAVE_STRUCT_UTMP_UT_TYPE
-		utp->ut_type == USER_PROCESS
-                &&
-#endif
-		utp->ut_name[0] != 0
-                && strncmp (user, utp->ut_name, sizeof(utp->ut_name)) == 0) {
+    while ((utp = getutxent()) != NULL) {
+        if ( utp->ut_type == USER_PROCESS && utp->ut_user[0] != 0
+                && strncmp (user, utp->ut_user, sizeof(utp->ut_user)) == 0) {
             if (debug)
                 continue;
-            endutent();
+            endutxent();
             return (utmped = DONE);
         }
     }
 
-    endutent();
+    endutxent();
+#endif /* HAVE_GETUTXENT */
     return (utmped = NOTOK);
 }
-#else
-static int
-logged_in (void)
-{
-    struct utmp ut;
-    FILE *uf;
-
-    if (utmped)
-	return utmped;
-
-    if ((uf = fopen (UTMP_FILE, "r")) == NULL)
-	return NOTOK;
-
-    while (fread ((char *) &ut, sizeof(ut), 1, uf) == 1) {
-	if (ut.ut_name[0] != 0
-	    && strncmp (user, ut.ut_name, sizeof(ut.ut_name)) == 0) {
-	    if (debug)
-		continue;
-	    fclose (uf);
-	    return (utmped = DONE);
-	}
-    }
-
-    fclose (uf);
-    return (utmped = NOTOK);
-}
-#endif
 
 #define	check(t,a,b)		if (t < a || t > b) return -1
 #define	cmpar(h1,m1,h2,m2)	if (h1 < h2 || (h1 == h2 && m1 < m2)) return 0
@@ -1067,7 +995,11 @@ usr_file (int fd, char *mailbox, int mbx_style)
     }
 
     /* close and unlock file */
-    mbx_close (mailbox, md);
+    if (mbx_close (mailbox, md) == NOTOK) {
+        if (verbose)
+	    adorn ("", "error closing:");
+        return -1;
+    }
 
     if (verbose)
 	verbose_printf (", success.\n");
@@ -1101,18 +1033,6 @@ usr_folder (int fd, char *string)
     /* use rcvstore to put message in folder */
     status = usr_pipe (fd, "rcvstore", rcvstoreproc, vec, 1);
 
-#if 0
-    /*
-     * Currently, verbose status messages are handled by usr_pipe().
-     */
-    if (verbose) {
-	if (status == 0)
-	    verbose_printf (", success.\n");
-	else
-	    verbose_printf (", failed.\n");
-    }
-#endif
-
     return status;
 }
 
@@ -1121,8 +1041,9 @@ usr_folder (int fd, char *string)
  */
 
 static int
-usr_pipe (int fd, char *cmd, char *pgm, char **vec, int suppress)
+usr_pipe (int fd_arg, char *cmd, char *pgm, char **vec, int suppress)
 {
+    volatile int fd = fd_arg;
     pid_t child_id;
     int i, bytes, seconds, status;
     struct stat st;
@@ -1171,7 +1092,7 @@ usr_pipe (int fd, char *cmd, char *pgm, char **vec, int suppress)
 
 	default: 
 	    /* parent process */
-	    if (!setjmp (myctx)) {
+	    if (! setjmp (myctx)) {
 		SIGNAL (SIGALRM, alrmser);
 		bytes = fstat (fd, &st) != -1 ? (int) st.st_size : 100;
 
@@ -1204,7 +1125,7 @@ usr_pipe (int fd, char *cmd, char *pgm, char **vec, int suppress)
 		 * Ruthlessly kill the child and anything
 		 * else in its process group.
 		 */
-		KILLPG(child_id, SIGKILL);
+		killpg(child_id, SIGKILL);
 		if (verbose)
 		    verbose_printf (", timed-out; terminated\n");
 		return -1;
@@ -1213,12 +1134,10 @@ usr_pipe (int fd, char *cmd, char *pgm, char **vec, int suppress)
 }
 
 
-static RETSIGTYPE
+static void
 alrmser (int i)
 {
-#ifndef RELIABLE_SIGNALS
-    SIGNAL (SIGALRM, alrmser);
-#endif
+    NMH_UNUSED (i);
 
     longjmp (myctx, DONE);
 }
@@ -1233,8 +1152,8 @@ static void
 get_sender (char *envelope, char **sender)
 {
     int i;
-    unsigned char *cp;
-    unsigned char buffer[BUFSIZ];
+    char *cp;
+    char buffer[BUFSIZ];
 
     if (envelope == NULL) {
 	*sender = getcpy ("");
@@ -1254,7 +1173,7 @@ get_sender (char *envelope, char **sender)
     *cp = 0;
 
     for (cp = buffer + strlen (buffer) - 1; cp >= buffer; cp--)
-	if (isspace (*cp))
+	if (isspace ((unsigned char) *cp))
 	    *cp = 0;
 	else
 	    break;
@@ -1274,19 +1193,18 @@ copy_message (int qd, char *tmpfil, int fold)
     int i, first = 1, fd1, fd2;
     char buffer[BUFSIZ];
     FILE *qfp, *ffp;
+    char *tfile = NULL;
 
-    strcpy (tmpfil, m_tmpfil (invo_name));
-
-    /* open temporary file to put message in */
-    if ((fd1 = open (tmpfil, O_RDWR | O_CREAT | O_TRUNC, 0600)) == -1)
-	return -1;
+    tfile = m_mktemp2(NULL, invo_name, &fd1, NULL);
+    if (tfile == NULL) return -1;
+    strncpy (tmpfil, tfile, BUFSIZ);
 
     if (!fold) {
 	while ((i = read (qd, buffer, sizeof(buffer))) > 0)
 	    if (write (fd1, buffer, i) != i) {
 you_lose:
 		close (fd1);
-		unlink (tmpfil);
+		(void) m_unlink (tmpfil);
 		return -1;
 	    }
 	if (i == -1)
@@ -1333,46 +1251,9 @@ you_lose:
 	if (first) {
 	    first = 0;
 	    if (!strncmp (buffer, "From ", i)) {
-#ifdef RPATHS
-		char *fp, *cp, *hp, *ep;
-#endif
 		/* get copy of envelope information ("From " line) */
 		envelope = getcpy (buffer);
 
-#if 0
-		/* First go ahead and put "From " line in message */
-		fputs (buffer, ffp);
-		if (ferror (ffp))
-		    goto fputs_error;
-#endif
-
-#ifdef RPATHS
-		/*
-		 * Now create a "Return-Path:" line
-		 * from the "From " line.
-		 */
-		hp = cp = strchr(fp = envelope + i, ' ');
-		while ((hp = strchr(++hp, 'r')))
-		    if (uprf (hp, "remote from")) {
-			hp = strrchr(hp, ' ');
-			break;
-		    }
-		if (hp) {
-		    /* return path for UUCP style addressing */
-		    ep = strchr(++hp, '\n');
-		    snprintf (buffer, sizeof(buffer), "Return-Path: %.*s!%.*s\n",
-			(int)(ep - hp), hp, (int)(cp - fp), fp);
-		} else {
-		    /* return path for standard domain addressing */
-		    snprintf (buffer, sizeof(buffer), "Return-Path: %.*s\n",
-			(int)(cp - fp), fp);
-		}
-
-		/* Add Return-Path header to message */
-		fputs (buffer, ffp);
-		if (ferror (ffp))
-		    goto fputs_error;
-#endif
 		/* Put the delivery date in message */
 		fputs (ddate, ffp);
 		if (ferror (ffp))
@@ -1413,7 +1294,7 @@ static char *
 trim (char *cp)
 {
     char buffer[BUFSIZ*4];
-    unsigned char *bp, *sp;
+    char *bp, *sp;
 
     if (cp == NULL)
 	return NULL;
@@ -1423,12 +1304,12 @@ trim (char *cp)
     bp = buffer;
 
     /* skip over leading whitespace */
-    while (isspace(*bp))
+    while (isspace((unsigned char) *bp))
 	bp++;
 
     /* start at the end and zap trailing whitespace */
     for (sp = bp + strlen(bp) - 1; sp >= bp; sp--) {
-	if (isspace(*sp))
+	if (isspace((unsigned char) *sp))
 	    *sp = 0;
 	else
 	    break;
@@ -1436,7 +1317,7 @@ trim (char *cp)
 
     /* replace remaining whitespace with spaces */
     for (sp = bp; *sp; sp++)
-	if (isspace(*sp))
+	if (isspace((unsigned char) *sp))
 	    *sp = ' ';
 
     /* now return a copy */
@@ -1522,6 +1403,7 @@ suppress_duplicates (int fd, char *file)
     datum key, value;
     DBM *db;
     FILE *in;
+    m_getfld_state_t gstate = 0;
 
     if ((fd1 = dup (fd)) == -1)
 	return -1;
@@ -1531,22 +1413,26 @@ suppress_duplicates (int fd, char *file)
     }
     rewind (in);
 
-    for (state = FLD;;) {
-	state = m_getfld (state, name, buf, sizeof(buf), in);
+    for (;;) {
+        int failed_to_lock = 0;
+	int bufsz = sizeof buf;
+	state = m_getfld (&gstate, name, buf, &bufsz, in);
 	switch (state) {
 	    case FLD:
 	    case FLDPLUS:
-	    case FLDEOF:
 		/* Search for the message ID */
-		if (mh_strcasecmp (name, "Message-ID")) {
-		    while (state == FLDPLUS)
-			state = m_getfld (state, name, buf, sizeof(buf), in);
+		if (strcasecmp (name, "Message-ID")) {
+		    while (state == FLDPLUS) {
+			bufsz = sizeof buf;
+			state = m_getfld (&gstate, name, buf, &bufsz, in);
+		    }
 		    continue;
 		}
 
 		cp = add (buf, NULL);
 		while (state == FLDPLUS) {
-		    state = m_getfld (state, name, buf, sizeof(buf), in);
+		    bufsz = sizeof buf;
+		    state = m_getfld (&gstate, name, buf, &bufsz, in);
 		    cp = add (buf, cp);
 		}
 		key.dptr = trimcpy (cp);
@@ -1566,7 +1452,8 @@ suppress_duplicates (int fd, char *file)
 		 * This will fail if your Maildelivery file doesn't
 		 * exist.
 		 */
-		if ((lockfd = lkopen(file, O_RDWR, 0)) == -1) {
+		if ((lockfd = lkopendata(file, O_RDWR, 0, &failed_to_lock))
+		    == -1) {
 		    advise (file, "unable to perform file locking on");
 		    free (cp);
 		    fclose (in);
@@ -1587,14 +1474,12 @@ suppress_duplicates (int fd, char *file)
 		}
 
 		dbm_close (db);
-		lkclose(lockfd, file);
+		lkclosedata(lockfd, file);
 		free (cp);
 		fclose (in);
 		return result;
-		break;
 
 	   case BODY:
-	   case BODYEOF:
 	   case FILEEOF:
 		break;
 
@@ -1606,6 +1491,7 @@ suppress_duplicates (int fd, char *file)
 
 	break;
     }
+    m_getfld_state_destroy (&gstate);
 
     fclose (in);
     return 0;
