@@ -2,8 +2,6 @@
 /*
  * sortm.c -- sort messages in a folder by date/time
  *
- * $Id: sortm.c,v 1.15 2007/11/04 11:54:36 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -13,29 +11,29 @@
 #include <h/tws.h>
 #include <h/utils.h>
 
-static struct swit switches[] = {
-#define DATESW                 0
-     { "datefield field", 0 },
-#define	TEXTSW                 1
-     { "textfield field", 0 },
-#define	NSUBJSW                2
-     { "notextfield", 0 },
-#define SUBJSW                 3
-     { "subject", -3 },		   /* backward-compatibility */
-#define LIMSW                  4
-     { "limit days", 0 },
-#define	NLIMSW                 5
-     { "nolimit", 0 },
-#define VERBSW                 6
-     { "verbose", 0 },
-#define NVERBSW                7
-     { "noverbose", 0 },
-#define VERSIONSW              8
-     { "version", 0 },
-#define HELPSW                 9
-     { "help", 0 },
-     { NULL, 0 }
-};
+#define SORTM_SWITCHES \
+    X("datefield field", 0, DATESW) \
+    X("textfield field", 0, TEXTSW) \
+    X("notextfield", 0, NSUBJSW) \
+    X("subject", -3, SUBJSW) /* backward-compatibility */ \
+    X("limit days", 0, LIMSW) \
+    X("nolimit", 0, NLIMSW) \
+    X("verbose", 0, VERBSW) \
+    X("noverbose", 0, NVERBSW) \
+    X("all", 0, ALLMSGS) \
+    X("noall", 0, NALLMSGS) \
+    X("check", 0, CHECKSW) \
+    X("nocheck", 0, NCHECKSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(SORTM);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(SORTM, switches);
+#undef X
 
 struct smsg {
     int s_msg;
@@ -47,9 +45,11 @@ static struct smsg *smsgs;
 int nmsgs;
 
 char *subjsort = (char *) 0;    /* sort on subject if != 0 */
-unsigned long datelimit = 0;
+time_t datelimit = 0;
 int submajor = 0;		/* if true, sort on subject-major */
 int verbose;
+int allmsgs = 1;
+int check_failed = 0;
 
 /* This keeps compiler happy on calls to qsort */
 typedef int (*qsort_comp) (const void *, const void *);
@@ -70,21 +70,15 @@ int
 main (int argc, char **argv)
 {
     int	i, msgnum;
-    unsigned char *cp;
-    char *maildir, *datesw = NULL;
+    char *cp, *maildir, *datesw = NULL;
     char *folder = NULL, buf[BUFSIZ], **argp;
     char **arguments;
     struct msgs_array msgs = { 0, 0, NULL };
     struct msgs *mp;
     struct smsg **dlist;
+    int checksw = 0;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -105,10 +99,10 @@ main (int argc, char **argv)
 		snprintf(buf, sizeof(buf), "%s [+folder] [msgs] [switches]",
 			invo_name);
 		print_help (buf, switches, 1);
-		done (1);
+		done (0);
 	    case VERSIONSW:
 		print_version(invo_name);
-		done (1);
+		done (0);
 
 	    case DATESW:
 		if (datesw)
@@ -140,7 +134,7 @@ main (int argc, char **argv)
 		    submajor++;		/* sort subject-major */
 		    continue;
 		}
-		if (!isdigit(*cp) || !(datelimit = atoi(cp)))
+		if (!isdigit((unsigned char) *cp) || !(datelimit = atoi(cp)))
 		    adios (NULL, "impossible limit %s", cp);
 		datelimit *= 60*60*24;
 		continue;
@@ -155,6 +149,20 @@ main (int argc, char **argv)
 	    case NVERBSW:
 		verbose = 0;
 		continue;
+
+	    case ALLMSGS:
+		allmsgs = 1;
+		continue;
+	    case NALLMSGS:
+		allmsgs = 0;
+		continue;
+
+	    case CHECKSW:
+		checksw = 1;
+		continue;
+	    case NCHECKSW:
+		checksw = 0;
+		continue;
 	    }
 	}
 	if (*cp == '+' || *cp == '@') {
@@ -168,8 +176,13 @@ main (int argc, char **argv)
 
     if (!context_find ("path"))
 	free (path ("./", TFOLDER));
-    if (!msgs.size)
-	app_msgarg(&msgs, "all");
+    if (!msgs.size) {
+	if (allmsgs) {
+	    app_msgarg(&msgs, "all");
+        } else {
+	    adios (NULL, "must specify messages to sort with -noall");
+        }
+    }
     if (!datesw)
 	datesw = "date";
     if (!folder)
@@ -180,7 +193,7 @@ main (int argc, char **argv)
 	adios (maildir, "unable to change directory to");
 
     /* read folder and create message structure */
-    if (!(mp = folder_read (folder)))
+    if (!(mp = folder_read (folder, 1)))
 	adios (NULL, "unable to read folder %s", folder);
 
     /* check for empty folder */
@@ -196,6 +209,10 @@ main (int argc, char **argv)
     if ((nmsgs = read_hdrs (mp, datesw)) <= 0)
 	adios (NULL, "no messages to sort");
 
+    if (checksw  &&  check_failed) {
+	adios (NULL, "errors found, no messages sorted");
+    }
+
     /*
      * sort a list of pointers to our "messages to be sorted".
      */
@@ -206,9 +223,10 @@ main (int argc, char **argv)
 
     if (verbose) {	/* announce what we're doing */
 	if (subjsort)
-	    printf ("sorting by %s-major %s-minor\n",
-		submajor ? subjsort : datesw,
-		submajor ? datesw : subjsort);
+	    if (submajor)
+		printf ("sorting by %s\n", subjsort);
+	    else
+		printf ("sorting by %s-major %s-minor\n", subjsort, datesw);
 	else
 	    printf ("sorting by datefield %s\n", datesw);
     }
@@ -269,6 +287,7 @@ main (int argc, char **argv)
 	    }
 	}
 	*fp = 0;
+	free (il);
 	free (slist);
 	free (dlist);
 	dlist = flist;
@@ -332,50 +351,56 @@ get_fields (char *datesw, int msg, struct smsg *smsg)
     register struct tws *tw;
     register char *datecomp = NULL, *subjcomp = NULL;
     register FILE *in;
+    m_getfld_state_t gstate = 0;
 
     if ((in = fopen (msgnam = m_name (msg), "r")) == NULL) {
 	admonish (msgnam, "unable to read message");
 	return (0);
     }
-    for (compnum = 1, state = FLD;;) {
-	switch (state = m_getfld (state, nam, buf, sizeof(buf), in)) {
+    for (compnum = 1;;) {
+	int bufsz = sizeof buf;
+	switch (state = m_getfld (&gstate, nam, buf, &bufsz, in)) {
 	case FLD:
-	case FLDEOF:
 	case FLDPLUS:
 	    compnum++;
-	    if (!mh_strcasecmp (nam, datesw)) {
+	    if (!strcasecmp (nam, datesw)) {
 		datecomp = add (buf, datecomp);
 		while (state == FLDPLUS) {
-		    state = m_getfld (state, nam, buf, sizeof(buf), in);
+		    bufsz = sizeof buf;
+		    state = m_getfld (&gstate, nam, buf, &bufsz, in);
 		    datecomp = add (buf, datecomp);
 		}
 		if (!subjsort || subjcomp)
 		    break;
-	    } else if (subjsort && !mh_strcasecmp (nam, subjsort)) {
+	    } else if (subjsort && !strcasecmp (nam, subjsort)) {
 		subjcomp = add (buf, subjcomp);
 		while (state == FLDPLUS) {
-		    state = m_getfld (state, nam, buf, sizeof(buf), in);
+		    bufsz = sizeof buf;
+		    state = m_getfld (&gstate, nam, buf, &bufsz, in);
 		    subjcomp = add (buf, subjcomp);
 		}
 		if (datecomp)
 		    break;
 	    } else {
 		/* just flush this guy */
-		while (state == FLDPLUS)
-		    state = m_getfld (state, nam, buf, sizeof(buf), in);
+		while (state == FLDPLUS) {
+		    bufsz = sizeof buf;
+		    state = m_getfld (&gstate, nam, buf, &bufsz, in);
+		}
 	    }
 	    continue;
 
 	case BODY:
-	case BODYEOF:
 	case FILEEOF:
 	    break;
 
 	case LENERR:
 	case FMTERR:
-	    if (state == LENERR || state == FMTERR)
+	    if (state == LENERR || state == FMTERR) {
 		admonish (NULL, "format error in message %d (header #%d)",
 		      msg, compnum);
+		check_failed = 1;
+	    }
 	    if (datecomp)
 		free (datecomp);
 	    if (subjcomp)
@@ -388,6 +413,7 @@ get_fields (char *datesw, int msg, struct smsg *smsg)
 	}
 	break;
     }
+    m_getfld_state_destroy (&gstate);
 
     /*
      * If no date component, then use the modification
@@ -396,9 +422,13 @@ get_fields (char *datesw, int msg, struct smsg *smsg)
     if (!datecomp || (tw = dparsetime (datecomp)) == NULL) {
 	struct stat st;
 
-	admonish (NULL, "can't parse %s field in message %d", datesw, msg);
+	advise (NULL,
+		"can't parse %s field in message %d, "
+		"will use file modification time",
+		datesw, msg);
 	fstat (fileno (in), &st);
 	smsg->s_clock = st.st_mtime;
+	check_failed = 1;
     } else {
 	smsg->s_clock = dmktime (tw);
     }
@@ -410,14 +440,13 @@ get_fields (char *datesw, int msg, struct smsg *smsg)
 	     * leading "re:", everything but letters & smash
 	     * letters to lower case.
 	     */
-	    register char  *cp, *cp2;
-	    register unsigned char c;
+	    register char  *cp, *cp2, c;
 
 	    cp = subjcomp;
 	    cp2 = subjcomp;
 	    if (strcmp (subjsort, "subject") == 0) {
 		while ((c = *cp)) {
-		    if (! isspace(c)) {
+		    if (! isspace((unsigned char) c)) {
 			if(uprf(cp, "re:"))
 			    cp += 2;
 			else
@@ -428,8 +457,9 @@ get_fields (char *datesw, int msg, struct smsg *smsg)
 	    }
 
 	    while ((c = *cp++)) {
-		if (isalnum(c))
-		    *cp2++ = isupper(c) ? tolower(c) : c;
+		if (isascii((unsigned char) c) && isalnum((unsigned char) c))
+		    *cp2++ = isupper((unsigned char) c) ?
+		    			tolower((unsigned char) c) : c;
 	    }
 
 	    *cp2 = '\0';
@@ -494,7 +524,7 @@ rename_chain (struct msgs *mp, struct smsg **mlist, int msg, int endmsg)
 {
     int nxt, old, new;
     char *newname, oldname[BUFSIZ];
-    char newbuf[MAXPATHLEN + 1];
+    char newbuf[PATH_MAX + 1];
 
     for (;;) {
 	nxt = mlist[msg] - smsgs;	/* mlist[msg] is a ptr into smsgs */
@@ -530,9 +560,9 @@ static void
 rename_msgs (struct msgs *mp, struct smsg **mlist)
 {
     int i, j, old, new;
-    seqset_t tmpset;
+    bvector_t tmpset = bvector_create (0);
     char f1[BUFSIZ], tmpfil[BUFSIZ];
-    char newbuf[MAXPATHLEN + 1];
+    char newbuf[PATH_MAX + 1];
     struct smsg *sp;
 
     strncpy (tmpfil, m_name (mp->hghmsg + 1), sizeof(tmpfil));
@@ -570,7 +600,7 @@ rename_msgs (struct msgs *mp, struct smsg **mlist)
 	if (rename (f1, tmpfil) == NOTOK)
 	    adios (tmpfil, "unable to rename %s to ", f1);
 
-	get_msg_flags (mp, &tmpset, old);
+	get_msg_flags (mp, tmpset, old);
 
 	rename_chain (mp, mlist, j, i);
 
@@ -585,7 +615,9 @@ rename_msgs (struct msgs *mp, struct smsg **mlist)
 	if (rename (tmpfil, m_name(new)) == NOTOK)
 	    adios (m_name(new), "unable to rename %s to", tmpfil);
 
-	set_msg_flags (mp, &tmpset, new);
+	set_msg_flags (mp, tmpset, new);
 	mp->msgflags |= SEQMOD;
     }
+
+    bvector_free (tmpset);
 }
