@@ -2,8 +2,6 @@
 /*
  * viamail.c -- send multiple files in a MIME message
  *
- * $Id: viamail.c,v 1.11 2007/11/04 11:54:36 jjr Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -13,57 +11,37 @@
 #include <fcntl.h>
 #include <h/signals.h>
 #include <h/md5.h>
-#include <errno.h>
-#include <signal.h>
 #include <h/mts.h>
 #include <h/tws.h>
 #include <h/mime.h>
 #include <h/mhparse.h>
 
-#ifdef HAVE_SYS_WAIT_H
-# include <sys/wait.h>
-#endif
+#define VIAMAIL_SWITCHES \
+    X("to mailpath", 0, TOSW) \
+    X("from mailpath", 0, FROMSW) \
+    X("subject subject", 0, SUBJECTSW) \
+    X("parameters arguments", 0, PARAMSW) \
+    X("description text", 0, DESCRIPTSW) \
+    X("comment text", 0, COMMENTSW) \
+    X("delay seconds", 0, DELAYSW) \
+    X("verbose", 0, VERBSW) \
+    X("noverbose", 0, NVERBSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+    X("debug", -5, DEBUGSW) \
 
-static struct swit switches[] = {
-#define	TOSW                    0
-    { "to mailpath", 0 },
-#define	FROMSW                  1
-    { "from mailpath", 0 },
-#define	SUBJECTSW               2
-    { "subject subject", 0 },
-#define	PARAMSW                 3
-    { "parameters arguments", 0 },
-#define	DESCRIPTSW              4
-    { "description text", 0 },
-#define	COMMENTSW               5
-    { "comment text", 0 },
-#define	DELAYSW                 6
-    { "delay seconds", 0 },
-#define	VERBSW                  7
-    { "verbose", 0 },
-#define	NVERBSW                 8
-    { "noverbose", 0 },
-#define VERSIONSW               9
-    { "version", 0 },
-#define	HELPSW                 10
-    { "help", 0 },
-#define DEBUGSW                11
-    { "debug", -5 },
-    { NULL, 0 }
-};
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(VIAMAIL);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(VIAMAIL, switches);
+#undef X
 
 extern int debugsw;
 extern int splitsw;
 extern int verbsw;
 
-int ebcdicsw = 0;	/* hack for linking purposes */
-
-/* mhmisc.c */
-void set_endian (void);
-
-/* mhoutsbr.c */
-int writeBase64aux (FILE *, FILE *);
-
 /*
  * static prototypes
  */
@@ -79,14 +57,7 @@ main (int argc, char **argv)
     char *cp, buf[BUFSIZ];
     char **argp, **arguments;
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* foil search of user profile/context */
-    if (context_foil (NULL) == -1)
-	done (1);
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 0);
     argp = arguments;
@@ -103,10 +74,10 @@ main (int argc, char **argv)
 	    case HELPSW: 
 		snprintf (buf, sizeof(buf), "%s [switches]", invo_name);
 		print_help (buf, switches, 1);
-		done (1);
+		done (0);
 	    case VERSIONSW:
 		print_version(invo_name);
-		done (1);
+		done (0);
     
 	    case TOSW:
 		if (!(f1 = *argp++))
@@ -158,8 +129,6 @@ main (int argc, char **argv)
 	}
     }
 
-    set_endian ();
-
     if (!f1)
 	adios (NULL, "missing -viamail \"mailpath\" switch");
 
@@ -176,21 +145,23 @@ static int
 via_mail (char *mailsw, char *subjsw, char *parmsw, char *descsw,
           char *cmntsw, int delay, char *fromsw)
 {
-    int	status, vecp = 1;
-    char tmpfil[BUFSIZ];
-    char *vec[MAXARGS];
+    int	status, vecp;
+    char tmpfil[BUFSIZ], *program;
+    char **vec;
     struct stat st;
     FILE *fp;
+    char *tfile = NULL;
+    char *cp;
 
     umask (~m_gmprot ());
 
-    strncpy (tmpfil, m_tmpfil (invo_name), sizeof(tmpfil));
-    if ((fp = fopen (tmpfil, "w+")) == NULL)
-	adios (tmpfil, "unable to open for writing");
-    chmod (tmpfil, 0600);
+    if ((tfile = m_mktemp2(NULL, invo_name, NULL, &fp)) == NULL) {
+	adios(NULL, "unable to create temporary file in %s", get_temp_dir());
+    }
+    strncpy (tmpfil, tfile, sizeof(tmpfil));
 
     if (!strchr(mailsw, '@'))
-	mailsw = concat (mailsw, "@", LocalName (), NULL);
+	mailsw = concat (mailsw, "@", LocalName (0), NULL);
     fprintf (fp, "To: %s\n", mailsw);
 
     if (subjsw)
@@ -198,7 +169,7 @@ via_mail (char *mailsw, char *subjsw, char *parmsw, char *descsw,
 
     if (fromsw) {
 	if (!strchr(fromsw, '@'))
-	    fromsw = concat (fromsw, "@", LocalName (), NULL);
+	    fromsw = concat (fromsw, "@", LocalName (0), NULL);
 	fprintf (fp, "From: %s\n", fromsw);
     }
 
@@ -219,7 +190,7 @@ via_mail (char *mailsw, char *subjsw, char *parmsw, char *descsw,
     if (fflush (fp))
 	adios (tmpfil, "error writing to");
 
-    writeBase64aux (stdin, fp);
+    writeBase64aux (stdin, fp, 0);
     if (fflush (fp))
 	adios (tmpfil, "error writing to");
 
@@ -232,11 +203,18 @@ via_mail (char *mailsw, char *subjsw, char *parmsw, char *descsw,
 	splitsw = delay;
 
     status = 0;
-    vec[0] = r1bindex (postproc, '/');
+
+    vec = argsplit(postproc, &program, &vecp);
     if (verbsw)
 	vec[vecp++] = "-verbose";
 
-    switch (sendsbr (vec, vecp, tmpfil, &st, 0, (char *)0, 0)) {
+    if ((cp = context_find ("credentials"))) {
+	/* post doesn't read context so need to pass credentials. */
+	vec[vecp++] = "-credentials";
+	vec[vecp++] = cp;
+    }
+
+    switch (sendsbr (vec, vecp, program, tmpfil, &st, 0)) {
 	case DONE:
 	case NOTOK:
 	    status++;
@@ -246,8 +224,8 @@ via_mail (char *mailsw, char *subjsw, char *parmsw, char *descsw,
     }
 
     fclose (fp);
-    if (unlink (tmpfil) == -1)
-	advise (NULL, "unable to remove temp file %s", tmpfil);
+    if (m_unlink (tmpfil) == -1)
+	advise (tmpfil, "unable to remove temp file %s", tmpfil);
     done (status);
     return 1;
 }
