@@ -2,8 +2,6 @@
 /*
  * vmhsbr.c -- routines to help vmh along
  *
- * $Id: vmhsbr.c,v 1.7 2006/01/02 03:17:42 bress Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -17,7 +15,6 @@
 #include <h/mh.h>
 #include <h/vmhsbr.h>
 #include <h/utils.h>
-#include <errno.h>
 
 static char *types[] = {
     "OK",
@@ -157,11 +154,13 @@ str2peer (char code, char *str)
 int
 fmt2peer (char code, char *fmt, ...)
 {
+    int     return_value;
     va_list ap;
 
     va_start(ap, fmt);
-    return verr2peer (code, NULL, fmt, ap);
+    return_value = verr2peer (code, NULL, fmt, ap);
     va_end(ap);
+    return return_value;
 }
 
 
