@@ -2,8 +2,6 @@
 /*
  * whatnowproc.c -- exec the "whatnowproc"
  *
- * $Id: whatnowproc.c,v 1.3 2002/07/02 22:09:15 kenh Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -21,13 +19,10 @@
  * it will call this routine directly without exec'ing it.
  */
 
-/* from whatnowsbr.c */
-int WhatNow (int, char **);
-
 
 int
 what_now (char *ed, int nedit, int use, char *file, char *altmsg, int dist,
-          struct msgs *mp, char *text, int inplace, char *cwd)
+          struct msgs *mp, char *text, int inplace, char *cwd, int atfile)
 {
     int found, k, msgnum, vecp;
     int len, buflen;
@@ -60,11 +55,12 @@ what_now (char *ed, int nedit, int use, char *file, char *altmsg, int dist,
     if (nedit) {
 	unputenv ("mheditor");
     } else {
-	m_putenv ("mheditor", ed ? ed : (ed = context_find ("editor"))
-	    ? ed : defaulteditor);
+	m_putenv ("mheditor", ed ? ed : (ed = get_default_editor()));
     }
     snprintf (buffer, sizeof(buffer), "%d", use);
     m_putenv ("mhuse", buffer);
+    snprintf (buffer, sizeof(buffer), "%d", atfile);
+    m_putenv ("mhatfile", buffer);
 
     unputenv ("mhmessages");
     unputenv ("mhannotate");
