@@ -2,8 +2,6 @@
 /*
  * whom.c -- report to whom a message would be sent
  *
- * $Id: whom.c,v 1.4 2002/07/02 22:09:15 kenh Exp $
- *
  * This code is Copyright (c) 2002, by the authors of nmh.  See the
  * COPYRIGHT file in the root directory of the nmh distribution for
  * complete copyright information.
@@ -11,54 +9,61 @@
 
 #include <h/mh.h>
 #include <h/signals.h>
-#include <signal.h>
 
-static struct swit switches[] = {
-#define	ALIASW              0
-    { "alias aliasfile", 0 },
-#define	CHKSW               1
-    { "check", 0 },
-#define	NOCHKSW             2
-    { "nocheck", 0 },
-#define	DRAFTSW             3
-    { "draft", 0 },
-#define	DFOLDSW             4
-    { "draftfolder +folder", 6 },
-#define	DMSGSW              5
-    { "draftmessage msg", 6 },
-#define	NDFLDSW             6
-    { "nodraftfolder", 0 },
-#define VERSIONSW           7
-    { "version", 0 },
-#define	HELPSW              8
-    { "help", 0 },
-#define	CLIESW              9
-    { "client host", -6 },
-#define	SERVSW             10
-    { "server host", -6 },
-#define	SNOOPSW            11
-    { "snoop", -5 },
-    { NULL, 0 }
-};
+#ifndef CYRUS_SASL
+# define SASLminc(a) (a)
+#else /* CYRUS_SASL */
+# define SASLminc(a)  0
+#endif /* CYRUS_SASL */
+
+#ifndef TLS_SUPPORT
+# define TLSminc(a)  (a)
+#else /* TLS_SUPPORT */
+# define TLSminc(a)   0
+#endif /* TLS_SUPPORT */
+
+#define WHOM_SWITCHES \
+    X("alias aliasfile", 0, ALIASW) \
+    X("check", 0, CHKSW) \
+    X("nocheck", 0, NOCHKSW) \
+    X("draft", 0, DRAFTSW) \
+    X("draftfolder +folder", 6, DFOLDSW) \
+    X("draftmessage msg", 6, DMSGSW) \
+    X("nodraftfolder", 0, NDFLDSW) \
+    X("version", 0, VERSIONSW) \
+    X("help", 0, HELPSW) \
+    X("client host", -6, CLIESW) \
+    X("server host", -6, SERVSW) \
+    X("snoop", -5, SNOOPSW) \
+    X("sasl", SASLminc(4), SASLSW) \
+    X("saslmech mechanism", SASLminc(-5), SASLMECHSW) \
+    X("user username", SASLminc(-4), USERSW) \
+    X("port server port name/number", 4, PORTSW) \
+    X("tls", TLSminc(-3), TLSSW) \
+    X("initialtls", TLSminc(-10), INITTLSSW) \
+    X("notls", TLSminc(-5), NTLSSW) \
+    X("mts smtp|sendmail/smtp|sendmail/pipe", 0, MTSSW) \
+
+#define X(sw, minchars, id) id,
+DEFINE_SWITCH_ENUM(WHOM);
+#undef X
+
+#define X(sw, minchars, id) { sw, minchars, id },
+DEFINE_SWITCH_ARRAY(WHOM, switches);
+#undef X
 
 
 int
 main (int argc, char **argv)
 {
-    pid_t child_id;
+    pid_t child_id = OK;
     int i, status, isdf = 0;
     int distsw = 0, vecp = 0;
     char *cp, *dfolder = NULL, *dmsg = NULL;
     char *msg = NULL, **ap, **argp, backup[BUFSIZ];
     char buf[BUFSIZ], **arguments, *vec[MAXARGS];
 
-#ifdef LOCALE
-    setlocale(LC_ALL, "");
-#endif
-    invo_name = r1bindex (argv[0], '/');
-
-    /* read user profile/context */
-    context_read();
+    if (nmh_init(argv[0], 1)) { return 1; }
 
     arguments = getarguments (invo_name, argc, argv, 1);
     argp = arguments;
@@ -68,6 +73,9 @@ main (int argc, char **argv)
     vec[vecp++] = "-library";
     vec[vecp++] = getcpy (m_maildir (""));
 
+    /* Don't need to feed fileproc or mhlproc to post because
+       it doesn't use them when used for whom. */
+
     while ((cp = *argp++)) {
 	if (*cp == '-') {
 	    switch (smatch (++cp, switches)) {
@@ -80,14 +88,18 @@ main (int argc, char **argv)
 		case HELPSW: 
 		    snprintf (buf, sizeof(buf), "%s [switches] [file]", invo_name);
 		    print_help (buf, switches, 1);
-		    done (1);
+		    done (0);
 		case VERSIONSW:
 		    print_version(invo_name);
-		    done (1);
+		    done (0);
 
 		case CHKSW: 
 		case NOCHKSW: 
 		case SNOOPSW:
+		case SASLSW:
+		case TLSSW:
+		case INITTLSSW:
+		case NTLSSW:
 		    vec[vecp++] = --cp;
 		    continue;
 
@@ -117,6 +129,10 @@ main (int argc, char **argv)
 		case ALIASW: 
 		case CLIESW: 
 		case SERVSW: 
+		case USERSW:
+		case PORTSW:
+		case SASLMECHSW:
+		case MTSSW:
 		    vec[vecp++] = --cp;
 		    if (!(cp = *argp++) || *cp == '-')
 			adios (NULL, "missing argument to %s", argp[-2]);
@@ -141,9 +157,6 @@ main (int argc, char **argv)
     }
 
     if (msg == NULL) {
-#ifdef	WHATNOW
-	if (dfolder || (cp = getenv ("mhdraft")) == NULL || *cp == '\0')
-#endif	/* WHATNOW */
 	    cp  = getcpy (m_draft (dfolder, dmsg, 1, &isdf));
 	msg = vec[vecp++] = cp;
     }
@@ -183,7 +196,7 @@ main (int argc, char **argv)
 
 	    status = pidwait(child_id, OK);
 
-	    unlink (msg);
+	    (void) m_unlink (msg);
 	    if (rename (backup, msg) == NOTOK)
 		adios (msg, "unable to rename %s to", backup);
 	    done (status);
